//
// Copyright 2003-2007 Sun Microsystems, Inc.  All Rights Reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
// CA 95054 USA or visit www.sun.com if you need additional information or
// have any questions.
//  
//

// Machine Generated File.  Do Not Edit!


#include "incls/_precompiled.incl"
#include "incls/_ad_x86_64.cpp.incl"


//SourceForm

#define   RELOC_IMM64    Assembler::imm64_operand
#define   RELOC_DISP32   Assembler::disp32_operand

#define __ _masm.

// !!!!! Special hack to get all types of calls to specify the byte offset
//       from the start of the call to the point where the return address
//       will point.
int MachCallStaticJavaNode::ret_addr_offset()
{
  return 5; // 5 bytes from start of call to where return address points
}

int MachCallDynamicJavaNode::ret_addr_offset()
{
  return 15; // 15 bytes from start of call to where return address points
}

// In os_cpu .ad file
// int MachCallRuntimeNode::ret_addr_offset()

// Indicate if the safepoint node needs the polling page as an input.
// Since amd64 does not have absolute addressing but RIP-relative
// addressing and the polling page is within 2G, it doesn't.
bool SafePointNode::needs_polling_address_input()
{
  return false;
}

//
// Compute padding required for nodes which need alignment
//

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallStaticJavaDirectNode::compute_padding(int current_offset) const
{
  current_offset += 1; // skip call opcode byte
  return round_to(current_offset, alignment_required()) - current_offset;
}

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallDynamicJavaDirectNode::compute_padding(int current_offset) const
{
  current_offset += 11; // skip movq instruction + call opcode byte
  return round_to(current_offset, alignment_required()) - current_offset;
}

#ifndef PRODUCT
void MachBreakpointNode::format(PhaseRegAlloc*) const
{
  tty->print("INT3");
}
#endif

// EMIT_RM()
void emit_rm(CodeBuffer &cbuf, int f1, int f2, int f3)
{
  unsigned char c = (unsigned char) ((f1 << 6) | (f2 << 3) | f3);
  *(cbuf.code_end()) = c;
  cbuf.set_code_end(cbuf.code_end() + 1);
}

// EMIT_CC()
void emit_cc(CodeBuffer &cbuf, int f1, int f2)
{
  unsigned char c = (unsigned char) (f1 | f2);
  *(cbuf.code_end()) = c;
  cbuf.set_code_end(cbuf.code_end() + 1);
}

// EMIT_OPCODE()
void emit_opcode(CodeBuffer &cbuf, int code)
{
  *(cbuf.code_end()) = (unsigned char) code;
  cbuf.set_code_end(cbuf.code_end() + 1);
}

// EMIT_OPCODE() w/ relocation information
void emit_opcode(CodeBuffer &cbuf, 
                 int code, relocInfo::relocType reloc, int offset, int format)
{
  cbuf.relocate(cbuf.inst_mark() + offset, reloc, format);
  emit_opcode(cbuf, code);    
}

// EMIT_D8()
void emit_d8(CodeBuffer &cbuf, int d8)
{
  *(cbuf.code_end()) = (unsigned char) d8;
  cbuf.set_code_end(cbuf.code_end() + 1);
}

// EMIT_D16()
void emit_d16(CodeBuffer &cbuf, int d16)
{
  *((short *)(cbuf.code_end())) = d16;
  cbuf.set_code_end(cbuf.code_end() + 2);
}

// EMIT_D32()
void emit_d32(CodeBuffer &cbuf, int d32)
{
  *((int *)(cbuf.code_end())) = d32;
  cbuf.set_code_end(cbuf.code_end() + 4);
}

// EMIT_D64()
void emit_d64(CodeBuffer &cbuf, int64_t d64)
{
  *((int64_t*) (cbuf.code_end())) = d64;
  cbuf.set_code_end(cbuf.code_end() + 8);
}

// emit 32 bit value and construct relocation entry from relocInfo::relocType
void emit_d32_reloc(CodeBuffer& cbuf, 
                    int d32, 
                    relocInfo::relocType reloc,
                    int format)
{
  assert(reloc != relocInfo::external_word_type, "use 2-arg emit_d32_reloc");
  cbuf.relocate(cbuf.inst_mark(), reloc, format);

  *((int*) (cbuf.code_end())) = d32;
  cbuf.set_code_end(cbuf.code_end() + 4);
}

// emit 32 bit value and construct relocation entry from RelocationHolder
void emit_d32_reloc(CodeBuffer& cbuf, 
                    int d32, 
                    RelocationHolder const& rspec,
                    int format)
{
#ifdef ASSERT
  if (rspec.reloc()->type() == relocInfo::oop_type && 
      d32 != 0 && d32 != (intptr_t) Universe::non_oop_word()) {
    assert(oop((intptr_t)d32)->is_oop() && oop((intptr_t)d32)->is_perm(), "cannot embed non-perm oops in code");
  }
#endif
  cbuf.relocate(cbuf.inst_mark(), rspec, format);

  *((int* )(cbuf.code_end())) = d32;
  cbuf.set_code_end(cbuf.code_end() + 4);
}

void emit_d32_reloc(CodeBuffer& cbuf, address addr) {
  address next_ip = cbuf.code_end() + 4;
  emit_d32_reloc(cbuf, (int) (addr - next_ip),
                 external_word_Relocation::spec(addr),
                 RELOC_DISP32);
}


// emit 64 bit value and construct relocation entry from relocInfo::relocType
void emit_d64_reloc(CodeBuffer& cbuf, 
                    int64_t d64,
                    relocInfo::relocType reloc,
                    int format)
{
  cbuf.relocate(cbuf.inst_mark(), reloc, format);

  *((int64_t*) (cbuf.code_end())) = d64;
  cbuf.set_code_end(cbuf.code_end() + 8);
}

// emit 64 bit value and construct relocation entry from RelocationHolder
void emit_d64_reloc(CodeBuffer& cbuf, 
                    int64_t d64,
                    RelocationHolder const& rspec,
                    int format)
{
#ifdef ASSERT
  if (rspec.reloc()->type() == relocInfo::oop_type && 
      d64 != 0 && d64 != (int64_t) Universe::non_oop_word()) {
    assert(oop(d64)->is_oop() && oop(d64)->is_perm(),
           "cannot embed non-perm oops in code");
  }
#endif
  cbuf.relocate(cbuf.inst_mark(), rspec, format);

  *((int64_t*) (cbuf.code_end())) = d64;
  cbuf.set_code_end(cbuf.code_end() + 8);
}

// Access stack slot for load or store
void store_to_stackslot(CodeBuffer &cbuf, int opcode, int rm_field, int disp)
{
  emit_opcode(cbuf, opcode);                  // (e.g., FILD   [RSP+src])
  if (-0x80 <= disp && disp < 0x80) {
    emit_rm(cbuf, 0x01, rm_field, RSP_enc);   // R/M byte
    emit_rm(cbuf, 0x00, RSP_enc, RSP_enc);    // SIB byte
    emit_d8(cbuf, disp);     // Displacement  // R/M byte
  } else {
    emit_rm(cbuf, 0x02, rm_field, RSP_enc);   // R/M byte
    emit_rm(cbuf, 0x00, RSP_enc, RSP_enc);    // SIB byte
    emit_d32(cbuf, disp);     // Displacement // R/M byte
  }
}

   // rRegI ereg, memory mem) %{    // emit_reg_mem
void encode_RegMem(CodeBuffer &cbuf, 
                   int reg, 
                   int base, int index, int scale, int disp, bool disp_is_oop) 
{
  assert(!disp_is_oop, "cannot have disp");
  int regenc = reg & 7;
  int baseenc = base & 7;
  int indexenc = index & 7;

  // There is no index & no scale, use form without SIB byte
  if (index == 0x4 && scale == 0 && base != RSP_enc && base != R12_enc) {
    // If no displacement, mode is 0x0; unless base is [RBP] or [R13]
    if (disp == 0 && base != RBP_enc && base != R13_enc) { 
      emit_rm(cbuf, 0x0, regenc, baseenc); // *
    } else if (-0x80 <= disp && disp < 0x80 && !disp_is_oop) {
      // If 8-bit displacement, mode 0x1
      emit_rm(cbuf, 0x1, regenc, baseenc); // *
      emit_d8(cbuf, disp);
    } else { 
      // If 32-bit displacement
      if (base == -1) { // Special flag for absolute address
        emit_rm(cbuf, 0x0, regenc, 0x5); // *
        if (disp_is_oop) {
          emit_d32_reloc(cbuf, disp, relocInfo::oop_type, RELOC_DISP32);
        } else {
          emit_d32(cbuf, disp);
        }
      } else {
        // Normal base + offset
        emit_rm(cbuf, 0x2, regenc, baseenc); // *
        if (disp_is_oop) {
          emit_d32_reloc(cbuf, disp, relocInfo::oop_type, RELOC_DISP32);
        } else {
          emit_d32(cbuf, disp);
        }
      }
    }
  } else { 
    // Else, encode with the SIB byte
    // If no displacement, mode is 0x0; unless base is [RBP] or [R13]
    if (disp == 0 && base != RBP_enc && base != R13_enc) {
      // If no displacement
      emit_rm(cbuf, 0x0, regenc, 0x4); // *
      emit_rm(cbuf, scale, indexenc, baseenc);
    } else {
      if (-0x80 <= disp && disp < 0x80 && !disp_is_oop) {
        // If 8-bit displacement, mode 0x1
        emit_rm(cbuf, 0x1, regenc, 0x4); // *
        emit_rm(cbuf, scale, indexenc, baseenc);
        emit_d8(cbuf, disp);
      } else {
        // If 32-bit displacement
        if (base == 0x04 ) {
          emit_rm(cbuf, 0x2, regenc, 0x4);
          emit_rm(cbuf, scale, indexenc, 0x04); // XXX is this valid???
        } else {
          emit_rm(cbuf, 0x2, regenc, 0x4);
          emit_rm(cbuf, scale, indexenc, baseenc); // *
        }
        if (disp_is_oop) {
          emit_d32_reloc(cbuf, disp, relocInfo::oop_type, RELOC_DISP32);
        } else {
          emit_d32(cbuf, disp);
        }
      }
    }
  }
}

void encode_copy(CodeBuffer &cbuf, int dstenc, int srcenc) 
{
  if (dstenc != srcenc) {
    if (dstenc < 8) {
      if (srcenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
        srcenc -= 8;
      }
    } else {
      if (srcenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
        srcenc -= 8;
      }
      dstenc -= 8;
    }
    
    emit_opcode(cbuf, 0x8B);
    emit_rm(cbuf, 0x3, dstenc, srcenc);
  }
}

void encode_CopyXD( CodeBuffer &cbuf, int dst_encoding, int src_encoding ) {
  if( dst_encoding == src_encoding ) {
    // reg-reg copy, use an empty encoding
  } else {
    MacroAssembler _masm(&cbuf);

    __ movdqa(as_XMMRegister(dst_encoding), as_XMMRegister(src_encoding));
  }
}


//=============================================================================
#ifndef PRODUCT
void MachPrologNode::format(PhaseRegAlloc* ra_) const
{
  Compile* C = ra_->C;

  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove wordSize for return adr already pushed
  // and another for the RBP we are going to save
  framesize -= 2*wordSize;
  bool need_nop = true;

  // Calls to C2R adapters often do not accept exceptional returns.
  // We require that their callers must bang for them.  But be
  // careful, because some VM calls (such as call site linkage) can
  // use several kilobytes of stack.  But the stack safety zone should
  // account for that.  See bugs 4446381, 4468289, 4497237.
  if (C->need_stack_bang(framesize)) {
    tty->print_cr("# stack bang"); tty->print("\t"); 
    need_nop = false;
  }
  tty->print_cr("pushq   rbp"); tty->print("\t");

  if (VerifyStackAtCalls) {
    // Majik cookie to verify stack depth
    tty->print_cr("pushq   0xffffffffbadb100d"
                  "\t# Majik cookie for stack depth check");
    tty->print("\t"); 
    framesize -= wordSize; // Remove 2 for cookie
    need_nop = false;
  }

  if (framesize) {
    tty->print("subq    rsp, #%d\t# Create frame", framesize);
    if (framesize < 0x80 && need_nop) {
      tty->print("\n\tnop\t# nop for patch_verified_entry");
    }
  }
}
#endif

void MachPrologNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const
{
  Compile* C = ra_->C;

  // WARNING: Initial instruction MUST be 5 bytes or longer so that
  // NativeJump::patch_verified_entry will be able to patch out the entry
  // code safely. The fldcw is ok at 6 bytes, the push to verify stack
  // depth is ok at 5 bytes, the frame allocation can be either 3 or
  // 6 bytes. So if we don't do the fldcw or the push then we must
  // use the 6 byte frame allocation even if we have no frame. :-(
  // If method sets FPU control word do it now

  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove wordSize for return adr already pushed
  // and another for the RBP we are going to save
  framesize -= 2*wordSize;
  bool need_nop = true;

  // Calls to C2R adapters often do not accept exceptional returns.
  // We require that their callers must bang for them.  But be
  // careful, because some VM calls (such as call site linkage) can
  // use several kilobytes of stack.  But the stack safety zone should
  // account for that.  See bugs 4446381, 4468289, 4497237.
  if (C->need_stack_bang(framesize)) {
    MacroAssembler masm(&cbuf);
    masm.generate_stack_overflow_check(framesize);
    need_nop = false;
  }

  // We always push rbp so that on return to interpreter rbp will be
  // restored correctly and we can correct the stack.
  emit_opcode(cbuf, 0x50 | RBP_enc);

  if (VerifyStackAtCalls) { 
    // Majik cookie to verify stack depth
    emit_opcode(cbuf, 0x68); // pushq (sign-extended) 0xbadb100d
    emit_d32(cbuf, 0xbadb100d);
    framesize -= wordSize; // Remove 2 for cookie
    need_nop = false;
  }

  if (framesize) {
    emit_opcode(cbuf, Assembler::REX_W);
    if (framesize < 0x80) {
      emit_opcode(cbuf, 0x83);   // sub  SP,#framesize
      emit_rm(cbuf, 0x3, 0x05, RSP_enc);
      emit_d8(cbuf, framesize);
      if (need_nop) { 
        emit_opcode(cbuf, 0x90); // nop
      }
    } else {
      emit_opcode(cbuf, 0x81);   // sub  SP,#framesize
      emit_rm(cbuf, 0x3, 0x05, RSP_enc); 
      emit_d32(cbuf, framesize);
    }
  }

  C->set_frame_complete(cbuf.code_end() - cbuf.code_begin());

#ifdef ASSERT 
  if (VerifyStackAtCalls) { 
    Label L;
    MacroAssembler masm(&cbuf);
    masm.pushq(rax);
    masm.movq(rax, rsp);
    masm.andq(rax, StackAlignmentInBytes-1);
    masm.cmpq(rax, StackAlignmentInBytes-wordSize);
    masm.popq(rax);
    masm.jcc(Assembler::equal, L);
    masm.stop("Stack is not properly aligned!");
    masm.bind(L);
  }
#endif
}

uint MachPrologNode::size(PhaseRegAlloc* ra_) const
{
  return MachNode::size(ra_); // too many variables; just compute it
                              // the hard way
}

int MachPrologNode::reloc() const
{
  return 0; // a large enough number
}

//=============================================================================
#ifndef PRODUCT
void MachEpilogNode::format(PhaseRegAlloc* ra_) const
{
  Compile* C = ra_->C;
  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove word for return adr already pushed
  // and RBP
  framesize -= 2*wordSize;

  if (framesize) {
    tty->print_cr("addq\trsp, %d\t# Destroy frame", framesize);
    tty->print("\t");
  }

  tty->print_cr("popq\trbp");
  if (do_polling() && C->is_method_compilation()) {
    tty->print_cr("\ttestl\trax, [rip + #offset_to_poll_page]\t"
                  "# Safepoint: poll for GC");
    tty->print("\t");
  }
}
#endif

void MachEpilogNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const
{
  Compile* C = ra_->C;
  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove word for return adr already pushed
  // and RBP
  framesize -= 2*wordSize;

  // Note that VerifyStackAtCalls' Majik cookie does not change the frame size popped here

  if (framesize) {
    emit_opcode(cbuf, Assembler::REX_W);
    if (framesize < 0x80) {
      emit_opcode(cbuf, 0x83); // addq rsp, #framesize
      emit_rm(cbuf, 0x3, 0x00, RSP_enc);
      emit_d8(cbuf, framesize);
    } else {
      emit_opcode(cbuf, 0x81); // addq rsp, #framesize
      emit_rm(cbuf, 0x3, 0x00, RSP_enc);
      emit_d32(cbuf, framesize);
    }
  }

  // popq rbp
  emit_opcode(cbuf, 0x58 | RBP_enc);

  if (do_polling() && C->is_method_compilation()) {
    // testl %rax, off(%rip) // Opcode + ModRM + Disp32 == 6 bytes
    // XXX reg_mem doesn't support RIP-relative addressing yet
    cbuf.set_inst_mark();
    cbuf.relocate(cbuf.inst_mark(), relocInfo::poll_return_type, 0); // XXX
    emit_opcode(cbuf, 0x85); // testl
    emit_rm(cbuf, 0x0, RAX_enc, 0x5); // 00 rax 101 == 0x5
    // cbuf.inst_mark() is beginning of instruction
    emit_d32_reloc(cbuf, os::get_polling_page());
//                    relocInfo::poll_return_type, 
  }
}

uint MachEpilogNode::size(PhaseRegAlloc* ra_) const
{
  Compile* C = ra_->C;
  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove word for return adr already pushed
  // and RBP
  framesize -= 2*wordSize;

  uint size = 0;

  if (do_polling() && C->is_method_compilation()) { 
    size += 6;
  }

  // count popq rbp
  size++;

  if (framesize) {
    if (framesize < 0x80) {
      size += 4;
    } else if (framesize) {
      size += 7;
    }
  }

  return size;
}

int MachEpilogNode::reloc() const 
{
  return 2; // a large enough number
}

const Pipeline* MachEpilogNode::pipeline() const
{
  return MachNode::pipeline_class();
}

int MachEpilogNode::safepoint_offset() const
{
  return 0;
}

//=============================================================================

enum RC {
  rc_bad, 
  rc_int, 
  rc_float,
  rc_stack 
};

static enum RC rc_class(OptoReg::Name reg) 
{
  if( !OptoReg::is_valid(reg)  ) return rc_bad;

  if (OptoReg::is_stack(reg)) return rc_stack;

  VMReg r = OptoReg::as_VMReg(reg);

  if (r->is_Register()) return rc_int;

  assert(r->is_XMMRegister(), "must be");
  return rc_float;
}

uint MachSpillCopyNode::implementation(CodeBuffer* cbuf, 
                                       PhaseRegAlloc* ra_, 
                                       bool do_size) const
{

  // Get registers to move 
  OptoReg::Name src_second = ra_->get_reg_second(in(1));
  OptoReg::Name src_first = ra_->get_reg_first(in(1));
  OptoReg::Name dst_second = ra_->get_reg_second(this);
  OptoReg::Name dst_first = ra_->get_reg_first(this);

  enum RC src_second_rc = rc_class(src_second);
  enum RC src_first_rc = rc_class(src_first);
  enum RC dst_second_rc = rc_class(dst_second);
  enum RC dst_first_rc = rc_class(dst_first);

  assert(OptoReg::is_valid(src_first) && OptoReg::is_valid(dst_first),
         "must move at least 1 register" );

  if (src_first == dst_first && src_second == dst_second) {
    // Self copy, no move
    return 0;
  } else if (src_first_rc == rc_stack) {
    // mem ->
    if (dst_first_rc == rc_stack) {
      // mem -> mem
      assert(src_second != dst_first, "overlap");
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int src_offset = ra_->reg2offset(src_first);
        int dst_offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          emit_opcode(*cbuf, 0xFF);
          encode_RegMem(*cbuf, RSI_enc, RSP_enc, 0x4, 0, src_offset, false);

          emit_opcode(*cbuf, 0x8F);
          encode_RegMem(*cbuf, RAX_enc, RSP_enc, 0x4, 0, dst_offset, false);

#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("pushq   [rsp + #%d]\t# 64-bit mem-mem spill\n\t"
                     "popq    [rsp + #%d]", 
                     src_offset,
                     dst_offset);
#endif
        }
        return 
          3 + ((src_offset == 0) ? 0 : (src_offset < 0x80 ? 1 : 4)) +
          3 + ((dst_offset == 0) ? 0 : (dst_offset < 0x80 ? 1 : 4));
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        // No pushl/popl, so:
        int src_offset = ra_->reg2offset(src_first);
        int dst_offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          emit_opcode(*cbuf, Assembler::REX_W);
          emit_opcode(*cbuf, 0x89);
          emit_opcode(*cbuf, 0x44);
          emit_opcode(*cbuf, 0x24);
          emit_opcode(*cbuf, 0xF8);

          emit_opcode(*cbuf, 0x8B);
          encode_RegMem(*cbuf, 
                        RAX_enc, 
                        RSP_enc, 0x4, 0, src_offset, 
                        false);

          emit_opcode(*cbuf, 0x89);
          encode_RegMem(*cbuf, 
                        RAX_enc, 
                        RSP_enc, 0x4, 0, dst_offset, 
                        false);
          
          emit_opcode(*cbuf, Assembler::REX_W);
          emit_opcode(*cbuf, 0x8B);
          emit_opcode(*cbuf, 0x44);
          emit_opcode(*cbuf, 0x24);
          emit_opcode(*cbuf, 0xF8);

#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movq    [rsp - #8], rax\t# 32-bit mem-mem spill\n\t"
                     "movl    rax, [rsp + #%d]\n\t"
                     "movl    [rsp + #%d], rax\n\t"
                     "movq    rax, [rsp - #8]",
                     src_offset,
                     dst_offset);
#endif
        }
        return 
          5 + // movq
          3 + ((src_offset == 0) ? 0 : (src_offset < 0x80 ? 1 : 4)) + // movl
          3 + ((dst_offset == 0) ? 0 : (dst_offset < 0x80 ? 1 : 4)) + // movl
          5; // movq
      }
    } else if (dst_first_rc == rc_int) {
      // mem -> gpr
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int offset = ra_->reg2offset(src_first);
        if (cbuf) {
          if (Matcher::_regEncode[dst_first] < 8) {
            emit_opcode(*cbuf, Assembler::REX_W);
          } else {
            emit_opcode(*cbuf, Assembler::REX_WR);
          }
          emit_opcode(*cbuf, 0x8B);
          encode_RegMem(*cbuf,
                        Matcher::_regEncode[dst_first], 
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movq    %s, [rsp + #%d]\t# spill", 
                     Matcher::regName[dst_first],
                     offset);
#endif
        }
        return 
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) + 4; // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        int offset = ra_->reg2offset(src_first);
        if (cbuf) {
          if (Matcher::_regEncode[dst_first] >= 8) {
            emit_opcode(*cbuf, Assembler::REX_R);
          }
          emit_opcode(*cbuf, 0x8B);
          encode_RegMem(*cbuf,
                        Matcher::_regEncode[dst_first], 
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movl    %s, [rsp + #%d]\t# spill", 
                     Matcher::regName[dst_first],
                     offset);
#endif
        }
        return 
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) +
          ((Matcher::_regEncode[dst_first] < 8)
           ? 3
           : 4); // REX
      }
    } else if (dst_first_rc == rc_float) {
      // mem-> xmm
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int offset = ra_->reg2offset(src_first);
        if (cbuf) {
          emit_opcode(*cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
          if (Matcher::_regEncode[dst_first] >= 8) {
            emit_opcode(*cbuf, Assembler::REX_R);
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
          encode_RegMem(*cbuf, 
                        Matcher::_regEncode[dst_first], 
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("%s  %s, [rsp + #%d]\t# spill", 
                     UseXmmLoadAndClearUpper ? "movsd " : "movlpd",
                     Matcher::regName[dst_first],
                     offset);
#endif
        }
        return 
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) +
          ((Matcher::_regEncode[dst_first] < 8)
           ? 5
           : 6); // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        int offset = ra_->reg2offset(src_first);
        if (cbuf) {
          emit_opcode(*cbuf, 0xF3);
          if (Matcher::_regEncode[dst_first] >= 8) {
            emit_opcode(*cbuf, Assembler::REX_R);
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x10);
          encode_RegMem(*cbuf, 
                        Matcher::_regEncode[dst_first], 
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movss   %s, [rsp + #%d]\t# spill", 
                     Matcher::regName[dst_first],
                     offset);
#endif
        }
        return 
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) +
          ((Matcher::_regEncode[dst_first] < 8)
           ? 5
           : 6); // REX
      }
    }
  } else if (src_first_rc == rc_int) {
    // gpr ->
    if (dst_first_rc == rc_stack) {
      // gpr -> mem
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          if (Matcher::_regEncode[src_first] < 8) {
            emit_opcode(*cbuf, Assembler::REX_W);
          } else {
            emit_opcode(*cbuf, Assembler::REX_WR);
          }
          emit_opcode(*cbuf, 0x89);
          encode_RegMem(*cbuf, 
                        Matcher::_regEncode[src_first], 
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movq    [rsp + #%d], %s\t# spill", 
                     offset,
                     Matcher::regName[src_first]);
#endif
        }
        return ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) + 4; // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        int offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          if (Matcher::_regEncode[src_first] >= 8) {
            emit_opcode(*cbuf, Assembler::REX_R);
          }
          emit_opcode(*cbuf, 0x89);
          encode_RegMem(*cbuf, 
                        Matcher::_regEncode[src_first], 
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movl    [rsp + #%d], %s\t# spill", 
                     offset,
                     Matcher::regName[src_first]);
#endif
        }
        return 
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) +
          ((Matcher::_regEncode[src_first] < 8)
           ? 3
           : 4); // REX
      }
    } else if (dst_first_rc == rc_int) {
      // gpr -> gpr
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        if (cbuf) {
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_W);
            } else {
              emit_opcode(*cbuf, Assembler::REX_WB);
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_WR);
            } else {
              emit_opcode(*cbuf, Assembler::REX_WRB);
            }
          }
          emit_opcode(*cbuf, 0x8B);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movq    %s, %s\t# spill",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return 3; // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        if (cbuf) {
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_B);
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_R);
            } else {
              emit_opcode(*cbuf, Assembler::REX_RB);
            }
          }
          emit_opcode(*cbuf, 0x8B);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movl    %s, %s\t# spill", 
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return 
          (Matcher::_regEncode[src_first] < 8 && Matcher::_regEncode[dst_first] < 8)
          ? 2
          : 3; // REX
      }
    } else if (dst_first_rc == rc_float) {
      // gpr -> xmm
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        if (cbuf) {
          emit_opcode(*cbuf, 0x66);
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_W);
            } else {
              emit_opcode(*cbuf, Assembler::REX_WB);
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_WR);
            } else {
              emit_opcode(*cbuf, Assembler::REX_WRB);
            }
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x6E);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movdq   %s, %s\t# spill", 
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return 5; // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        if (cbuf) {
          emit_opcode(*cbuf, 0x66);
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_B);
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_R);
            } else {
              emit_opcode(*cbuf, Assembler::REX_RB);
            }
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x6E);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movdl   %s, %s\t# spill", 
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return 
          (Matcher::_regEncode[src_first] < 8 && Matcher::_regEncode[dst_first] < 8)
          ? 4
          : 5; // REX
      }
    }
  } else if (src_first_rc == rc_float) {
    // xmm ->
    if (dst_first_rc == rc_stack) {
      // xmm -> mem
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          emit_opcode(*cbuf, 0xF2);
          if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_R);
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x11);
          encode_RegMem(*cbuf, 
                        Matcher::_regEncode[src_first], 
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movsd   [rsp + #%d], %s\t# spill", 
                     offset,
                     Matcher::regName[src_first]);
#endif
        }
        return 
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) +
          ((Matcher::_regEncode[src_first] < 8)
           ? 5
           : 6); // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        int offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          emit_opcode(*cbuf, 0xF3);
          if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_R);
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x11);
          encode_RegMem(*cbuf, 
                        Matcher::_regEncode[src_first], 
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movss   [rsp + #%d], %s\t# spill", 
                     offset,
                     Matcher::regName[src_first]);
#endif
        }
        return 
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) +
          ((Matcher::_regEncode[src_first] < 8)
           ? 5
           : 6); // REX
      }
    } else if (dst_first_rc == rc_int) {
      // xmm -> gpr
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        if (cbuf) {
          emit_opcode(*cbuf, 0x66);
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_W);
            } else {
              emit_opcode(*cbuf, Assembler::REX_WR); // attention!
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_WB); // attention!
            } else {
              emit_opcode(*cbuf, Assembler::REX_WRB);
            }
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x7E);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movdq   %s, %s\t# spill", 
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return 5; // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        if (cbuf) {
          emit_opcode(*cbuf, 0x66);
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_R); // attention!
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_B); // attention!
            } else {
              emit_opcode(*cbuf, Assembler::REX_RB);
            }
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x7E);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("movdl   %s, %s\t# spill", 
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return 
          (Matcher::_regEncode[src_first] < 8 && Matcher::_regEncode[dst_first] < 8)
          ? 4
          : 5; // REX
      }
    } else if (dst_first_rc == rc_float) {
      // xmm -> xmm
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        if (cbuf) {
          emit_opcode(*cbuf, UseXmmRegToRegMoveAll ? 0x66 : 0xF2);
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_B);
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_R);
            } else {
              emit_opcode(*cbuf, Assembler::REX_RB);
            }
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, UseXmmRegToRegMoveAll ? 0x28 : 0x10);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("%s  %s, %s\t# spill", 
                     UseXmmRegToRegMoveAll ? "movapd" : "movsd ",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return 
          (Matcher::_regEncode[src_first] < 8 && Matcher::_regEncode[dst_first] < 8)
          ? 4
          : 5; // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        if (cbuf) {
          if (!UseXmmRegToRegMoveAll) 
            emit_opcode(*cbuf, 0xF3);
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_B);
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_R);
            } else {
              emit_opcode(*cbuf, Assembler::REX_RB);
            }
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, UseXmmRegToRegMoveAll ? 0x28 : 0x10);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          tty->print("%s  %s, %s\t# spill", 
                     UseXmmRegToRegMoveAll ? "movaps" : "movss ",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return 
          (Matcher::_regEncode[src_first] < 8 && Matcher::_regEncode[dst_first] < 8)
          ? (UseXmmRegToRegMoveAll ? 3 : 4)
          : (UseXmmRegToRegMoveAll ? 4 : 5); // REX
      }
    }
  }    

  assert(0," foo ");
  Unimplemented();

  return 0;
}

#ifndef PRODUCT
void MachSpillCopyNode::format(PhaseRegAlloc *ra_) const 
{
  implementation(NULL, ra_, false);
}
#endif

void MachSpillCopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const 
{
  implementation(&cbuf, ra_, false);
}

uint MachSpillCopyNode::size(PhaseRegAlloc *ra_) const 
{
  return implementation(NULL, ra_, true);
}

//=============================================================================
#ifndef PRODUCT
void MachNopNode::format(PhaseRegAlloc*) const
{
  tty->print("nop \t# %d bytes pad for loops and calls", _count);
}
#endif

void MachNopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc*) const 
{
  MacroAssembler _masm(&cbuf);
  __ nop(_count);
}

uint MachNopNode::size(PhaseRegAlloc*) const
{
  return _count;
}


//=============================================================================
#ifndef PRODUCT
void BoxLockNode::format(PhaseRegAlloc* ra_) const
{
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_reg_first(this);
  tty->print("leaq    %s, [rsp + #%d]\t# box lock", 
             Matcher::regName[reg], offset);
}
#endif

void BoxLockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const
{
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_encode(this);
  if (offset >= 0x80) {
    emit_opcode(cbuf, reg < 8 ? Assembler::REX_W : Assembler::REX_WR);
    emit_opcode(cbuf, 0x8D); // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x2, reg & 7, 0x04);
    emit_rm(cbuf, 0x0, 0x04, RSP_enc);
    emit_d32(cbuf, offset);
  } else {
    emit_opcode(cbuf, reg < 8 ? Assembler::REX_W : Assembler::REX_WR);
    emit_opcode(cbuf, 0x8D); // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x1, reg & 7, 0x04);
    emit_rm(cbuf, 0x0, 0x04, RSP_enc);
    emit_d8(cbuf, offset);
  }
}

uint BoxLockNode::size(PhaseRegAlloc *ra_) const
{
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  return (offset < 0x80) ? 5 : 8; // REX
}

//=============================================================================

// emit call stub, compiled java to interpreter
void emit_java_to_interp(CodeBuffer& cbuf)
{
  // Stub is fixed up when the corresponding call is converted from
  // calling compiled code to calling interpreted code.
  // movq rbx, 0
  // jmp -5 # to self

  address mark = cbuf.inst_mark();  // get mark within main instrs section

  // Note that the code buffer's inst_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a stub.
  MacroAssembler _masm(&cbuf);

  address base =
  __ start_a_stub(Compile::MAX_stubs_size);
  if (base == NULL)  return;  // CodeBuffer::expand failed
  // static stub relocation stores the instruction address of the call
  __ relocate(static_stub_Relocation::spec(mark), RELOC_IMM64);
  // static stub relocation also tags the methodOop in the code-stream.
  __ movoop(rbx, (jobject) NULL);  // method is zapped till fixup time
  __ jump(RuntimeAddress(__ pc()));

  // Update current stubs pointer and restore code_end.
  __ end_a_stub();
}

// size of call stub, compiled java to interpretor
uint size_java_to_interp()
{
  return 15;  // movq (1+1+8); jmp (1+4)
}

// relocation entries for call stub, compiled java to interpretor
uint reloc_java_to_interp() 
{
  return 4; // 3 in emit_java_to_interp + 1 in Java_Static_Call
}

//=============================================================================
#ifndef PRODUCT
void MachUEPNode::format(PhaseRegAlloc* ra_) const
{
  tty->print_cr("cmpq    rax, [j_rarg0 + oopDesc::klass_offset_in_bytes() #%d]\t"
                "# Inline cache check", oopDesc::klass_offset_in_bytes());
  tty->print_cr("\tjne     SharedRuntime::_ic_miss_stub");
  tty->print_cr("\tnop");
  if (!OptoBreakpoint) {
    tty->print_cr("\tnop");
  }
}
#endif

void MachUEPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const
{
  MacroAssembler masm(&cbuf);
#ifdef ASSERT
  uint code_size = cbuf.code_size();
#endif
  masm.cmpq(rax, Address(j_rarg0, oopDesc::klass_offset_in_bytes()));

  masm.jump_cc(Assembler::notEqual, RuntimeAddress(SharedRuntime::get_ic_miss_stub()));

  /* WARNING these NOPs are critical so that verified entry point is properly
     aligned for patching by NativeJump::patch_verified_entry() */
  int nops_cnt = 1;
  if (!OptoBreakpoint) {
    // Leave space for int3
     nops_cnt += 1;
  }
  masm.nop(nops_cnt);

  assert(cbuf.code_size() - code_size == size(ra_), 
         "checking code size of inline cache node");
}

uint MachUEPNode::size(PhaseRegAlloc* ra_) const
{
  return OptoBreakpoint ? 11 : 12;
}


//=============================================================================
uint size_exception_handler()
{
  // NativeCall instruction size is the same as NativeJump.
  // Note that this value is also credited (in output.cpp) to
  // the size of the code section.
  return NativeJump::instruction_size;
}

// Emit exception handler code. 
int emit_exception_handler(CodeBuffer& cbuf) 
{

  // Note that the code buffer's inst_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  MacroAssembler _masm(&cbuf);
  address base =
  __ start_a_stub(size_exception_handler());
  if (base == NULL)  return 0;  // CodeBuffer::expand failed
  int offset = __ offset();
  __ jump(RuntimeAddress(OptoRuntime::exception_blob()->instructions_begin()));
  assert(__ offset() - offset <= (int) size_exception_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

uint size_deopt_handler()
{
  // three 5 byte instructions
  return 15;
}

// Emit deopt handler code.  
int emit_deopt_handler(CodeBuffer& cbuf) 
{

  // Note that the code buffer's inst_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  MacroAssembler _masm(&cbuf);
  address base =
  __ start_a_stub(size_deopt_handler());
  if (base == NULL)  return 0;  // CodeBuffer::expand failed
  int offset = __ offset();
  address the_pc = (address) __ pc();
  Label next;
  // push a "the_pc" on the stack without destroying any registers
  // as they all may be live.

  // push address of "next"
  __ call(next, relocInfo::none); // reloc none is fine since it is a disp32
  __ bind(next);
  // adjust it so it matches "the_pc"
  __ subq(Address(rsp, 0), __ offset() - offset);
  __ jump(RuntimeAddress(SharedRuntime::deopt_blob()->unpack()));
  assert(__ offset() - offset <= (int) size_deopt_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

static void emit_double_constant(CodeBuffer& cbuf, double x) {
  int mark = cbuf.insts()->mark_off();
  MacroAssembler _masm(&cbuf);
  address double_address = __ double_constant(x);
  cbuf.insts()->set_mark_off(mark);  // preserve mark across masm shift
  emit_d32_reloc(cbuf,
		 (int) (double_address - cbuf.code_end() - 4),
		 internal_word_Relocation::spec(double_address),
		 RELOC_DISP32);
}

static void emit_float_constant(CodeBuffer& cbuf, float x) {
  int mark = cbuf.insts()->mark_off();
  MacroAssembler _masm(&cbuf);
  address float_address = __ float_constant(x);
  cbuf.insts()->set_mark_off(mark);  // preserve mark across masm shift
  emit_d32_reloc(cbuf, 
		 (int) (float_address - cbuf.code_end() - 4),
		 internal_word_Relocation::spec(float_address),
		 RELOC_DISP32);
}


int Matcher::regnum_to_fpu_offset(int regnum)
{
  return regnum - 32; // The FP registers are in the second chunk
}

// This is UltraSparc specific, true just means we have fast l2f conversion
const bool Matcher::convL2FSupported(void) {
  return true;
}

// Vector width in bytes
const uint Matcher::vector_width_in_bytes(void) {
  return 8;
}

// Vector ideal reg
const uint Matcher::vector_ideal_reg(void) {
  return Op_RegD;
}

// Is this branch offset short enough that a short branch can be used?
//
// NOTE: If the platform does not provide any short branch variants, then
//       this method should return false for offset 0.
bool Matcher::is_short_branch_offset(int offset) 
{
  return -0x80 <= offset && offset < 0x80;
}

const bool Matcher::isSimpleConstant64(jlong value) {
  // Will one (StoreL ConL) be cheaper than two (StoreI ConI)?.
  //return value == (int) value;  // Cf. storeImmL and immL32.

  // Probably always true, even if a temp register is required.
  return true;
}

// The ecx parameter to rep stosq for the ClearArray node is in words.
const bool Matcher::init_array_count_is_in_bytes = false;

// Threshold size for cleararray.
const int Matcher::init_array_short_size = 8 * BytesPerLong;

// Should the Matcher clone shifts on addressing modes, expecting them
// to be subsumed into complex addressing expressions or compute them
// into registers?  True for Intel but false for most RISCs
const bool Matcher::clone_shift_expressions = true;

// Is it better to copy float constants, or load them directly from
// memory?  Intel can load a float constant from a direct address,
// requiring no extra registers.  Most RISCs will have to materialize
// an address into a register first, so they would do better to copy
// the constant from stack.
const bool Matcher::rematerialize_float_constants = true; // XXX

// If CPU can load and store mis-aligned doubles directly then no
// fixup is needed.  Else we split the double into 2 integer pieces
// and move it piece-by-piece.  Only happens when passing doubles into
// C code as the Java calling convention forces doubles to be aligned.
const bool Matcher::misaligned_doubles_ok = true;

// No-op on amd64
void Matcher::pd_implicit_null_fixup(MachNode *node, uint idx) {}

// Advertise here if the CPU requires explicit rounding operations to
// implement the UseStrictFP mode.
const bool Matcher::strict_fp_requires_explicit_rounding = true;

// Do floats take an entire double register or just half?
const bool Matcher::float_in_double = true;
// Do ints take an entire long register or just half?
const bool Matcher::int_in_long = true;

// Return whether or not this register is ever used as an argument.
// This function is used on startup to build the trampoline stubs in
// generateOptoStub.  Registers not mentioned will be killed by the VM
// call in the trampoline, and arguments in those registers not be
// available to the callee.
bool Matcher::can_be_java_arg(int reg)
{
  return
    reg ==  RDI_num || reg ==  RDI_H_num || 
    reg ==  RSI_num || reg ==  RSI_H_num || 
    reg ==  RDX_num || reg ==  RDX_H_num ||
    reg ==  RCX_num || reg ==  RCX_H_num || 
    reg ==   R8_num || reg ==   R8_H_num || 
    reg ==   R9_num || reg ==   R9_H_num || 
    reg == XMM0_num || reg == XMM0_H_num ||
    reg == XMM1_num || reg == XMM1_H_num ||
    reg == XMM2_num || reg == XMM2_H_num ||
    reg == XMM3_num || reg == XMM3_H_num ||
    reg == XMM4_num || reg == XMM4_H_num ||
    reg == XMM5_num || reg == XMM5_H_num ||
    reg == XMM6_num || reg == XMM6_H_num ||
    reg == XMM7_num || reg == XMM7_H_num;
}

bool Matcher::is_spillable_arg(int reg)
{
  return can_be_java_arg(reg);
}

// Register for DIVI projection of divmodI
RegMask Matcher::divI_proj_mask() {
  return INT_RAX_REG_mask;
}

// Register for MODI projection of divmodI
RegMask Matcher::modI_proj_mask() {
  return INT_RDX_REG_mask;
}

// Register for DIVL projection of divmodL
RegMask Matcher::divL_proj_mask() {
  return LONG_RAX_REG_mask;
}

// Register for MODL projection of divmodL
RegMask Matcher::modL_proj_mask() {
  return LONG_RDX_REG_mask;
}



//SourceForm


int MachCallRuntimeNode::ret_addr_offset() {
  return 13; // movq r10,#addr; callq (r10)
}

// emit an interrupt that is caught by the debugger
void emit_break(CodeBuffer& cbuf) {
  // Debugger doesn't really catch this but best we can do so far QQQ
  MacroAssembler* masm = new MacroAssembler(&cbuf);
  masm->call(RuntimeAddress(CAST_FROM_FN_PTR(address, os::breakpoint)));
}

void MachBreakpointNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  emit_break(cbuf);
}

uint MachBreakpointNode::size(PhaseRegAlloc* ra_) const {
  return 5;
}



#ifndef PRODUCT
void Compile::adlc_verification() {

  // Following assertions generated from definition section
}
#endif

// Map from machine-independent register number to register_save_policy
const        char register_save_policy[] = {
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'N',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'N',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C' // no trailing comma
};

// Map from machine-independent register number to c_reg_save_policy
const        char c_reg_save_policy[] = {
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'E',
  'E',
  'C',
  'C',
  'E',
  'E',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'E',
  'E',
  'E',
  'E',
  'E',
  'E',
  'E',
  'E',
  'N',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C' // no trailing comma
};

// Map from machine-independent register number to register_save_type
const        int register_save_type[] = {
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  0 // no trailing comma
};


const        int   reduceOp[] = {
  0, // 0
  0, // 1
  0, // 2
  0, // 3
  0, // 4
  0, // 5
  0, // 6
  0, // 7
  immI_rule, // 8
  immI0_rule, // 9
  immI1_rule, // 10
  immI_M1_rule, // 11
  immI2_rule, // 12
  immI8_rule, // 13
  immI16_rule, // 14
  immI_32_rule, // 15
  immI_64_rule, // 16
  immP_rule, // 17
  immP0_rule, // 18
  immP31_rule, // 19
  immL_rule, // 20
  immL8_rule, // 21
  immUL32_rule, // 22
  immL32_rule, // 23
  immL0_rule, // 24
  immL1_rule, // 25
  immL_M1_rule, // 26
  immL10_rule, // 27
  immL_127_rule, // 28
  immL_32bits_rule, // 29
  immF0_rule, // 30
  immF_rule, // 31
  immD0_rule, // 32
  immD_rule, // 33
  immI_16_rule, // 34
  immI_24_rule, // 35
  immI_255_rule, // 36
  immI_65535_rule, // 37
  immL_255_rule, // 38
  immL_65535_rule, // 39
  rRegI_rule, // 40
  rax_RegI_rule, // 41
  rbx_RegI_rule, // 42
  rcx_RegI_rule, // 43
  rdx_RegI_rule, // 44
  rdi_RegI_rule, // 45
  no_rcx_RegI_rule, // 46
  no_rax_rdx_RegI_rule, // 47
  any_RegP_rule, // 48
  rRegP_rule, // 49
  no_rax_RegP_rule, // 50
  no_rbp_RegP_rule, // 51
  no_rax_rbx_RegP_rule, // 52
  rax_RegP_rule, // 53
  rbx_RegP_rule, // 54
  rsi_RegP_rule, // 55
  rdi_RegP_rule, // 56
  rbp_RegP_rule, // 57
  r15_RegP_rule, // 58
  rRegL_rule, // 59
  no_rax_rdx_RegL_rule, // 60
  no_rax_RegL_rule, // 61
  no_rcx_RegL_rule, // 62
  rax_RegL_rule, // 63
  rcx_RegL_rule, // 64
  rdx_RegL_rule, // 65
  rFlagsReg_rule, // 66
  rFlagsRegU_rule, // 67
  regF_rule, // 68
  regD_rule, // 69
  indirect_rule, // 70
  indOffset8_rule, // 71
  indOffset32_rule, // 72
  indIndexOffset_rule, // 73
  indIndex_rule, // 74
  indIndexScale_rule, // 75
  indIndexScaleOffset_rule, // 76
  stackSlotP_rule, // 77
  stackSlotI_rule, // 78
  stackSlotF_rule, // 79
  stackSlotD_rule, // 80
  stackSlotL_rule, // 81
  cmpOp_rule, // 82
  cmpOpU_rule, // 83
  // last operand
  memory_rule, // 84
  // last operand class
  _AddP_any_RegP_rRegL_rule, // 85
  _LShiftL_rRegL_immI2_rule, // 86
  _AddP_any_RegP__LShiftL_rRegL_immI2_rule, // 87
  _LoadB_memory__rule, // 88
  _LoadI_memory__rule, // 89
  _LoadL_memory__rule, // 90
  _ReverseBytesI_rRegI__rule, // 91
  _ReverseBytesL_rRegL__rule, // 92
  _AddL__LShiftL_rRegL_immI2_immL32_rule, // 93
  _Binary_cmpOp_rFlagsReg_rule, // 94
  _Binary_rRegI_rRegI_rule, // 95
  _Binary_cmpOpU_rFlagsRegU_rule, // 96
  _Binary_rRegI__LoadI_memory__rule, // 97
  _Binary_rRegP_rRegP_rule, // 98
  _Binary_rRegL_rRegL_rule, // 99
  _Binary_rRegL__LoadL_memory__rule, // 100
  _Binary_regF_regF_rule, // 101
  _Binary_regD_regD_rule, // 102
  _AddI__LoadI_memory__rRegI_rule, // 103
  _AddI_rRegI__LoadI_memory__rule, // 104
  _AddI__LoadI_memory__immI_rule, // 105
  _AddI__LoadI_memory__immI1_rule, // 106
  _AddI__LoadI_memory__immI_M1_rule, // 107
  _AddL__LoadL_memory__rRegL_rule, // 108
  _AddL_rRegL__LoadL_memory__rule, // 109
  _AddL__LoadL_memory__immL32_rule, // 110
  _AddL__LoadL_memory__immL1_rule, // 111
  _AddL__LoadL_memory__immL_M1_rule, // 112
  _Binary_rax_RegP_rRegP_rule, // 113
  _Binary_rax_RegL_rRegL_rule, // 114
  _StoreLConditional_memory__Binary_rax_RegL_rRegL_rule, // 115
  _Binary_rax_RegI_rRegI_rule, // 116
  _SubI__LoadI_memory__rRegI_rule, // 117
  _SubI__LoadI_memory__immI_rule, // 118
  _SubL__LoadL_memory__rRegL_rule, // 119
  _SubL__LoadL_memory__immL32_rule, // 120
  _SubI_immI0_rRegI_rule, // 121
  _SubI_immI0__LoadI_memory__rule, // 122
  _SubL_immL0__LoadL_memory__rule, // 123
  _LShiftI__LoadI_memory__immI1_rule, // 124
  _LShiftI__LoadI_memory__immI8_rule, // 125
  _LShiftI__LoadI_memory__rcx_RegI_rule, // 126
  _RShiftI__LoadI_memory__immI1_rule, // 127
  _RShiftI__LoadI_memory__immI8_rule, // 128
  _RShiftI__LoadI_memory__rcx_RegI_rule, // 129
  _URShiftI__LoadI_memory__immI1_rule, // 130
  _URShiftI__LoadI_memory__immI8_rule, // 131
  _URShiftI__LoadI_memory__rcx_RegI_rule, // 132
  _LShiftL__LoadL_memory__immI1_rule, // 133
  _LShiftL__LoadL_memory__immI8_rule, // 134
  _LShiftL__LoadL_memory__rcx_RegI_rule, // 135
  _RShiftL__LoadL_memory__immI1_rule, // 136
  _RShiftL__LoadL_memory__immI8_rule, // 137
  _RShiftL__LoadL_memory__rcx_RegI_rule, // 138
  _URShiftL__LoadL_memory__immI1_rule, // 139
  _URShiftL__LoadL_memory__immI8_rule, // 140
  _URShiftL__LoadL_memory__rcx_RegI_rule, // 141
  _LShiftI_rRegI_immI_24_rule, // 142
  _LShiftI_rRegI_immI_16_rule, // 143
  _LShiftI_rRegI_immI1_rule, // 144
  _URShiftI_rRegI_immI_M1_rule, // 145
  _LShiftI_rRegI_immI8_rule, // 146
  _URShiftI_rRegI_immI8_rule, // 147
  _LShiftI_no_rcx_RegI_rcx_RegI_rule, // 148
  _SubI_immI0_rcx_RegI_rule, // 149
  _URShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule, // 150
  _SubI_immI_32_rcx_RegI_rule, // 151
  _URShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule, // 152
  _URShiftI_rRegI_immI1_rule, // 153
  _LShiftI_rRegI_immI_M1_rule, // 154
  _URShiftI_no_rcx_RegI_rcx_RegI_rule, // 155
  _LShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule, // 156
  _LShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule, // 157
  _LShiftL_rRegL_immI1_rule, // 158
  _URShiftL_rRegL_immI_M1_rule, // 159
  _LShiftL_rRegL_immI8_rule, // 160
  _URShiftL_rRegL_immI8_rule, // 161
  _LShiftL_no_rcx_RegL_rcx_RegI_rule, // 162
  _URShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule, // 163
  _SubI_immI_64_rcx_RegI_rule, // 164
  _URShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule, // 165
  _URShiftL_rRegL_immI1_rule, // 166
  _LShiftL_rRegL_immI_M1_rule, // 167
  _URShiftL_no_rcx_RegL_rcx_RegI_rule, // 168
  _LShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule, // 169
  _LShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule, // 170
  _AndI_rRegI_immI_255_rule, // 171
  _AndI_rRegI_immI_65535_rule, // 172
  _AndI__LoadI_memory__rRegI_rule, // 173
  _AndI_rRegI__LoadI_memory__rule, // 174
  _AndI__LoadI_memory__immI_rule, // 175
  _OrI__LoadI_memory__rRegI_rule, // 176
  _OrI_rRegI__LoadI_memory__rule, // 177
  _OrI__LoadI_memory__immI_rule, // 178
  _XorI__LoadI_memory__rRegI_rule, // 179
  _XorI_rRegI__LoadI_memory__rule, // 180
  _XorI__LoadI_memory__immI_rule, // 181
  _AndL__LoadL_memory__rRegL_rule, // 182
  _AndL_rRegL__LoadL_memory__rule, // 183
  _AndL__LoadL_memory__immL32_rule, // 184
  _OrL__LoadL_memory__rRegL_rule, // 185
  _OrL_rRegL__LoadL_memory__rule, // 186
  _OrL__LoadL_memory__immL32_rule, // 187
  _XorL__LoadL_memory__rRegL_rule, // 188
  _XorL_rRegL__LoadL_memory__rule, // 189
  _XorL__LoadL_memory__immL32_rule, // 190
  _CmpLTMask_rRegI_rRegI_rule, // 191
  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule, // 192
  _SubI_rRegI_rRegI_rule, // 193
  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule, // 194
  _LoadF_memory__rule, // 195
  _LoadD_memory__rule, // 196
  _ConvF2D_regF__rule, // 197
  _SqrtD__ConvF2D_regF___rule, // 198
  _ConvF2D__LoadF_memory___rule, // 199
  _SqrtD__ConvF2D__LoadF_memory____rule, // 200
  _ConvF2D_immF__rule, // 201
  _SqrtD__ConvF2D_immF___rule, // 202
  _ConvI2L_rRegI__rule, // 203
  _ConvI2L__LoadI_memory___rule, // 204
  _AndI_rRegI_immI_rule, // 205
  _LoadP_memory__rule, // 206
  _AndL_rRegL_immL32_rule, // 207
  _PartialSubtypeCheck_rsi_RegP_rax_RegP_rule, // 208
  // last internally defined operand
  stackSlotI_rule, // 209
  stackSlotL_rule, // 210
  stackSlotP_rule, // 211
  stackSlotF_rule, // 212
  stackSlotD_rule, // 213
  rRegP_rule, // 214
  rRegP_rule, // 215
  rRegP_rule, // 216
  rRegP_rule, // 217
  rRegP_rule, // 218
  rRegI_rule, // 219
  rRegI_rule, // 220
  rRegL_rule, // 221
  rRegL_rule, // 222
  rRegL_rule, // 223
  rRegL_rule, // 224
  rRegP_rule, // 225
  rRegP_rule, // 226
  rRegP_rule, // 227
  regF_rule, // 228
  regF_rule, // 229
  regD_rule, // 230
  regD_rule, // 231
  rRegI_rule, // 232
  rRegL_rule, // 233
  rRegP_rule, // 234
  regF_rule, // 235
  regD_rule, // 236
  rRegP_rule, // 237
  rRegL_rule, // 238
  rRegP_rule, // 239
  rRegP_rule, // 240
  rFlagsReg_rule, // 241
  rFlagsReg_rule, // 242
  rFlagsRegU_rule, // 243
  rFlagsRegU_rule, // 244
  rFlagsRegU_rule, // 245
  rFlagsRegU_rule, // 246
  rFlagsRegU_rule, // 247
  rFlagsRegU_rule, // 248
  regF_rule, // 249
  regD_rule, // 250
  regF_rule, // 251
  regD_rule, // 252
  regF_rule, // 253
  regD_rule, // 254
  rRegI_rule, // 255
  regF_rule, // 256
  rRegL_rule, // 257
  regD_rule, // 258
  regD_rule, // 259
  rRegI_rule, // 260
  rRegL_rule, // 261
  regF_rule, // 262
  regD_rule, // 263
  rFlagsReg_rule, // 264
  rFlagsReg_rule, // 265
  rFlagsReg_rule, // 266
  rFlagsReg_rule, // 267
  rFlagsReg_rule, // 268
  rFlagsReg_rule, // 269
  rFlagsReg_rule, // 270
  rFlagsRegU_rule, // 271
  rFlagsRegU_rule, // 272
  rFlagsRegU_rule, // 273
  rFlagsRegU_rule, // 274
  rFlagsRegU_rule, // 275
  rFlagsRegU_rule, // 276
  rFlagsRegU_rule, // 277
  rFlagsReg_rule, // 278
  rFlagsReg_rule, // 279
  rFlagsReg_rule, // 280
  rFlagsReg_rule, // 281
  rFlagsReg_rule, // 282
  rFlagsReg_rule, // 283
  rFlagsReg_rule, // 284
  rFlagsReg_rule, // 285
  rFlagsReg_rule, // 286
  rFlagsReg_rule, // 287
  rFlagsReg_rule, // 288
  rFlagsReg_rule, // 289
  Universe_rule, // 290
  r15_RegP_rule, // 291
  rRegI_rule, // 292
  rRegI_rule, // 293
  rRegI_rule, // 294
  rRegI_rule, // 295
  rRegI_rule, // 296
  rRegL_rule, // 297
  rRegI_rule, // 298
  rRegP_rule, // 299
  rRegP_rule, // 300
  regF_rule, // 301
  regD_rule, // 302
  regD_rule, // 303
  regD_rule, // 304
  regD_rule, // 305
  regD_rule, // 306
  regD_rule, // 307
  regD_rule, // 308
  Universe_rule, // 309
  Universe_rule, // 310
  Universe_rule, // 311
  Universe_rule, // 312
  Universe_rule, // 313
  Universe_rule, // 314
  Universe_rule, // 315
  Universe_rule, // 316
  Universe_rule, // 317
  Universe_rule, // 318
  Universe_rule, // 319
  Universe_rule, // 320
  Universe_rule, // 321
  Universe_rule, // 322
  Universe_rule, // 323
  Universe_rule, // 324
  Universe_rule, // 325
  Universe_rule, // 326
  Universe_rule, // 327
  Universe_rule, // 328
  Universe_rule, // 329
  Universe_rule, // 330
  Universe_rule, // 331
  Universe_rule, // 332
  Universe_rule, // 333
  Universe_rule, // 334
  Universe_rule, // 335
  rRegI_rule, // 336
  rRegL_rule, // 337
  rRegI_rule, // 338
  rRegL_rule, // 339
  Universe_rule, // 340
  Universe_rule, // 341
  Universe_rule, // 342
  Universe_rule, // 343
  Universe_rule, // 344
  Universe_rule, // 345
  Universe_rule, // 346
  Universe_rule, // 347
  Universe_rule, // 348
  Universe_rule, // 349
  Universe_rule, // 350
  rRegI_rule, // 351
  rRegI_rule, // 352
  rRegI_rule, // 353
  rRegI_rule, // 354
  rRegP_rule, // 355
  rRegP_rule, // 356
  rRegL_rule, // 357
  rRegL_rule, // 358
  rRegL_rule, // 359
  rRegL_rule, // 360
  regF_rule, // 361
  regF_rule, // 362
  regD_rule, // 363
  regD_rule, // 364
  rRegI_rule, // 365
  rRegI_rule, // 366
  rRegI_rule, // 367
  rRegI_rule, // 368
  Universe_rule, // 369
  Universe_rule, // 370
  Universe_rule, // 371
  rRegI_rule, // 372
  Universe_rule, // 373
  rRegI_rule, // 374
  Universe_rule, // 375
  rRegI_rule, // 376
  rRegL_rule, // 377
  rRegL_rule, // 378
  rRegL_rule, // 379
  rRegL_rule, // 380
  Universe_rule, // 381
  Universe_rule, // 382
  Universe_rule, // 383
  rRegI_rule, // 384
  Universe_rule, // 385
  rRegL_rule, // 386
  Universe_rule, // 387
  rRegL_rule, // 388
  rRegP_rule, // 389
  rRegP_rule, // 390
  rRegP_rule, // 391
  rRegI_rule, // 392
  rRegP_rule, // 393
  rRegL_rule, // 394
  rRegI_rule, // 395
  rRegI_rule, // 396
  rRegI_rule, // 397
  rRegI_rule, // 398
  rRegI_rule, // 399
  rRegI_rule, // 400
  rRegI_rule, // 401
  Universe_rule, // 402
  Universe_rule, // 403
  rRegL_rule, // 404
  rRegI_rule, // 405
  rRegL_rule, // 406
  Universe_rule, // 407
  Universe_rule, // 408
  rRegP_rule, // 409
  rRegI_rule, // 410
  Universe_rule, // 411
  rRegL_rule, // 412
  Universe_rule, // 413
  rRegI_rule, // 414
  rRegI_rule, // 415
  rRegI_rule, // 416
  rRegI_rule, // 417
  rRegI_rule, // 418
  rRegL_rule, // 419
  rRegL_rule, // 420
  rRegL_rule, // 421
  rRegL_rule, // 422
  rRegL_rule, // 423
  rax_RegI_rule, // 424
  rax_RegL_rule, // 425
  Universe_rule, // 426
  Universe_rule, // 427
  0, // 428
  0, // 429
  0, // 430
  0, // 431
  rdx_RegL_rule, // 432
  rdx_RegI_rule, // 433
  rdx_RegL_rule, // 434
  rRegI_rule, // 435
  Universe_rule, // 436
  rRegI_rule, // 437
  Universe_rule, // 438
  rRegI_rule, // 439
  Universe_rule, // 440
  rRegI_rule, // 441
  Universe_rule, // 442
  rRegI_rule, // 443
  Universe_rule, // 444
  rRegI_rule, // 445
  Universe_rule, // 446
  rRegI_rule, // 447
  Universe_rule, // 448
  rRegI_rule, // 449
  Universe_rule, // 450
  rRegI_rule, // 451
  Universe_rule, // 452
  rRegL_rule, // 453
  Universe_rule, // 454
  rRegL_rule, // 455
  Universe_rule, // 456
  rRegL_rule, // 457
  Universe_rule, // 458
  rRegL_rule, // 459
  Universe_rule, // 460
  rRegL_rule, // 461
  Universe_rule, // 462
  rRegL_rule, // 463
  Universe_rule, // 464
  rRegL_rule, // 465
  Universe_rule, // 466
  rRegL_rule, // 467
  Universe_rule, // 468
  rRegL_rule, // 469
  Universe_rule, // 470
  rRegI_rule, // 471
  rRegI_rule, // 472
  0, // 473
  0, // 474
  0, // 475
  rRegI_rule, // 476
  rRegI_rule, // 477
  rRegI_rule, // 478
  rRegI_rule, // 479
  no_rcx_RegI_rule, // 480
  no_rcx_RegI_rule, // 481
  no_rcx_RegI_rule, // 482
  no_rcx_RegI_rule, // 483
  0, // 484
  0, // 485
  0, // 486
  rRegI_rule, // 487
  rRegI_rule, // 488
  rRegI_rule, // 489
  rRegI_rule, // 490
  no_rcx_RegI_rule, // 491
  no_rcx_RegI_rule, // 492
  no_rcx_RegI_rule, // 493
  no_rcx_RegI_rule, // 494
  0, // 495
  0, // 496
  0, // 497
  rRegL_rule, // 498
  rRegL_rule, // 499
  rRegL_rule, // 500
  rRegL_rule, // 501
  no_rcx_RegL_rule, // 502
  no_rcx_RegL_rule, // 503
  no_rcx_RegL_rule, // 504
  no_rcx_RegL_rule, // 505
  0, // 506
  0, // 507
  0, // 508
  rRegL_rule, // 509
  rRegL_rule, // 510
  rRegL_rule, // 511
  rRegL_rule, // 512
  no_rcx_RegL_rule, // 513
  no_rcx_RegL_rule, // 514
  no_rcx_RegL_rule, // 515
  no_rcx_RegL_rule, // 516
  rRegI_rule, // 517
  rRegI_rule, // 518
  rRegL_rule, // 519
  rRegI_rule, // 520
  rRegL_rule, // 521
  rRegI_rule, // 522
  rRegI_rule, // 523
  rRegI_rule, // 524
  Universe_rule, // 525
  Universe_rule, // 526
  Universe_rule, // 527
  rRegI_rule, // 528
  rRegI_rule, // 529
  rRegI_rule, // 530
  rRegI_rule, // 531
  Universe_rule, // 532
  Universe_rule, // 533
  Universe_rule, // 534
  rRegI_rule, // 535
  rRegI_rule, // 536
  rRegI_rule, // 537
  rRegI_rule, // 538
  Universe_rule, // 539
  Universe_rule, // 540
  Universe_rule, // 541
  rRegL_rule, // 542
  rRegL_rule, // 543
  rRegI_rule, // 544
  rRegL_rule, // 545
  rRegL_rule, // 546
  rRegL_rule, // 547
  Universe_rule, // 548
  Universe_rule, // 549
  Universe_rule, // 550
  rRegL_rule, // 551
  rRegL_rule, // 552
  rRegL_rule, // 553
  rRegL_rule, // 554
  Universe_rule, // 555
  Universe_rule, // 556
  Universe_rule, // 557
  rRegL_rule, // 558
  rRegL_rule, // 559
  rRegL_rule, // 560
  rRegL_rule, // 561
  Universe_rule, // 562
  Universe_rule, // 563
  Universe_rule, // 564
  rRegI_rule, // 565
  rRegI_rule, // 566
  rRegI_rule, // 567
  rRegI_rule, // 568
  rRegI_rule, // 569
  rRegI_rule, // 570
  rRegI_rule, // 571
  rRegI_rule, // 572
  rRegI_rule, // 573
  rRegI_rule, // 574
  rRegI_rule, // 575
  rRegI_rule, // 576
  rRegI_rule, // 577
  rRegI_rule, // 578
  regF_rule, // 579
  regF_rule, // 580
  regF_rule, // 581
  regF_rule, // 582
  regD_rule, // 583
  regD_rule, // 584
  regD_rule, // 585
  regD_rule, // 586
  regF_rule, // 587
  regF_rule, // 588
  regF_rule, // 589
  regD_rule, // 590
  regD_rule, // 591
  regD_rule, // 592
  regF_rule, // 593
  regF_rule, // 594
  regF_rule, // 595
  regF_rule, // 596
  regD_rule, // 597
  regD_rule, // 598
  regD_rule, // 599
  regD_rule, // 600
  regF_rule, // 601
  regF_rule, // 602
  regF_rule, // 603
  regD_rule, // 604
  regD_rule, // 605
  regD_rule, // 606
  regF_rule, // 607
  regF_rule, // 608
  regF_rule, // 609
  regD_rule, // 610
  regD_rule, // 611
  regD_rule, // 612
  regD_rule, // 613
  regD_rule, // 614
  regD_rule, // 615
  regD_rule, // 616
  regD_rule, // 617
  regD_rule, // 618
  regD_rule, // 619
  regF_rule, // 620
  regF_rule, // 621
  rRegI_rule, // 622
  rRegL_rule, // 623
  rRegI_rule, // 624
  rRegL_rule, // 625
  regF_rule, // 626
  regF_rule, // 627
  regD_rule, // 628
  regD_rule, // 629
  regF_rule, // 630
  regF_rule, // 631
  regD_rule, // 632
  regD_rule, // 633
  rRegL_rule, // 634
  rRegL_rule, // 635
  rRegL_rule, // 636
  rRegL_rule, // 637
  rRegL_rule, // 638
  rRegI_rule, // 639
  stackSlotI_rule, // 640
  stackSlotF_rule, // 641
  stackSlotL_rule, // 642
  stackSlotD_rule, // 643
  regD_rule, // 644
  regD_rule, // 645
  regD_rule, // 646
  regD_rule, // 647
  regD_rule, // 648
  regD_rule, // 649
  regD_rule, // 650
  regD_rule, // 651
  regD_rule, // 652
  regD_rule, // 653
  regD_rule, // 654
  regD_rule, // 655
  regD_rule, // 656
  regD_rule, // 657
  regD_rule, // 658
  Universe_rule, // 659
  rcx_RegI_rule, // 660
  rRegI_rule, // 661
  0, // 662
  rRegI_rule, // 663
  0, // 664
  rRegI_rule, // 665
  Universe_rule, // 666
  Universe_rule, // 667
  Universe_rule, // 668
  Universe_rule, // 669
  Universe_rule, // 670
  rdi_RegP_rule, // 671
  Universe_rule, // 672
  Universe_rule, // 673
  Universe_rule, // 674
  Universe_rule, // 675
  Universe_rule, // 676
  Universe_rule, // 677
  Universe_rule, // 678
  Universe_rule, // 679
  Universe_rule, // 680
  Universe_rule, // 681
  Universe_rule, // 682
  Universe_rule, // 683
  Universe_rule, // 684
  rax_RegP_rule, // 685
  Universe_rule, // 686
  Universe_rule, // 687
  // last instruction
  0 // no trailing comma
};

const        int   leftOp[] = {
  0, // 0
  0, // 1
  0, // 2
  0, // 3
  0, // 4
  0, // 5
  0, // 6
  0, // 7
  0, // 8
  0, // 9
  0, // 10
  0, // 11
  0, // 12
  0, // 13
  0, // 14
  0, // 15
  0, // 16
  0, // 17
  0, // 18
  0, // 19
  0, // 20
  0, // 21
  0, // 22
  0, // 23
  0, // 24
  0, // 25
  0, // 26
  0, // 27
  0, // 28
  0, // 29
  0, // 30
  0, // 31
  0, // 32
  0, // 33
  0, // 34
  0, // 35
  0, // 36
  0, // 37
  0, // 38
  0, // 39
  0, // 40
  0, // 41
  0, // 42
  0, // 43
  0, // 44
  0, // 45
  0, // 46
  0, // 47
  0, // 48
  0, // 49
  0, // 50
  0, // 51
  0, // 52
  0, // 53
  0, // 54
  0, // 55
  0, // 56
  0, // 57
  0, // 58
  0, // 59
  0, // 60
  0, // 61
  0, // 62
  0, // 63
  0, // 64
  0, // 65
  0, // 66
  0, // 67
  0, // 68
  0, // 69
  0, // 70
  any_RegP_rule, // 71
  any_RegP_rule, // 72
  _AddP_any_RegP_rRegL_rule, // 73
  any_RegP_rule, // 74
  any_RegP_rule, // 75
  _AddP_any_RegP__LShiftL_rRegL_immI2_rule, // 76
  0, // 77
  0, // 78
  0, // 79
  0, // 80
  0, // 81
  0, // 82
  0, // 83
  // last operand
  0, // 84
  // last operand class
  any_RegP_rule, // 85
  rRegL_rule, // 86
  any_RegP_rule, // 87
  memory_rule, // 88
  memory_rule, // 89
  memory_rule, // 90
  rRegI_rule, // 91
  rRegL_rule, // 92
  _LShiftL_rRegL_immI2_rule, // 93
  cmpOp_rule, // 94
  rRegI_rule, // 95
  cmpOpU_rule, // 96
  rRegI_rule, // 97
  rRegP_rule, // 98
  rRegL_rule, // 99
  rRegL_rule, // 100
  regF_rule, // 101
  regD_rule, // 102
  _LoadI_memory__rule, // 103
  rRegI_rule, // 104
  _LoadI_memory__rule, // 105
  _LoadI_memory__rule, // 106
  _LoadI_memory__rule, // 107
  _LoadL_memory__rule, // 108
  rRegL_rule, // 109
  _LoadL_memory__rule, // 110
  _LoadL_memory__rule, // 111
  _LoadL_memory__rule, // 112
  rax_RegP_rule, // 113
  rax_RegL_rule, // 114
  memory_rule, // 115
  rax_RegI_rule, // 116
  _LoadI_memory__rule, // 117
  _LoadI_memory__rule, // 118
  _LoadL_memory__rule, // 119
  _LoadL_memory__rule, // 120
  immI0_rule, // 121
  immI0_rule, // 122
  immL0_rule, // 123
  _LoadI_memory__rule, // 124
  _LoadI_memory__rule, // 125
  _LoadI_memory__rule, // 126
  _LoadI_memory__rule, // 127
  _LoadI_memory__rule, // 128
  _LoadI_memory__rule, // 129
  _LoadI_memory__rule, // 130
  _LoadI_memory__rule, // 131
  _LoadI_memory__rule, // 132
  _LoadL_memory__rule, // 133
  _LoadL_memory__rule, // 134
  _LoadL_memory__rule, // 135
  _LoadL_memory__rule, // 136
  _LoadL_memory__rule, // 137
  _LoadL_memory__rule, // 138
  _LoadL_memory__rule, // 139
  _LoadL_memory__rule, // 140
  _LoadL_memory__rule, // 141
  rRegI_rule, // 142
  rRegI_rule, // 143
  rRegI_rule, // 144
  rRegI_rule, // 145
  rRegI_rule, // 146
  rRegI_rule, // 147
  no_rcx_RegI_rule, // 148
  immI0_rule, // 149
  no_rcx_RegI_rule, // 150
  immI_32_rule, // 151
  no_rcx_RegI_rule, // 152
  rRegI_rule, // 153
  rRegI_rule, // 154
  no_rcx_RegI_rule, // 155
  no_rcx_RegI_rule, // 156
  no_rcx_RegI_rule, // 157
  rRegL_rule, // 158
  rRegL_rule, // 159
  rRegL_rule, // 160
  rRegL_rule, // 161
  no_rcx_RegL_rule, // 162
  no_rcx_RegL_rule, // 163
  immI_64_rule, // 164
  no_rcx_RegL_rule, // 165
  rRegL_rule, // 166
  rRegL_rule, // 167
  no_rcx_RegL_rule, // 168
  no_rcx_RegL_rule, // 169
  no_rcx_RegL_rule, // 170
  rRegI_rule, // 171
  rRegI_rule, // 172
  _LoadI_memory__rule, // 173
  rRegI_rule, // 174
  _LoadI_memory__rule, // 175
  _LoadI_memory__rule, // 176
  rRegI_rule, // 177
  _LoadI_memory__rule, // 178
  _LoadI_memory__rule, // 179
  rRegI_rule, // 180
  _LoadI_memory__rule, // 181
  _LoadL_memory__rule, // 182
  rRegL_rule, // 183
  _LoadL_memory__rule, // 184
  _LoadL_memory__rule, // 185
  rRegL_rule, // 186
  _LoadL_memory__rule, // 187
  _LoadL_memory__rule, // 188
  rRegL_rule, // 189
  _LoadL_memory__rule, // 190
  rRegI_rule, // 191
  _CmpLTMask_rRegI_rRegI_rule, // 192
  rRegI_rule, // 193
  rRegI_rule, // 194
  memory_rule, // 195
  memory_rule, // 196
  regF_rule, // 197
  _ConvF2D_regF__rule, // 198
  _LoadF_memory__rule, // 199
  _ConvF2D__LoadF_memory___rule, // 200
  immF_rule, // 201
  _ConvF2D_immF__rule, // 202
  rRegI_rule, // 203
  _LoadI_memory__rule, // 204
  rRegI_rule, // 205
  memory_rule, // 206
  rRegL_rule, // 207
  rsi_RegP_rule, // 208
  // last internally defined operand
  rRegI_rule, // 209
  rRegL_rule, // 210
  rRegP_rule, // 211
  regF_rule, // 212
  regD_rule, // 213
  indOffset8_rule, // 214
  indOffset32_rule, // 215
  indIndexOffset_rule, // 216
  indIndexScale_rule, // 217
  indIndexScaleOffset_rule, // 218
  immI_rule, // 219
  immI0_rule, // 220
  immL_rule, // 221
  immL0_rule, // 222
  immUL32_rule, // 223
  immL32_rule, // 224
  immP_rule, // 225
  immP0_rule, // 226
  immP31_rule, // 227
  immF_rule, // 228
  immF0_rule, // 229
  immD_rule, // 230
  immD0_rule, // 231
  stackSlotI_rule, // 232
  stackSlotL_rule, // 233
  stackSlotP_rule, // 234
  stackSlotF_rule, // 235
  stackSlotD_rule, // 236
  rRegL_rule, // 237
  rRegP_rule, // 238
  rRegP_rule, // 239
  rRegP_rule, // 240
  memory_rule, // 241
  _StoreLConditional_memory__Binary_rax_RegL_rRegL_rule, // 242
  regF_rule, // 243
  regF_rule, // 244
  regF_rule, // 245
  regD_rule, // 246
  regD_rule, // 247
  regD_rule, // 248
  regF_rule, // 249
  regD_rule, // 250
  regF_rule, // 251
  regD_rule, // 252
  regF_rule, // 253
  regD_rule, // 254
  stackSlotF_rule, // 255
  stackSlotI_rule, // 256
  stackSlotD_rule, // 257
  stackSlotL_rule, // 258
  stackSlotL_rule, // 259
  regF_rule, // 260
  regD_rule, // 261
  rRegI_rule, // 262
  rRegL_rule, // 263
  rRegI_rule, // 264
  rRegI_rule, // 265
  rRegI_rule, // 266
  rRegI_rule, // 267
  _AndI_rRegI_immI_rule, // 268
  _AndI_rRegI__LoadI_memory__rule, // 269
  _AndI__LoadI_memory__rRegI_rule, // 270
  rRegI_rule, // 271
  rRegI_rule, // 272
  rRegI_rule, // 273
  rRegI_rule, // 274
  rRegP_rule, // 275
  rRegP_rule, // 276
  rRegP_rule, // 277
  rRegP_rule, // 278
  _LoadP_memory__rule, // 279
  rRegL_rule, // 280
  rRegL_rule, // 281
  rRegL_rule, // 282
  rRegL_rule, // 283
  _AndL_rRegL_immL32_rule, // 284
  _AndL_rRegL__LoadL_memory__rule, // 285
  _AndL__LoadL_memory__rRegL_rule, // 286
  _PartialSubtypeCheck_rsi_RegP_rax_RegP_rule, // 287
  rRegP_rule, // 288
  rRegP_rule, // 289
  0, // 290
  0, // 291
  memory_rule, // 292
  _LoadB_memory__rule, // 293
  memory_rule, // 294
  memory_rule, // 295
  memory_rule, // 296
  memory_rule, // 297
  memory_rule, // 298
  memory_rule, // 299
  memory_rule, // 300
  memory_rule, // 301
  memory_rule, // 302
  memory_rule, // 303
  memory_rule, // 304
  memory_rule, // 305
  memory_rule, // 306
  memory_rule, // 307
  memory_rule, // 308
  memory_rule, // 309
  memory_rule, // 310
  memory_rule, // 311
  memory_rule, // 312
  memory_rule, // 313
  memory_rule, // 314
  memory_rule, // 315
  memory_rule, // 316
  memory_rule, // 317
  memory_rule, // 318
  memory_rule, // 319
  memory_rule, // 320
  memory_rule, // 321
  memory_rule, // 322
  memory_rule, // 323
  memory_rule, // 324
  memory_rule, // 325
  memory_rule, // 326
  memory_rule, // 327
  memory_rule, // 328
  memory_rule, // 329
  memory_rule, // 330
  memory_rule, // 331
  memory_rule, // 332
  memory_rule, // 333
  memory_rule, // 334
  memory_rule, // 335
  rRegI_rule, // 336
  rRegL_rule, // 337
  _LoadI_memory__rule, // 338
  _LoadL_memory__rule, // 339
  memory_rule, // 340
  memory_rule, // 341
  0, // 342
  0, // 343
  0, // 344
  0, // 345
  0, // 346
  0, // 347
  _LShiftL_rRegL_immI2_rule, // 348
  _AddL__LShiftL_rRegL_immI2_immL32_rule, // 349
  rRegL_rule, // 350
  _Binary_cmpOp_rFlagsReg_rule, // 351
  _Binary_cmpOpU_rFlagsRegU_rule, // 352
  _Binary_cmpOp_rFlagsReg_rule, // 353
  _Binary_cmpOpU_rFlagsRegU_rule, // 354
  _Binary_cmpOp_rFlagsReg_rule, // 355
  _Binary_cmpOpU_rFlagsRegU_rule, // 356
  _Binary_cmpOp_rFlagsReg_rule, // 357
  _Binary_cmpOp_rFlagsReg_rule, // 358
  _Binary_cmpOpU_rFlagsRegU_rule, // 359
  _Binary_cmpOpU_rFlagsRegU_rule, // 360
  _Binary_cmpOp_rFlagsReg_rule, // 361
  _Binary_cmpOpU_rFlagsRegU_rule, // 362
  _Binary_cmpOp_rFlagsReg_rule, // 363
  _Binary_cmpOpU_rFlagsRegU_rule, // 364
  rRegI_rule, // 365
  rRegI_rule, // 366
  rRegI_rule, // 367
  _LoadI_memory__rule, // 368
  memory_rule, // 369
  memory_rule, // 370
  memory_rule, // 371
  rRegI_rule, // 372
  memory_rule, // 373
  rRegI_rule, // 374
  memory_rule, // 375
  rRegI_rule, // 376
  rRegL_rule, // 377
  rRegL_rule, // 378
  rRegL_rule, // 379
  _LoadL_memory__rule, // 380
  memory_rule, // 381
  memory_rule, // 382
  memory_rule, // 383
  rRegI_rule, // 384
  memory_rule, // 385
  rRegL_rule, // 386
  memory_rule, // 387
  rRegL_rule, // 388
  rRegP_rule, // 389
  rRegP_rule, // 390
  rRegP_rule, // 391
  rRegI_rule, // 392
  memory_rule, // 393
  memory_rule, // 394
  memory_rule, // 395
  memory_rule, // 396
  memory_rule, // 397
  memory_rule, // 398
  rRegI_rule, // 399
  rRegI_rule, // 400
  rRegI_rule, // 401
  memory_rule, // 402
  memory_rule, // 403
  rRegL_rule, // 404
  rRegI_rule, // 405
  rRegL_rule, // 406
  memory_rule, // 407
  memory_rule, // 408
  rRegP_rule, // 409
  immI0_rule, // 410
  memory_rule, // 411
  immL0_rule, // 412
  memory_rule, // 413
  rRegI_rule, // 414
  rRegI_rule, // 415
  rRegI_rule, // 416
  _LoadI_memory__rule, // 417
  _LoadI_memory__rule, // 418
  rRegL_rule, // 419
  rRegL_rule, // 420
  rRegL_rule, // 421
  _LoadL_memory__rule, // 422
  _LoadL_memory__rule, // 423
  rax_RegI_rule, // 424
  rax_RegL_rule, // 425
  rax_RegI_rule, // 426
  rax_RegL_rule, // 427
  0, // 428
  0, // 429
  0, // 430
  0, // 431
  no_rax_RegL_rule, // 432
  rax_RegI_rule, // 433
  rax_RegL_rule, // 434
  rRegI_rule, // 435
  memory_rule, // 436
  rRegI_rule, // 437
  memory_rule, // 438
  rRegI_rule, // 439
  memory_rule, // 440
  rRegI_rule, // 441
  memory_rule, // 442
  rRegI_rule, // 443
  memory_rule, // 444
  rRegI_rule, // 445
  memory_rule, // 446
  rRegI_rule, // 447
  memory_rule, // 448
  rRegI_rule, // 449
  memory_rule, // 450
  rRegI_rule, // 451
  memory_rule, // 452
  rRegL_rule, // 453
  memory_rule, // 454
  rRegL_rule, // 455
  memory_rule, // 456
  rRegL_rule, // 457
  memory_rule, // 458
  rRegL_rule, // 459
  memory_rule, // 460
  rRegL_rule, // 461
  memory_rule, // 462
  rRegL_rule, // 463
  memory_rule, // 464
  rRegL_rule, // 465
  memory_rule, // 466
  rRegL_rule, // 467
  memory_rule, // 468
  rRegL_rule, // 469
  memory_rule, // 470
  _LShiftI_rRegI_immI_24_rule, // 471
  _LShiftI_rRegI_immI_16_rule, // 472
  0, // 473
  0, // 474
  0, // 475
  _LShiftI_rRegI_immI1_rule, // 476
  _URShiftI_rRegI_immI_M1_rule, // 477
  _LShiftI_rRegI_immI8_rule, // 478
  _URShiftI_rRegI_immI8_rule, // 479
  _LShiftI_no_rcx_RegI_rcx_RegI_rule, // 480
  _URShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule, // 481
  _LShiftI_no_rcx_RegI_rcx_RegI_rule, // 482
  _URShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule, // 483
  0, // 484
  0, // 485
  0, // 486
  _URShiftI_rRegI_immI1_rule, // 487
  _LShiftI_rRegI_immI_M1_rule, // 488
  _URShiftI_rRegI_immI8_rule, // 489
  _LShiftI_rRegI_immI8_rule, // 490
  _URShiftI_no_rcx_RegI_rcx_RegI_rule, // 491
  _LShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule, // 492
  _URShiftI_no_rcx_RegI_rcx_RegI_rule, // 493
  _LShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule, // 494
  0, // 495
  0, // 496
  0, // 497
  _LShiftL_rRegL_immI1_rule, // 498
  _URShiftL_rRegL_immI_M1_rule, // 499
  _LShiftL_rRegL_immI8_rule, // 500
  _URShiftL_rRegL_immI8_rule, // 501
  _LShiftL_no_rcx_RegL_rcx_RegI_rule, // 502
  _URShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule, // 503
  _LShiftL_no_rcx_RegL_rcx_RegI_rule, // 504
  _URShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule, // 505
  0, // 506
  0, // 507
  0, // 508
  _URShiftL_rRegL_immI1_rule, // 509
  _LShiftL_rRegL_immI_M1_rule, // 510
  _URShiftL_rRegL_immI8_rule, // 511
  _LShiftL_rRegL_immI8_rule, // 512
  _URShiftL_no_rcx_RegL_rcx_RegI_rule, // 513
  _LShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule, // 514
  _URShiftL_no_rcx_RegL_rcx_RegI_rule, // 515
  _LShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule, // 516
  rRegI_rule, // 517
  rRegI_rule, // 518
  _AndI_rRegI_immI_255_rule, // 519
  rRegI_rule, // 520
  _AndI_rRegI_immI_65535_rule, // 521
  rRegI_rule, // 522
  rRegI_rule, // 523
  _LoadI_memory__rule, // 524
  memory_rule, // 525
  memory_rule, // 526
  memory_rule, // 527
  rRegI_rule, // 528
  rRegI_rule, // 529
  rRegI_rule, // 530
  _LoadI_memory__rule, // 531
  memory_rule, // 532
  memory_rule, // 533
  memory_rule, // 534
  rRegI_rule, // 535
  rRegI_rule, // 536
  rRegI_rule, // 537
  _LoadI_memory__rule, // 538
  memory_rule, // 539
  memory_rule, // 540
  memory_rule, // 541
  rRegL_rule, // 542
  rRegL_rule, // 543
  rRegI_rule, // 544
  rRegL_rule, // 545
  rRegL_rule, // 546
  _LoadL_memory__rule, // 547
  memory_rule, // 548
  memory_rule, // 549
  memory_rule, // 550
  rRegL_rule, // 551
  rRegL_rule, // 552
  rRegL_rule, // 553
  _LoadL_memory__rule, // 554
  memory_rule, // 555
  memory_rule, // 556
  memory_rule, // 557
  rRegL_rule, // 558
  rRegL_rule, // 559
  rRegL_rule, // 560
  _LoadL_memory__rule, // 561
  memory_rule, // 562
  memory_rule, // 563
  memory_rule, // 564
  rRegI_rule, // 565
  rRegP_rule, // 566
  rRegI_rule, // 567
  rRegI_rule, // 568
  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule, // 569
  _SubI_rRegI_rRegI_rule, // 570
  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule, // 571
  _SubI_rRegI_rRegI_rule, // 572
  regF_rule, // 573
  regF_rule, // 574
  regF_rule, // 575
  regD_rule, // 576
  regD_rule, // 577
  regD_rule, // 578
  regF_rule, // 579
  regF_rule, // 580
  _LoadF_memory__rule, // 581
  regF_rule, // 582
  regD_rule, // 583
  regD_rule, // 584
  _LoadD_memory__rule, // 585
  regD_rule, // 586
  regF_rule, // 587
  regF_rule, // 588
  regF_rule, // 589
  regD_rule, // 590
  regD_rule, // 591
  regD_rule, // 592
  regF_rule, // 593
  regF_rule, // 594
  _LoadF_memory__rule, // 595
  regF_rule, // 596
  regD_rule, // 597
  regD_rule, // 598
  _LoadD_memory__rule, // 599
  regD_rule, // 600
  regF_rule, // 601
  regF_rule, // 602
  regF_rule, // 603
  regD_rule, // 604
  regD_rule, // 605
  regD_rule, // 606
  _SqrtD__ConvF2D_regF___rule, // 607
  _SqrtD__ConvF2D__LoadF_memory____rule, // 608
  _SqrtD__ConvF2D_immF___rule, // 609
  regD_rule, // 610
  _LoadD_memory__rule, // 611
  immD_rule, // 612
  regD_rule, // 613
  regD_rule, // 614
  regD_rule, // 615
  regD_rule, // 616
  regD_rule, // 617
  regF_rule, // 618
  _LoadF_memory__rule, // 619
  regD_rule, // 620
  _LoadD_memory__rule, // 621
  regF_rule, // 622
  regF_rule, // 623
  regD_rule, // 624
  regD_rule, // 625
  rRegI_rule, // 626
  _LoadI_memory__rule, // 627
  rRegI_rule, // 628
  _LoadI_memory__rule, // 629
  rRegL_rule, // 630
  _LoadL_memory__rule, // 631
  rRegL_rule, // 632
  _LoadL_memory__rule, // 633
  rRegI_rule, // 634
  _LoadI_memory__rule, // 635
  _ConvI2L_rRegI__rule, // 636
  _ConvI2L__LoadI_memory___rule, // 637
  rRegL_rule, // 638
  rRegL_rule, // 639
  regF_rule, // 640
  rRegI_rule, // 641
  regD_rule, // 642
  rRegL_rule, // 643
  regD_rule, // 644
  rRegI_rule, // 645
  immI0_rule, // 646
  regD_rule, // 647
  rRegI_rule, // 648
  immI0_rule, // 649
  regD_rule, // 650
  rRegI_rule, // 651
  immI0_rule, // 652
  regD_rule, // 653
  rRegI_rule, // 654
  immI0_rule, // 655
  regD_rule, // 656
  regF_rule, // 657
  immF0_rule, // 658
  rcx_RegL_rule, // 659
  rdi_RegP_rule, // 660
  rRegL_rule, // 661
  0, // 662
  rRegI_rule, // 663
  0, // 664
  rRegI_rule, // 665
  0, // 666
  cmpOp_rule, // 667
  cmpOp_rule, // 668
  cmpOpU_rule, // 669
  cmpOpU_rule, // 670
  rsi_RegP_rule, // 671
  0, // 672
  cmpOp_rule, // 673
  cmpOp_rule, // 674
  cmpOpU_rule, // 675
  cmpOpU_rule, // 676
  0, // 677
  0, // 678
  0, // 679
  0, // 680
  0, // 681
  0, // 682
  no_rbp_RegP_rule, // 683
  no_rbp_RegP_rule, // 684
  0, // 685
  0, // 686
  0, // 687
  // last instruction
  0 // no trailing comma
};

const        int   rightOp[] = {
  0, // 0
  0, // 1
  0, // 2
  0, // 3
  0, // 4
  0, // 5
  0, // 6
  0, // 7
  0, // 8
  0, // 9
  0, // 10
  0, // 11
  0, // 12
  0, // 13
  0, // 14
  0, // 15
  0, // 16
  0, // 17
  0, // 18
  0, // 19
  0, // 20
  0, // 21
  0, // 22
  0, // 23
  0, // 24
  0, // 25
  0, // 26
  0, // 27
  0, // 28
  0, // 29
  0, // 30
  0, // 31
  0, // 32
  0, // 33
  0, // 34
  0, // 35
  0, // 36
  0, // 37
  0, // 38
  0, // 39
  0, // 40
  0, // 41
  0, // 42
  0, // 43
  0, // 44
  0, // 45
  0, // 46
  0, // 47
  0, // 48
  0, // 49
  0, // 50
  0, // 51
  0, // 52
  0, // 53
  0, // 54
  0, // 55
  0, // 56
  0, // 57
  0, // 58
  0, // 59
  0, // 60
  0, // 61
  0, // 62
  0, // 63
  0, // 64
  0, // 65
  0, // 66
  0, // 67
  0, // 68
  0, // 69
  0, // 70
  immL8_rule, // 71
  immL32_rule, // 72
  immL32_rule, // 73
  rRegL_rule, // 74
  _LShiftL_rRegL_immI2_rule, // 75
  immL32_rule, // 76
  0, // 77
  0, // 78
  0, // 79
  0, // 80
  0, // 81
  0, // 82
  0, // 83
  // last operand
  0, // 84
  // last operand class
  rRegL_rule, // 85
  immI2_rule, // 86
  _LShiftL_rRegL_immI2_rule, // 87
  0, // 88
  0, // 89
  0, // 90
  0, // 91
  0, // 92
  immL32_rule, // 93
  rFlagsReg_rule, // 94
  rRegI_rule, // 95
  rFlagsRegU_rule, // 96
  _LoadI_memory__rule, // 97
  rRegP_rule, // 98
  rRegL_rule, // 99
  _LoadL_memory__rule, // 100
  regF_rule, // 101
  regD_rule, // 102
  rRegI_rule, // 103
  _LoadI_memory__rule, // 104
  immI_rule, // 105
  immI1_rule, // 106
  immI_M1_rule, // 107
  rRegL_rule, // 108
  _LoadL_memory__rule, // 109
  immL32_rule, // 110
  immL1_rule, // 111
  immL_M1_rule, // 112
  rRegP_rule, // 113
  rRegL_rule, // 114
  _Binary_rax_RegL_rRegL_rule, // 115
  rRegI_rule, // 116
  rRegI_rule, // 117
  immI_rule, // 118
  rRegL_rule, // 119
  immL32_rule, // 120
  rRegI_rule, // 121
  _LoadI_memory__rule, // 122
  _LoadL_memory__rule, // 123
  immI1_rule, // 124
  immI8_rule, // 125
  rcx_RegI_rule, // 126
  immI1_rule, // 127
  immI8_rule, // 128
  rcx_RegI_rule, // 129
  immI1_rule, // 130
  immI8_rule, // 131
  rcx_RegI_rule, // 132
  immI1_rule, // 133
  immI8_rule, // 134
  rcx_RegI_rule, // 135
  immI1_rule, // 136
  immI8_rule, // 137
  rcx_RegI_rule, // 138
  immI1_rule, // 139
  immI8_rule, // 140
  rcx_RegI_rule, // 141
  immI_24_rule, // 142
  immI_16_rule, // 143
  immI1_rule, // 144
  immI_M1_rule, // 145
  immI8_rule, // 146
  immI8_rule, // 147
  rcx_RegI_rule, // 148
  rcx_RegI_rule, // 149
  _SubI_immI0_rcx_RegI_rule, // 150
  rcx_RegI_rule, // 151
  _SubI_immI_32_rcx_RegI_rule, // 152
  immI1_rule, // 153
  immI_M1_rule, // 154
  rcx_RegI_rule, // 155
  _SubI_immI0_rcx_RegI_rule, // 156
  _SubI_immI_32_rcx_RegI_rule, // 157
  immI1_rule, // 158
  immI_M1_rule, // 159
  immI8_rule, // 160
  immI8_rule, // 161
  rcx_RegI_rule, // 162
  _SubI_immI0_rcx_RegI_rule, // 163
  rcx_RegI_rule, // 164
  _SubI_immI_64_rcx_RegI_rule, // 165
  immI1_rule, // 166
  immI_M1_rule, // 167
  rcx_RegI_rule, // 168
  _SubI_immI0_rcx_RegI_rule, // 169
  _SubI_immI_64_rcx_RegI_rule, // 170
  immI_255_rule, // 171
  immI_65535_rule, // 172
  rRegI_rule, // 173
  _LoadI_memory__rule, // 174
  immI_rule, // 175
  rRegI_rule, // 176
  _LoadI_memory__rule, // 177
  immI_rule, // 178
  rRegI_rule, // 179
  _LoadI_memory__rule, // 180
  immI_rule, // 181
  rRegL_rule, // 182
  _LoadL_memory__rule, // 183
  immL32_rule, // 184
  rRegL_rule, // 185
  _LoadL_memory__rule, // 186
  immL32_rule, // 187
  rRegL_rule, // 188
  _LoadL_memory__rule, // 189
  immL32_rule, // 190
  rRegI_rule, // 191
  rRegI_rule, // 192
  rRegI_rule, // 193
  _CmpLTMask_rRegI_rRegI_rule, // 194
  0, // 195
  0, // 196
  0, // 197
  0, // 198
  0, // 199
  0, // 200
  0, // 201
  0, // 202
  0, // 203
  0, // 204
  immI_rule, // 205
  0, // 206
  immL32_rule, // 207
  rax_RegP_rule, // 208
  // last internally defined operand
  0, // 209
  0, // 210
  0, // 211
  0, // 212
  0, // 213
  0, // 214
  0, // 215
  0, // 216
  0, // 217
  0, // 218
  0, // 219
  0, // 220
  0, // 221
  0, // 222
  0, // 223
  0, // 224
  0, // 225
  0, // 226
  0, // 227
  0, // 228
  0, // 229
  0, // 230
  0, // 231
  0, // 232
  0, // 233
  0, // 234
  0, // 235
  0, // 236
  0, // 237
  0, // 238
  0, // 239
  0, // 240
  _Binary_rax_RegP_rRegP_rule, // 241
  immI0_rule, // 242
  regF_rule, // 243
  _LoadF_memory__rule, // 244
  immF_rule, // 245
  regD_rule, // 246
  _LoadD_memory__rule, // 247
  immD_rule, // 248
  0, // 249
  0, // 250
  0, // 251
  0, // 252
  0, // 253
  0, // 254
  0, // 255
  0, // 256
  0, // 257
  0, // 258
  0, // 259
  0, // 260
  0, // 261
  0, // 262
  0, // 263
  rRegI_rule, // 264
  immI_rule, // 265
  _LoadI_memory__rule, // 266
  immI0_rule, // 267
  immI0_rule, // 268
  immI0_rule, // 269
  immI0_rule, // 270
  rRegI_rule, // 271
  immI_rule, // 272
  _LoadI_memory__rule, // 273
  immI0_rule, // 274
  rRegP_rule, // 275
  _LoadP_memory__rule, // 276
  _LoadP_memory__rule, // 277
  immP0_rule, // 278
  immP0_rule, // 279
  rRegL_rule, // 280
  immL32_rule, // 281
  _LoadL_memory__rule, // 282
  immL0_rule, // 283
  immL0_rule, // 284
  immL0_rule, // 285
  immL0_rule, // 286
  immP0_rule, // 287
  rRegP_rule, // 288
  rax_RegP_rule, // 289
  0, // 290
  0, // 291
  0, // 292
  immI_255_rule, // 293
  0, // 294
  0, // 295
  0, // 296
  0, // 297
  0, // 298
  0, // 299
  0, // 300
  0, // 301
  0, // 302
  0, // 303
  0, // 304
  0, // 305
  0, // 306
  0, // 307
  0, // 308
  0, // 309
  0, // 310
  0, // 311
  0, // 312
  0, // 313
  0, // 314
  0, // 315
  0, // 316
  rRegI_rule, // 317
  rRegI_rule, // 318
  rRegI_rule, // 319
  rRegL_rule, // 320
  any_RegP_rule, // 321
  immP31_rule, // 322
  immI_rule, // 323
  immL32_rule, // 324
  immI16_rule, // 325
  immI8_rule, // 326
  regD_rule, // 327
  regD_rule, // 328
  regD_rule, // 329
  immI0_rule, // 330
  regD_rule, // 331
  regF_rule, // 332
  immF_rule, // 333
  regD_rule, // 334
  immD0_rule, // 335
  0, // 336
  0, // 337
  0, // 338
  0, // 339
  _ReverseBytesI_rRegI__rule, // 340
  _ReverseBytesL_rRegL__rule, // 341
  0, // 342
  0, // 343
  0, // 344
  0, // 345
  0, // 346
  0, // 347
  0, // 348
  0, // 349
  0, // 350
  _Binary_rRegI_rRegI_rule, // 351
  _Binary_rRegI_rRegI_rule, // 352
  _Binary_rRegI__LoadI_memory__rule, // 353
  _Binary_rRegI__LoadI_memory__rule, // 354
  _Binary_rRegP_rRegP_rule, // 355
  _Binary_rRegP_rRegP_rule, // 356
  _Binary_rRegL_rRegL_rule, // 357
  _Binary_rRegL__LoadL_memory__rule, // 358
  _Binary_rRegL_rRegL_rule, // 359
  _Binary_rRegL__LoadL_memory__rule, // 360
  _Binary_regF_regF_rule, // 361
  _Binary_regF_regF_rule, // 362
  _Binary_regD_regD_rule, // 363
  _Binary_regD_regD_rule, // 364
  rRegI_rule, // 365
  immI_rule, // 366
  _LoadI_memory__rule, // 367
  rRegI_rule, // 368
  _AddI__LoadI_memory__rRegI_rule, // 369
  _AddI_rRegI__LoadI_memory__rule, // 370
  _AddI__LoadI_memory__immI_rule, // 371
  immI1_rule, // 372
  _AddI__LoadI_memory__immI1_rule, // 373
  immI_M1_rule, // 374
  _AddI__LoadI_memory__immI_M1_rule, // 375
  immI_rule, // 376
  rRegL_rule, // 377
  immL32_rule, // 378
  _LoadL_memory__rule, // 379
  rRegL_rule, // 380
  _AddL__LoadL_memory__rRegL_rule, // 381
  _AddL_rRegL__LoadL_memory__rule, // 382
  _AddL__LoadL_memory__immL32_rule, // 383
  immL1_rule, // 384
  _AddL__LoadL_memory__immL1_rule, // 385
  immL_M1_rule, // 386
  _AddL__LoadL_memory__immL_M1_rule, // 387
  immL32_rule, // 388
  rRegL_rule, // 389
  immL32_rule, // 390
  immL32_rule, // 391
  0, // 392
  0, // 393
  0, // 394
  _Binary_rax_RegL_rRegL_rule, // 395
  _Binary_rax_RegP_rRegP_rule, // 396
  _Binary_rax_RegL_rRegL_rule, // 397
  _Binary_rax_RegI_rRegI_rule, // 398
  rRegI_rule, // 399
  immI_rule, // 400
  _LoadI_memory__rule, // 401
  _SubI__LoadI_memory__rRegI_rule, // 402
  _SubI__LoadI_memory__immI_rule, // 403
  rRegL_rule, // 404
  immL32_rule, // 405
  _LoadL_memory__rule, // 406
  _SubL__LoadL_memory__rRegL_rule, // 407
  _SubL__LoadL_memory__immL32_rule, // 408
  _SubI_immI0_rRegI_rule, // 409
  rRegI_rule, // 410
  _SubI_immI0__LoadI_memory__rule, // 411
  rRegL_rule, // 412
  _SubL_immL0__LoadL_memory__rule, // 413
  rRegI_rule, // 414
  immI_rule, // 415
  _LoadI_memory__rule, // 416
  rRegI_rule, // 417
  immI_rule, // 418
  rRegL_rule, // 419
  immL32_rule, // 420
  _LoadL_memory__rule, // 421
  rRegL_rule, // 422
  immL32_rule, // 423
  no_rax_rdx_RegI_rule, // 424
  no_rax_rdx_RegL_rule, // 425
  no_rax_rdx_RegI_rule, // 426
  no_rax_rdx_RegL_rule, // 427
  0, // 428
  0, // 429
  0, // 430
  0, // 431
  immL10_rule, // 432
  no_rax_rdx_RegI_rule, // 433
  no_rax_rdx_RegL_rule, // 434
  immI1_rule, // 435
  _LShiftI__LoadI_memory__immI1_rule, // 436
  immI8_rule, // 437
  _LShiftI__LoadI_memory__immI8_rule, // 438
  rcx_RegI_rule, // 439
  _LShiftI__LoadI_memory__rcx_RegI_rule, // 440
  immI1_rule, // 441
  _RShiftI__LoadI_memory__immI1_rule, // 442
  immI8_rule, // 443
  _RShiftI__LoadI_memory__immI8_rule, // 444
  rcx_RegI_rule, // 445
  _RShiftI__LoadI_memory__rcx_RegI_rule, // 446
  immI1_rule, // 447
  _URShiftI__LoadI_memory__immI1_rule, // 448
  immI8_rule, // 449
  _URShiftI__LoadI_memory__immI8_rule, // 450
  rcx_RegI_rule, // 451
  _URShiftI__LoadI_memory__rcx_RegI_rule, // 452
  immI1_rule, // 453
  _LShiftL__LoadL_memory__immI1_rule, // 454
  immI8_rule, // 455
  _LShiftL__LoadL_memory__immI8_rule, // 456
  rcx_RegI_rule, // 457
  _LShiftL__LoadL_memory__rcx_RegI_rule, // 458
  immI1_rule, // 459
  _RShiftL__LoadL_memory__immI1_rule, // 460
  immI8_rule, // 461
  _RShiftL__LoadL_memory__immI8_rule, // 462
  rcx_RegI_rule, // 463
  _RShiftL__LoadL_memory__rcx_RegI_rule, // 464
  immI1_rule, // 465
  _URShiftL__LoadL_memory__immI1_rule, // 466
  immI8_rule, // 467
  _URShiftL__LoadL_memory__immI8_rule, // 468
  rcx_RegI_rule, // 469
  _URShiftL__LoadL_memory__rcx_RegI_rule, // 470
  immI_24_rule, // 471
  immI_16_rule, // 472
  0, // 473
  0, // 474
  0, // 475
  _URShiftI_rRegI_immI_M1_rule, // 476
  _LShiftI_rRegI_immI1_rule, // 477
  _URShiftI_rRegI_immI8_rule, // 478
  _LShiftI_rRegI_immI8_rule, // 479
  _URShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule, // 480
  _LShiftI_no_rcx_RegI_rcx_RegI_rule, // 481
  _URShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule, // 482
  _LShiftI_no_rcx_RegI_rcx_RegI_rule, // 483
  0, // 484
  0, // 485
  0, // 486
  _LShiftI_rRegI_immI_M1_rule, // 487
  _URShiftI_rRegI_immI1_rule, // 488
  _LShiftI_rRegI_immI8_rule, // 489
  _URShiftI_rRegI_immI8_rule, // 490
  _LShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule, // 491
  _URShiftI_no_rcx_RegI_rcx_RegI_rule, // 492
  _LShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule, // 493
  _URShiftI_no_rcx_RegI_rcx_RegI_rule, // 494
  0, // 495
  0, // 496
  0, // 497
  _URShiftL_rRegL_immI_M1_rule, // 498
  _LShiftL_rRegL_immI1_rule, // 499
  _URShiftL_rRegL_immI8_rule, // 500
  _LShiftL_rRegL_immI8_rule, // 501
  _URShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule, // 502
  _LShiftL_no_rcx_RegL_rcx_RegI_rule, // 503
  _URShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule, // 504
  _LShiftL_no_rcx_RegL_rcx_RegI_rule, // 505
  0, // 506
  0, // 507
  0, // 508
  _LShiftL_rRegL_immI_M1_rule, // 509
  _URShiftL_rRegL_immI1_rule, // 510
  _LShiftL_rRegL_immI8_rule, // 511
  _URShiftL_rRegL_immI8_rule, // 512
  _LShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule, // 513
  _URShiftL_no_rcx_RegL_rcx_RegI_rule, // 514
  _LShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule, // 515
  _URShiftL_no_rcx_RegL_rcx_RegI_rule, // 516
  rRegI_rule, // 517
  immI_255_rule, // 518
  0, // 519
  immI_65535_rule, // 520
  0, // 521
  immI_rule, // 522
  _LoadI_memory__rule, // 523
  rRegI_rule, // 524
  _AndI__LoadI_memory__rRegI_rule, // 525
  _AndI_rRegI__LoadI_memory__rule, // 526
  _AndI__LoadI_memory__immI_rule, // 527
  rRegI_rule, // 528
  immI_rule, // 529
  _LoadI_memory__rule, // 530
  rRegI_rule, // 531
  _OrI__LoadI_memory__rRegI_rule, // 532
  _OrI_rRegI__LoadI_memory__rule, // 533
  _OrI__LoadI_memory__immI_rule, // 534
  rRegI_rule, // 535
  immI_rule, // 536
  _LoadI_memory__rule, // 537
  rRegI_rule, // 538
  _XorI__LoadI_memory__rRegI_rule, // 539
  _XorI_rRegI__LoadI_memory__rule, // 540
  _XorI__LoadI_memory__immI_rule, // 541
  rRegL_rule, // 542
  immL_255_rule, // 543
  immL_65535_rule, // 544
  immL32_rule, // 545
  _LoadL_memory__rule, // 546
  rRegL_rule, // 547
  _AndL__LoadL_memory__rRegL_rule, // 548
  _AndL_rRegL__LoadL_memory__rule, // 549
  _AndL__LoadL_memory__immL32_rule, // 550
  rRegL_rule, // 551
  immL32_rule, // 552
  _LoadL_memory__rule, // 553
  rRegL_rule, // 554
  _OrL__LoadL_memory__rRegL_rule, // 555
  _OrL_rRegL__LoadL_memory__rule, // 556
  _OrL__LoadL_memory__immL32_rule, // 557
  rRegL_rule, // 558
  immL32_rule, // 559
  _LoadL_memory__rule, // 560
  rRegL_rule, // 561
  _XorL__LoadL_memory__rRegL_rule, // 562
  _XorL_rRegL__LoadL_memory__rule, // 563
  _XorL__LoadL_memory__immL32_rule, // 564
  0, // 565
  0, // 566
  rRegI_rule, // 567
  immI0_rule, // 568
  _SubI_rRegI_rRegI_rule, // 569
  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule, // 570
  _SubI_rRegI_rRegI_rule, // 571
  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule, // 572
  regF_rule, // 573
  _LoadF_memory__rule, // 574
  immF_rule, // 575
  regD_rule, // 576
  _LoadD_memory__rule, // 577
  immD_rule, // 578
  regF_rule, // 579
  _LoadF_memory__rule, // 580
  regF_rule, // 581
  immF_rule, // 582
  regD_rule, // 583
  _LoadD_memory__rule, // 584
  regD_rule, // 585
  immD_rule, // 586
  regF_rule, // 587
  _LoadF_memory__rule, // 588
  immF_rule, // 589
  regD_rule, // 590
  _LoadD_memory__rule, // 591
  immD_rule, // 592
  regF_rule, // 593
  _LoadF_memory__rule, // 594
  regF_rule, // 595
  immF_rule, // 596
  regD_rule, // 597
  _LoadD_memory__rule, // 598
  regD_rule, // 599
  immD_rule, // 600
  regF_rule, // 601
  _LoadF_memory__rule, // 602
  immF_rule, // 603
  regD_rule, // 604
  _LoadD_memory__rule, // 605
  immD_rule, // 606
  0, // 607
  0, // 608
  0, // 609
  0, // 610
  0, // 611
  0, // 612
  0, // 613
  0, // 614
  0, // 615
  0, // 616
  0, // 617
  0, // 618
  0, // 619
  0, // 620
  0, // 621
  0, // 622
  0, // 623
  0, // 624
  0, // 625
  0, // 626
  0, // 627
  0, // 628
  0, // 629
  0, // 630
  0, // 631
  0, // 632
  0, // 633
  0, // 634
  0, // 635
  immL_32bits_rule, // 636
  immL_32bits_rule, // 637
  immL_32bits_rule, // 638
  0, // 639
  0, // 640
  0, // 641
  0, // 642
  0, // 643
  0, // 644
  0, // 645
  0, // 646
  0, // 647
  0, // 648
  0, // 649
  0, // 650
  0, // 651
  0, // 652
  0, // 653
  0, // 654
  0, // 655
  0, // 656
  0, // 657
  0, // 658
  rdi_RegP_rule, // 659
  rsi_RegP_rule, // 660
  rRegL_rule, // 661
  0, // 662
  rRegI_rule, // 663
  0, // 664
  rRegI_rule, // 665
  0, // 666
  rFlagsReg_rule, // 667
  rFlagsReg_rule, // 668
  rFlagsRegU_rule, // 669
  rFlagsRegU_rule, // 670
  rax_RegP_rule, // 671
  0, // 672
  rFlagsReg_rule, // 673
  rFlagsReg_rule, // 674
  rFlagsRegU_rule, // 675
  rFlagsRegU_rule, // 676
  0, // 677
  0, // 678
  0, // 679
  0, // 680
  0, // 681
  0, // 682
  rbx_RegP_rule, // 683
  rax_RegP_rule, // 684
  0, // 685
  0, // 686
  0, // 687
  // last instruction
  0 // no trailing comma
};

const char        *ruleName[] = {
  "UNIVERSE", // 0
  "LABEL", // 1
  "SREGI", // 2
  "SREGP", // 3
  "SREGF", // 4
  "SREGD", // 5
  "SREGL", // 6
  "METHOD", // 7
  "IMMI", // 8
  "IMMI0", // 9
  "IMMI1", // 10
  "IMMI_M1", // 11
  "IMMI2", // 12
  "IMMI8", // 13
  "IMMI16", // 14
  "IMMI_32", // 15
  "IMMI_64", // 16
  "IMMP", // 17
  "IMMP0", // 18
  "IMMP31", // 19
  "IMML", // 20
  "IMML8", // 21
  "IMMUL32", // 22
  "IMML32", // 23
  "IMML0", // 24
  "IMML1", // 25
  "IMML_M1", // 26
  "IMML10", // 27
  "IMML_127", // 28
  "IMML_32BITS", // 29
  "IMMF0", // 30
  "IMMF", // 31
  "IMMD0", // 32
  "IMMD", // 33
  "IMMI_16", // 34
  "IMMI_24", // 35
  "IMMI_255", // 36
  "IMMI_65535", // 37
  "IMML_255", // 38
  "IMML_65535", // 39
  "RREGI", // 40
  "RAX_REGI", // 41
  "RBX_REGI", // 42
  "RCX_REGI", // 43
  "RDX_REGI", // 44
  "RDI_REGI", // 45
  "NO_RCX_REGI", // 46
  "NO_RAX_RDX_REGI", // 47
  "ANY_REGP", // 48
  "RREGP", // 49
  "NO_RAX_REGP", // 50
  "NO_RBP_REGP", // 51
  "NO_RAX_RBX_REGP", // 52
  "RAX_REGP", // 53
  "RBX_REGP", // 54
  "RSI_REGP", // 55
  "RDI_REGP", // 56
  "RBP_REGP", // 57
  "R15_REGP", // 58
  "RREGL", // 59
  "NO_RAX_RDX_REGL", // 60
  "NO_RAX_REGL", // 61
  "NO_RCX_REGL", // 62
  "RAX_REGL", // 63
  "RCX_REGL", // 64
  "RDX_REGL", // 65
  "RFLAGSREG", // 66
  "RFLAGSREGU", // 67
  "REGF", // 68
  "REGD", // 69
  "INDIRECT", // 70
  "INDOFFSET8", // 71
  "INDOFFSET32", // 72
  "INDINDEXOFFSET", // 73
  "INDINDEX", // 74
  "INDINDEXSCALE", // 75
  "INDINDEXSCALEOFFSET", // 76
  "STACKSLOTP", // 77
  "STACKSLOTI", // 78
  "STACKSLOTF", // 79
  "STACKSLOTD", // 80
  "STACKSLOTL", // 81
  "CMPOP", // 82
  "CMPOPU", // 83
  // last operand
  "MEMORY", // 84
  // last operand class
  "_AddP_any_RegP_rRegL", // 85
  "_LShiftL_rRegL_immI2", // 86
  "_AddP_any_RegP__LShiftL_rRegL_immI2", // 87
  "_LoadB_memory_", // 88
  "_LoadI_memory_", // 89
  "_LoadL_memory_", // 90
  "_ReverseBytesI_rRegI_", // 91
  "_ReverseBytesL_rRegL_", // 92
  "_AddL__LShiftL_rRegL_immI2_immL32", // 93
  "_Binary_cmpOp_rFlagsReg", // 94
  "_Binary_rRegI_rRegI", // 95
  "_Binary_cmpOpU_rFlagsRegU", // 96
  "_Binary_rRegI__LoadI_memory_", // 97
  "_Binary_rRegP_rRegP", // 98
  "_Binary_rRegL_rRegL", // 99
  "_Binary_rRegL__LoadL_memory_", // 100
  "_Binary_regF_regF", // 101
  "_Binary_regD_regD", // 102
  "_AddI__LoadI_memory__rRegI", // 103
  "_AddI_rRegI__LoadI_memory_", // 104
  "_AddI__LoadI_memory__immI", // 105
  "_AddI__LoadI_memory__immI1", // 106
  "_AddI__LoadI_memory__immI_M1", // 107
  "_AddL__LoadL_memory__rRegL", // 108
  "_AddL_rRegL__LoadL_memory_", // 109
  "_AddL__LoadL_memory__immL32", // 110
  "_AddL__LoadL_memory__immL1", // 111
  "_AddL__LoadL_memory__immL_M1", // 112
  "_Binary_rax_RegP_rRegP", // 113
  "_Binary_rax_RegL_rRegL", // 114
  "_StoreLConditional_memory__Binary_rax_RegL_rRegL", // 115
  "_Binary_rax_RegI_rRegI", // 116
  "_SubI__LoadI_memory__rRegI", // 117
  "_SubI__LoadI_memory__immI", // 118
  "_SubL__LoadL_memory__rRegL", // 119
  "_SubL__LoadL_memory__immL32", // 120
  "_SubI_immI0_rRegI", // 121
  "_SubI_immI0__LoadI_memory_", // 122
  "_SubL_immL0__LoadL_memory_", // 123
  "_LShiftI__LoadI_memory__immI1", // 124
  "_LShiftI__LoadI_memory__immI8", // 125
  "_LShiftI__LoadI_memory__rcx_RegI", // 126
  "_RShiftI__LoadI_memory__immI1", // 127
  "_RShiftI__LoadI_memory__immI8", // 128
  "_RShiftI__LoadI_memory__rcx_RegI", // 129
  "_URShiftI__LoadI_memory__immI1", // 130
  "_URShiftI__LoadI_memory__immI8", // 131
  "_URShiftI__LoadI_memory__rcx_RegI", // 132
  "_LShiftL__LoadL_memory__immI1", // 133
  "_LShiftL__LoadL_memory__immI8", // 134
  "_LShiftL__LoadL_memory__rcx_RegI", // 135
  "_RShiftL__LoadL_memory__immI1", // 136
  "_RShiftL__LoadL_memory__immI8", // 137
  "_RShiftL__LoadL_memory__rcx_RegI", // 138
  "_URShiftL__LoadL_memory__immI1", // 139
  "_URShiftL__LoadL_memory__immI8", // 140
  "_URShiftL__LoadL_memory__rcx_RegI", // 141
  "_LShiftI_rRegI_immI_24", // 142
  "_LShiftI_rRegI_immI_16", // 143
  "_LShiftI_rRegI_immI1", // 144
  "_URShiftI_rRegI_immI_M1", // 145
  "_LShiftI_rRegI_immI8", // 146
  "_URShiftI_rRegI_immI8", // 147
  "_LShiftI_no_rcx_RegI_rcx_RegI", // 148
  "_SubI_immI0_rcx_RegI", // 149
  "_URShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI", // 150
  "_SubI_immI_32_rcx_RegI", // 151
  "_URShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI", // 152
  "_URShiftI_rRegI_immI1", // 153
  "_LShiftI_rRegI_immI_M1", // 154
  "_URShiftI_no_rcx_RegI_rcx_RegI", // 155
  "_LShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI", // 156
  "_LShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI", // 157
  "_LShiftL_rRegL_immI1", // 158
  "_URShiftL_rRegL_immI_M1", // 159
  "_LShiftL_rRegL_immI8", // 160
  "_URShiftL_rRegL_immI8", // 161
  "_LShiftL_no_rcx_RegL_rcx_RegI", // 162
  "_URShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI", // 163
  "_SubI_immI_64_rcx_RegI", // 164
  "_URShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI", // 165
  "_URShiftL_rRegL_immI1", // 166
  "_LShiftL_rRegL_immI_M1", // 167
  "_URShiftL_no_rcx_RegL_rcx_RegI", // 168
  "_LShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI", // 169
  "_LShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI", // 170
  "_AndI_rRegI_immI_255", // 171
  "_AndI_rRegI_immI_65535", // 172
  "_AndI__LoadI_memory__rRegI", // 173
  "_AndI_rRegI__LoadI_memory_", // 174
  "_AndI__LoadI_memory__immI", // 175
  "_OrI__LoadI_memory__rRegI", // 176
  "_OrI_rRegI__LoadI_memory_", // 177
  "_OrI__LoadI_memory__immI", // 178
  "_XorI__LoadI_memory__rRegI", // 179
  "_XorI_rRegI__LoadI_memory_", // 180
  "_XorI__LoadI_memory__immI", // 181
  "_AndL__LoadL_memory__rRegL", // 182
  "_AndL_rRegL__LoadL_memory_", // 183
  "_AndL__LoadL_memory__immL32", // 184
  "_OrL__LoadL_memory__rRegL", // 185
  "_OrL_rRegL__LoadL_memory_", // 186
  "_OrL__LoadL_memory__immL32", // 187
  "_XorL__LoadL_memory__rRegL", // 188
  "_XorL_rRegL__LoadL_memory_", // 189
  "_XorL__LoadL_memory__immL32", // 190
  "_CmpLTMask_rRegI_rRegI", // 191
  "_AndI__CmpLTMask_rRegI_rRegI_rRegI", // 192
  "_SubI_rRegI_rRegI", // 193
  "_AndI_rRegI__CmpLTMask_rRegI_rRegI", // 194
  "_LoadF_memory_", // 195
  "_LoadD_memory_", // 196
  "_ConvF2D_regF_", // 197
  "_SqrtD__ConvF2D_regF__", // 198
  "_ConvF2D__LoadF_memory__", // 199
  "_SqrtD__ConvF2D__LoadF_memory___", // 200
  "_ConvF2D_immF_", // 201
  "_SqrtD__ConvF2D_immF__", // 202
  "_ConvI2L_rRegI_", // 203
  "_ConvI2L__LoadI_memory__", // 204
  "_AndI_rRegI_immI", // 205
  "_LoadP_memory_", // 206
  "_AndL_rRegL_immL32", // 207
  "_PartialSubtypeCheck_rsi_RegP_rax_RegP", // 208
  // last internally defined operand
  "storeSSI", // 209
  "storeSSL", // 210
  "storeSSP", // 211
  "storeSSF", // 212
  "storeSSD", // 213
  "leaP8", // 214
  "leaP32", // 215
  "leaPIdxOff", // 216
  "leaPIdxScale", // 217
  "leaPIdxScaleOff", // 218
  "loadConI", // 219
  "loadConI0", // 220
  "loadConL", // 221
  "loadConL0", // 222
  "loadConUL32", // 223
  "loadConL32", // 224
  "loadConP", // 225
  "loadConP0", // 226
  "loadConP31", // 227
  "loadConF", // 228
  "loadConF0", // 229
  "loadConD", // 230
  "loadConD0", // 231
  "loadSSI", // 232
  "loadSSL", // 233
  "loadSSP", // 234
  "loadSSF", // 235
  "loadSSD", // 236
  "castX2P", // 237
  "castP2X", // 238
  "checkCastPP", // 239
  "castPP", // 240
  "storePConditional", // 241
  "storeLConditional_flags", // 242
  "cmpF_cc_reg", // 243
  "cmpF_cc_mem", // 244
  "cmpF_cc_imm", // 245
  "cmpD_cc_reg", // 246
  "cmpD_cc_mem", // 247
  "cmpD_cc_imm", // 248
  "absF_reg", // 249
  "absD_reg", // 250
  "negF_reg", // 251
  "negD_reg", // 252
  "roundFloat_nop", // 253
  "roundDouble_nop", // 254
  "MoveF2I_stack_reg", // 255
  "MoveI2F_stack_reg", // 256
  "MoveD2L_stack_reg", // 257
  "MoveL2D_stack_reg_partial", // 258
  "MoveL2D_stack_reg", // 259
  "MoveF2I_reg_reg", // 260
  "MoveD2L_reg_reg", // 261
  "MoveI2F_reg_reg", // 262
  "MoveL2D_reg_reg", // 263
  "compI_rReg", // 264
  "compI_rReg_imm", // 265
  "compI_rReg_mem", // 266
  "testI_reg", // 267
  "testI_reg_imm", // 268
  "testI_reg_mem", // 269
  "testI_reg_mem_0", // 270
  "compU_rReg", // 271
  "compU_rReg_imm", // 272
  "compU_rReg_mem", // 273
  "testU_reg", // 274
  "compP_rReg", // 275
  "compP_rReg_mem", // 276
  "compP_mem_rReg", // 277
  "testP_reg", // 278
  "testP_reg_mem", // 279
  "compL_rReg", // 280
  "compL_rReg_imm", // 281
  "compL_rReg_mem", // 282
  "testL_reg", // 283
  "testL_reg_imm", // 284
  "testL_reg_mem", // 285
  "testL_reg_mem_0", // 286
  "partialSubtypeCheck_vs_Zero", // 287
  "cmpFastLock", // 288
  "cmpFastUnlock", // 289
  "safePoint_poll", // 290
  "tlsLoadP", // 291
  "loadB", // 292
  "loadUB", // 293
  "loadS", // 294
  "loadC", // 295
  "loadI", // 296
  "loadL", // 297
  "loadRange", // 298
  "loadP", // 299
  "loadKlass", // 300
  "loadF", // 301
  "loadD_partial", // 302
  "loadD", // 303
  "loadA8B", // 304
  "loadA4S", // 305
  "loadA4C", // 306
  "load2IU", // 307
  "loadA2F", // 308
  "prefetchr", // 309
  "prefetchrNTA", // 310
  "prefetchrT0", // 311
  "prefetchrT2", // 312
  "prefetchw", // 313
  "prefetchwNTA", // 314
  "prefetchwT0", // 315
  "prefetchwT2", // 316
  "storeB", // 317
  "storeC", // 318
  "storeI", // 319
  "storeL", // 320
  "storeP", // 321
  "storeImmP", // 322
  "storeImmI", // 323
  "storeImmL", // 324
  "storeImmI16", // 325
  "storeImmB", // 326
  "storeA8B", // 327
  "storeA4C", // 328
  "storeA2I", // 329
  "storeImmCM0", // 330
  "storeA2F", // 331
  "storeF", // 332
  "storeF_imm", // 333
  "storeD", // 334
  "storeD0_imm", // 335
  "bytes_reverse_int", // 336
  "bytes_reverse_long", // 337
  "loadI_reversed", // 338
  "loadL_reversed", // 339
  "storeI_reversed", // 340
  "storeL_reversed", // 341
  "membar_acquire", // 342
  "membar_acquire_lock", // 343
  "membar_release", // 344
  "membar_release_lock", // 345
  "membar_volatile", // 346
  "unnecessary_membar_volatile", // 347
  "jumpXtnd_offset", // 348
  "jumpXtnd_addr", // 349
  "jumpXtnd", // 350
  "cmovI_reg", // 351
  "cmovI_regU", // 352
  "cmovI_mem", // 353
  "cmovI_memU", // 354
  "cmovP_reg", // 355
  "cmovP_regU", // 356
  "cmovL_reg", // 357
  "cmovL_mem", // 358
  "cmovL_regU", // 359
  "cmovL_memU", // 360
  "cmovF_reg", // 361
  "cmovF_regU", // 362
  "cmovD_reg", // 363
  "cmovD_regU", // 364
  "addI_rReg", // 365
  "addI_rReg_imm", // 366
  "addI_rReg_mem", // 367
  "addI_rReg_mem_0", // 368
  "addI_mem_rReg", // 369
  "addI_mem_rReg_0", // 370
  "addI_mem_imm", // 371
  "incI_rReg", // 372
  "incI_mem", // 373
  "decI_rReg", // 374
  "decI_mem", // 375
  "leaI_rReg_immI", // 376
  "addL_rReg", // 377
  "addL_rReg_imm", // 378
  "addL_rReg_mem", // 379
  "addL_rReg_mem_0", // 380
  "addL_mem_rReg", // 381
  "addL_mem_rReg_0", // 382
  "addL_mem_imm", // 383
  "incL_rReg", // 384
  "incL_mem", // 385
  "decL_rReg", // 386
  "decL_mem", // 387
  "leaL_rReg_immL", // 388
  "addP_rReg", // 389
  "addP_rReg_imm", // 390
  "leaP_rReg_imm", // 391
  "castII", // 392
  "loadPLocked", // 393
  "loadLLocked", // 394
  "storeLConditional", // 395
  "compareAndSwapP", // 396
  "compareAndSwapL", // 397
  "compareAndSwapI", // 398
  "subI_rReg", // 399
  "subI_rReg_imm", // 400
  "subI_rReg_mem", // 401
  "subI_mem_rReg", // 402
  "subI_mem_imm", // 403
  "subL_rReg", // 404
  "subL_rReg_imm", // 405
  "subL_rReg_mem", // 406
  "subL_mem_rReg", // 407
  "subL_mem_imm", // 408
  "subP_rReg", // 409
  "negI_rReg", // 410
  "negI_mem", // 411
  "negL_rReg", // 412
  "negL_mem", // 413
  "mulI_rReg", // 414
  "mulI_rReg_imm", // 415
  "mulI_mem", // 416
  "mulI_mem_0", // 417
  "mulI_mem_imm", // 418
  "mulL_rReg", // 419
  "mulL_rReg_imm", // 420
  "mulL_mem", // 421
  "mulL_mem_0", // 422
  "mulL_mem_imm", // 423
  "divI_rReg", // 424
  "divL_rReg", // 425
  "divModI_rReg_divmod", // 426
  "divModL_rReg_divmod", // 427
  "loadConL_0x6666666666666667", // 428
  "mul_hi", // 429
  "sarL_rReg_63", // 430
  "sarL_rReg_2", // 431
  "divL_10", // 432
  "modI_rReg", // 433
  "modL_rReg", // 434
  "salI_rReg_1", // 435
  "salI_mem_1", // 436
  "salI_rReg_imm", // 437
  "salI_mem_imm", // 438
  "salI_rReg_CL", // 439
  "salI_mem_CL", // 440
  "sarI_rReg_1", // 441
  "sarI_mem_1", // 442
  "sarI_rReg_imm", // 443
  "sarI_mem_imm", // 444
  "sarI_rReg_CL", // 445
  "sarI_mem_CL", // 446
  "shrI_rReg_1", // 447
  "shrI_mem_1", // 448
  "shrI_rReg_imm", // 449
  "shrI_mem_imm", // 450
  "shrI_rReg_CL", // 451
  "shrI_mem_CL", // 452
  "salL_rReg_1", // 453
  "salL_mem_1", // 454
  "salL_rReg_imm", // 455
  "salL_mem_imm", // 456
  "salL_rReg_CL", // 457
  "salL_mem_CL", // 458
  "sarL_rReg_1", // 459
  "sarL_mem_1", // 460
  "sarL_rReg_imm", // 461
  "sarL_mem_imm", // 462
  "sarL_rReg_CL", // 463
  "sarL_mem_CL", // 464
  "shrL_rReg_1", // 465
  "shrL_mem_1", // 466
  "shrL_rReg_imm", // 467
  "shrL_mem_imm", // 468
  "shrL_rReg_CL", // 469
  "shrL_mem_CL", // 470
  "i2b", // 471
  "i2s", // 472
  "rolI_rReg_imm1", // 473
  "rolI_rReg_imm8", // 474
  "rolI_rReg_CL", // 475
  "rolI_rReg_i1", // 476
  "rolI_rReg_i1_0", // 477
  "rolI_rReg_i8", // 478
  "rolI_rReg_i8_0", // 479
  "rolI_rReg_Var_C0", // 480
  "rolI_rReg_Var_C0_0", // 481
  "rolI_rReg_Var_C32", // 482
  "rolI_rReg_Var_C32_0", // 483
  "rorI_rReg_imm1", // 484
  "rorI_rReg_imm8", // 485
  "rorI_rReg_CL", // 486
  "rorI_rReg_i1", // 487
  "rorI_rReg_i1_0", // 488
  "rorI_rReg_i8", // 489
  "rorI_rReg_i8_0", // 490
  "rorI_rReg_Var_C0", // 491
  "rorI_rReg_Var_C0_0", // 492
  "rorI_rReg_Var_C32", // 493
  "rorI_rReg_Var_C32_0", // 494
  "rolL_rReg_imm1", // 495
  "rolL_rReg_imm8", // 496
  "rolL_rReg_CL", // 497
  "rolL_rReg_i1", // 498
  "rolL_rReg_i1_0", // 499
  "rolL_rReg_i8", // 500
  "rolL_rReg_i8_0", // 501
  "rolL_rReg_Var_C0", // 502
  "rolL_rReg_Var_C0_0", // 503
  "rolL_rReg_Var_C64", // 504
  "rolL_rReg_Var_C64_0", // 505
  "rorL_rReg_imm1", // 506
  "rorL_rReg_imm8", // 507
  "rorL_rReg_CL", // 508
  "rorL_rReg_i1", // 509
  "rorL_rReg_i1_0", // 510
  "rorL_rReg_i8", // 511
  "rorL_rReg_i8_0", // 512
  "rorL_rReg_Var_C0", // 513
  "rorL_rReg_Var_C0_0", // 514
  "rorL_rReg_Var_C64", // 515
  "rorL_rReg_Var_C64_0", // 516
  "andI_rReg", // 517
  "andI_rReg_imm255", // 518
  "andI2L_rReg_imm255", // 519
  "andI_rReg_imm65535", // 520
  "andI2L_rReg_imm65535", // 521
  "andI_rReg_imm", // 522
  "andI_rReg_mem", // 523
  "andI_rReg_mem_0", // 524
  "andI_mem_rReg", // 525
  "andI_mem_rReg_0", // 526
  "andI_mem_imm", // 527
  "orI_rReg", // 528
  "orI_rReg_imm", // 529
  "orI_rReg_mem", // 530
  "orI_rReg_mem_0", // 531
  "orI_mem_rReg", // 532
  "orI_mem_rReg_0", // 533
  "orI_mem_imm", // 534
  "xorI_rReg", // 535
  "xorI_rReg_imm", // 536
  "xorI_rReg_mem", // 537
  "xorI_rReg_mem_0", // 538
  "xorI_mem_rReg", // 539
  "xorI_mem_rReg_0", // 540
  "xorI_mem_imm", // 541
  "andL_rReg", // 542
  "andL_rReg_imm255", // 543
  "andL_rReg_imm65535", // 544
  "andL_rReg_imm", // 545
  "andL_rReg_mem", // 546
  "andL_rReg_mem_0", // 547
  "andL_mem_rReg", // 548
  "andL_mem_rReg_0", // 549
  "andL_mem_imm", // 550
  "orL_rReg", // 551
  "orL_rReg_imm", // 552
  "orL_rReg_mem", // 553
  "orL_rReg_mem_0", // 554
  "orL_mem_rReg", // 555
  "orL_mem_rReg_0", // 556
  "orL_mem_imm", // 557
  "xorL_rReg", // 558
  "xorL_rReg_imm", // 559
  "xorL_rReg_mem", // 560
  "xorL_rReg_mem_0", // 561
  "xorL_mem_rReg", // 562
  "xorL_mem_rReg_0", // 563
  "xorL_mem_imm", // 564
  "convI2B", // 565
  "convP2B", // 566
  "cmpLTMask", // 567
  "cmpLTMask0", // 568
  "cadd_cmpLTMask", // 569
  "cadd_cmpLTMask_1", // 570
  "cadd_cmpLTMask_0", // 571
  "cadd_cmpLTMask_2", // 572
  "cmpF_reg", // 573
  "cmpF_mem", // 574
  "cmpF_imm", // 575
  "cmpD_reg", // 576
  "cmpD_mem", // 577
  "cmpD_imm", // 578
  "addF_reg", // 579
  "addF_mem", // 580
  "addF_mem_0", // 581
  "addF_imm", // 582
  "addD_reg", // 583
  "addD_mem", // 584
  "addD_mem_0", // 585
  "addD_imm", // 586
  "subF_reg", // 587
  "subF_mem", // 588
  "subF_imm", // 589
  "subD_reg", // 590
  "subD_mem", // 591
  "subD_imm", // 592
  "mulF_reg", // 593
  "mulF_mem", // 594
  "mulF_mem_0", // 595
  "mulF_imm", // 596
  "mulD_reg", // 597
  "mulD_mem", // 598
  "mulD_mem_0", // 599
  "mulD_imm", // 600
  "divF_reg", // 601
  "divF_mem", // 602
  "divF_imm", // 603
  "divD_reg", // 604
  "divD_mem", // 605
  "divD_imm", // 606
  "sqrtF_reg", // 607
  "sqrtF_mem", // 608
  "sqrtF_imm", // 609
  "sqrtD_reg", // 610
  "sqrtD_mem", // 611
  "sqrtD_imm", // 612
  "cosD_reg", // 613
  "sinD_reg", // 614
  "tanD_reg", // 615
  "log10D_reg", // 616
  "logD_reg", // 617
  "convF2D_reg_reg", // 618
  "convF2D_reg_mem", // 619
  "convD2F_reg_reg", // 620
  "convD2F_reg_mem", // 621
  "convF2I_reg_reg", // 622
  "convF2L_reg_reg", // 623
  "convD2I_reg_reg", // 624
  "convD2L_reg_reg", // 625
  "convI2F_reg_reg", // 626
  "convI2F_reg_mem", // 627
  "convI2D_reg_reg", // 628
  "convI2D_reg_mem", // 629
  "convL2F_reg_reg", // 630
  "convL2F_reg_mem", // 631
  "convL2D_reg_reg", // 632
  "convL2D_reg_mem", // 633
  "convI2L_reg_reg", // 634
  "convI2L_reg_mem", // 635
  "convI2L_reg_reg_zex", // 636
  "convI2L_reg_mem_zex", // 637
  "zerox_long_reg_reg", // 638
  "convL2I_reg_reg", // 639
  "MoveF2I_reg_stack", // 640
  "MoveI2F_reg_stack", // 641
  "MoveD2L_reg_stack", // 642
  "MoveL2D_reg_stack", // 643
  "Repl8B_reg", // 644
  "Repl8B_rRegI", // 645
  "Repl8B_immI0", // 646
  "Repl4S_reg", // 647
  "Repl4S_rRegI", // 648
  "Repl4S_immI0", // 649
  "Repl4C_reg", // 650
  "Repl4C_rRegI", // 651
  "Repl4C_immI0", // 652
  "Repl2I_reg", // 653
  "Repl2I_rRegI", // 654
  "Repl2I_immI0", // 655
  "Repl2F_reg", // 656
  "Repl2F_regF", // 657
  "Repl2F_immF0", // 658
  "rep_stos", // 659
  "string_compare", // 660
  "cmpL3_reg_reg", // 661
  "cmovI_reg_g", // 662
  "minI_rReg", // 663
  "cmovI_reg_l", // 664
  "maxI_rReg", // 665
  "jmpDir", // 666
  "jmpCon", // 667
  "jmpLoopEnd", // 668
  "jmpLoopEndU", // 669
  "jmpConU", // 670
  "partialSubtypeCheck", // 671
  "jmpDir_short", // 672
  "jmpCon_short", // 673
  "jmpLoopEnd_short", // 674
  "jmpLoopEndU_short", // 675
  "jmpConU_short", // 676
  "CallStaticJavaDirect", // 677
  "CallDynamicJavaDirect", // 678
  "CallRuntimeDirect", // 679
  "CallLeafDirect", // 680
  "CallLeafNoFPDirect", // 681
  "Ret", // 682
  "TailCalljmpInd", // 683
  "tailjmpInd", // 684
  "CreateException", // 685
  "RethrowException", // 686
  "ShouldNotReachHere", // 687
  // last instruction
  "no trailing comma"
};

const        bool  swallowed[] = {
  false, // 0
  false, // 1
  false, // 2
  false, // 3
  false, // 4
  false, // 5
  false, // 6
  false, // 7
  true, // 8
  true, // 9
  true, // 10
  true, // 11
  true, // 12
  true, // 13
  true, // 14
  true, // 15
  true, // 16
  true, // 17
  true, // 18
  true, // 19
  true, // 20
  true, // 21
  true, // 22
  true, // 23
  true, // 24
  true, // 25
  true, // 26
  true, // 27
  true, // 28
  true, // 29
  true, // 30
  true, // 31
  true, // 32
  true, // 33
  true, // 34
  true, // 35
  true, // 36
  true, // 37
  true, // 38
  true, // 39
  false, // 40
  false, // 41
  false, // 42
  false, // 43
  false, // 44
  false, // 45
  false, // 46
  false, // 47
  false, // 48
  false, // 49
  false, // 50
  false, // 51
  false, // 52
  false, // 53
  false, // 54
  false, // 55
  false, // 56
  false, // 57
  false, // 58
  false, // 59
  false, // 60
  false, // 61
  false, // 62
  false, // 63
  false, // 64
  false, // 65
  false, // 66
  false, // 67
  false, // 68
  false, // 69
  false, // 70
  false, // 71
  false, // 72
  false, // 73
  false, // 74
  false, // 75
  false, // 76
  false, // 77
  false, // 78
  false, // 79
  false, // 80
  false, // 81
  true, // 82
  true, // 83
  // last operand
  false, // 84
  // last operand class
  false, // 85
  false, // 86
  false, // 87
  false, // 88
  false, // 89
  false, // 90
  false, // 91
  false, // 92
  false, // 93
  false, // 94
  false, // 95
  false, // 96
  false, // 97
  false, // 98
  false, // 99
  false, // 100
  false, // 101
  false, // 102
  false, // 103
  false, // 104
  false, // 105
  false, // 106
  false, // 107
  false, // 108
  false, // 109
  false, // 110
  false, // 111
  false, // 112
  false, // 113
  false, // 114
  false, // 115
  false, // 116
  false, // 117
  false, // 118
  false, // 119
  false, // 120
  false, // 121
  false, // 122
  false, // 123
  false, // 124
  false, // 125
  false, // 126
  false, // 127
  false, // 128
  false, // 129
  false, // 130
  false, // 131
  false, // 132
  false, // 133
  false, // 134
  false, // 135
  false, // 136
  false, // 137
  false, // 138
  false, // 139
  false, // 140
  false, // 141
  false, // 142
  false, // 143
  false, // 144
  false, // 145
  false, // 146
  false, // 147
  false, // 148
  false, // 149
  false, // 150
  false, // 151
  false, // 152
  false, // 153
  false, // 154
  false, // 155
  false, // 156
  false, // 157
  false, // 158
  false, // 159
  false, // 160
  false, // 161
  false, // 162
  false, // 163
  false, // 164
  false, // 165
  false, // 166
  false, // 167
  false, // 168
  false, // 169
  false, // 170
  false, // 171
  false, // 172
  false, // 173
  false, // 174
  false, // 175
  false, // 176
  false, // 177
  false, // 178
  false, // 179
  false, // 180
  false, // 181
  false, // 182
  false, // 183
  false, // 184
  false, // 185
  false, // 186
  false, // 187
  false, // 188
  false, // 189
  false, // 190
  false, // 191
  false, // 192
  false, // 193
  false, // 194
  false, // 195
  false, // 196
  false, // 197
  false, // 198
  false, // 199
  false, // 200
  false, // 201
  false, // 202
  false, // 203
  false, // 204
  false, // 205
  false, // 206
  false, // 207
  false, // 208
  // last internally defined operand
  false, // 209
  false, // 210
  false, // 211
  false, // 212
  false, // 213
  false, // 214
  false, // 215
  false, // 216
  false, // 217
  false, // 218
  false, // 219
  false, // 220
  false, // 221
  false, // 222
  false, // 223
  false, // 224
  false, // 225
  false, // 226
  false, // 227
  false, // 228
  false, // 229
  false, // 230
  false, // 231
  false, // 232
  false, // 233
  false, // 234
  false, // 235
  false, // 236
  false, // 237
  false, // 238
  false, // 239
  false, // 240
  false, // 241
  false, // 242
  false, // 243
  false, // 244
  false, // 245
  false, // 246
  false, // 247
  false, // 248
  false, // 249
  false, // 250
  false, // 251
  false, // 252
  false, // 253
  false, // 254
  false, // 255
  false, // 256
  false, // 257
  false, // 258
  false, // 259
  false, // 260
  false, // 261
  false, // 262
  false, // 263
  false, // 264
  false, // 265
  false, // 266
  false, // 267
  false, // 268
  false, // 269
  false, // 270
  false, // 271
  false, // 272
  false, // 273
  false, // 274
  false, // 275
  false, // 276
  false, // 277
  false, // 278
  false, // 279
  false, // 280
  false, // 281
  false, // 282
  false, // 283
  false, // 284
  false, // 285
  false, // 286
  false, // 287
  false, // 288
  false, // 289
  false, // 290
  false, // 291
  false, // 292
  false, // 293
  false, // 294
  false, // 295
  false, // 296
  false, // 297
  false, // 298
  false, // 299
  false, // 300
  false, // 301
  false, // 302
  false, // 303
  false, // 304
  false, // 305
  false, // 306
  false, // 307
  false, // 308
  false, // 309
  false, // 310
  false, // 311
  false, // 312
  false, // 313
  false, // 314
  false, // 315
  false, // 316
  false, // 317
  false, // 318
  false, // 319
  false, // 320
  false, // 321
  false, // 322
  false, // 323
  false, // 324
  false, // 325
  false, // 326
  false, // 327
  false, // 328
  false, // 329
  false, // 330
  false, // 331
  false, // 332
  false, // 333
  false, // 334
  false, // 335
  false, // 336
  false, // 337
  false, // 338
  false, // 339
  false, // 340
  false, // 341
  false, // 342
  false, // 343
  false, // 344
  false, // 345
  false, // 346
  false, // 347
  false, // 348
  false, // 349
  false, // 350
  false, // 351
  false, // 352
  false, // 353
  false, // 354
  false, // 355
  false, // 356
  false, // 357
  false, // 358
  false, // 359
  false, // 360
  false, // 361
  false, // 362
  false, // 363
  false, // 364
  false, // 365
  false, // 366
  false, // 367
  false, // 368
  false, // 369
  false, // 370
  false, // 371
  false, // 372
  false, // 373
  false, // 374
  false, // 375
  false, // 376
  false, // 377
  false, // 378
  false, // 379
  false, // 380
  false, // 381
  false, // 382
  false, // 383
  false, // 384
  false, // 385
  false, // 386
  false, // 387
  false, // 388
  false, // 389
  false, // 390
  false, // 391
  false, // 392
  false, // 393
  false, // 394
  false, // 395
  false, // 396
  false, // 397
  false, // 398
  false, // 399
  false, // 400
  false, // 401
  false, // 402
  false, // 403
  false, // 404
  false, // 405
  false, // 406
  false, // 407
  false, // 408
  false, // 409
  false, // 410
  false, // 411
  false, // 412
  false, // 413
  false, // 414
  false, // 415
  false, // 416
  false, // 417
  false, // 418
  false, // 419
  false, // 420
  false, // 421
  false, // 422
  false, // 423
  false, // 424
  false, // 425
  false, // 426
  false, // 427
  false, // 428
  false, // 429
  false, // 430
  false, // 431
  false, // 432
  false, // 433
  false, // 434
  false, // 435
  false, // 436
  false, // 437
  false, // 438
  false, // 439
  false, // 440
  false, // 441
  false, // 442
  false, // 443
  false, // 444
  false, // 445
  false, // 446
  false, // 447
  false, // 448
  false, // 449
  false, // 450
  false, // 451
  false, // 452
  false, // 453
  false, // 454
  false, // 455
  false, // 456
  false, // 457
  false, // 458
  false, // 459
  false, // 460
  false, // 461
  false, // 462
  false, // 463
  false, // 464
  false, // 465
  false, // 466
  false, // 467
  false, // 468
  false, // 469
  false, // 470
  false, // 471
  false, // 472
  false, // 473
  false, // 474
  false, // 475
  false, // 476
  false, // 477
  false, // 478
  false, // 479
  false, // 480
  false, // 481
  false, // 482
  false, // 483
  false, // 484
  false, // 485
  false, // 486
  false, // 487
  false, // 488
  false, // 489
  false, // 490
  false, // 491
  false, // 492
  false, // 493
  false, // 494
  false, // 495
  false, // 496
  false, // 497
  false, // 498
  false, // 499
  false, // 500
  false, // 501
  false, // 502
  false, // 503
  false, // 504
  false, // 505
  false, // 506
  false, // 507
  false, // 508
  false, // 509
  false, // 510
  false, // 511
  false, // 512
  false, // 513
  false, // 514
  false, // 515
  false, // 516
  false, // 517
  false, // 518
  false, // 519
  false, // 520
  false, // 521
  false, // 522
  false, // 523
  false, // 524
  false, // 525
  false, // 526
  false, // 527
  false, // 528
  false, // 529
  false, // 530
  false, // 531
  false, // 532
  false, // 533
  false, // 534
  false, // 535
  false, // 536
  false, // 537
  false, // 538
  false, // 539
  false, // 540
  false, // 541
  false, // 542
  false, // 543
  false, // 544
  false, // 545
  false, // 546
  false, // 547
  false, // 548
  false, // 549
  false, // 550
  false, // 551
  false, // 552
  false, // 553
  false, // 554
  false, // 555
  false, // 556
  false, // 557
  false, // 558
  false, // 559
  false, // 560
  false, // 561
  false, // 562
  false, // 563
  false, // 564
  false, // 565
  false, // 566
  false, // 567
  false, // 568
  false, // 569
  false, // 570
  false, // 571
  false, // 572
  false, // 573
  false, // 574
  false, // 575
  false, // 576
  false, // 577
  false, // 578
  false, // 579
  false, // 580
  false, // 581
  false, // 582
  false, // 583
  false, // 584
  false, // 585
  false, // 586
  false, // 587
  false, // 588
  false, // 589
  false, // 590
  false, // 591
  false, // 592
  false, // 593
  false, // 594
  false, // 595
  false, // 596
  false, // 597
  false, // 598
  false, // 599
  false, // 600
  false, // 601
  false, // 602
  false, // 603
  false, // 604
  false, // 605
  false, // 606
  false, // 607
  false, // 608
  false, // 609
  false, // 610
  false, // 611
  false, // 612
  false, // 613
  false, // 614
  false, // 615
  false, // 616
  false, // 617
  false, // 618
  false, // 619
  false, // 620
  false, // 621
  false, // 622
  false, // 623
  false, // 624
  false, // 625
  false, // 626
  false, // 627
  false, // 628
  false, // 629
  false, // 630
  false, // 631
  false, // 632
  false, // 633
  false, // 634
  false, // 635
  false, // 636
  false, // 637
  false, // 638
  false, // 639
  false, // 640
  false, // 641
  false, // 642
  false, // 643
  false, // 644
  false, // 645
  false, // 646
  false, // 647
  false, // 648
  false, // 649
  false, // 650
  false, // 651
  false, // 652
  false, // 653
  false, // 654
  false, // 655
  false, // 656
  false, // 657
  false, // 658
  false, // 659
  false, // 660
  false, // 661
  false, // 662
  false, // 663
  false, // 664
  false, // 665
  false, // 666
  false, // 667
  false, // 668
  false, // 669
  false, // 670
  false, // 671
  false, // 672
  false, // 673
  false, // 674
  false, // 675
  false, // 676
  false, // 677
  false, // 678
  false, // 679
  false, // 680
  false, // 681
  false, // 682
  false, // 683
  false, // 684
  false, // 685
  false, // 686
  false, // 687
  // last instruction
  false // no trailing comma
};

// Mapping from machine-independent opcode to boolean
const        char must_clone[] = {
  0, // Node: 0
  0, // Set: 1
  0, // RegI: 2
  0, // RegP: 3
  0, // RegF: 4
  0, // RegD: 5
  0, // RegL: 6
  0, // RegFlags: 7
  0, // _last_machine_leaf: 8
  0, // AbsD: 9
  0, // AbsF: 10
  0, // AbsI: 11
  0, // AddD: 12
  0, // AddF: 13
  0, // AddI: 14
  0, // AddL: 15
  0, // AddP: 16
  0, // Allocate: 17
  0, // AllocateArray: 18
  0, // AndI: 19
  0, // AndL: 20
  0, // AtanD: 21
  1, // Binary: 22
  1, // Bool: 23
  0, // BoxLock: 24
  0, // ReverseBytesI: 25
  0, // ReverseBytesL: 26
  0, // CProj: 27
  0, // CallDynamicJava: 28
  0, // CallJava: 29
  0, // CallLeaf: 30
  0, // CallLeafNoFP: 31
  0, // CallRuntime: 32
  0, // CallStaticJava: 33
  0, // CastII: 34
  0, // CastX2P: 35
  0, // CastP2X: 36
  0, // CastPP: 37
  0, // Catch: 38
  0, // CatchProj: 39
  0, // CheckCastPP: 40
  0, // ClearArray: 41
  0, // ConstraintCast: 42
  0, // CMoveD: 43
  0, // CMoveF: 44
  0, // CMoveI: 45
  0, // CMoveL: 46
  0, // CMoveP: 47
  1, // CmpD: 48
  0, // CmpD3: 49
  1, // CmpF: 50
  0, // CmpF3: 51
  1, // CmpI: 52
  1, // CmpL: 53
  0, // CmpL3: 54
  0, // CmpLTMask: 55
  1, // CmpP: 56
  1, // CmpU: 57
  0, // CompareAndSwapI: 58
  0, // CompareAndSwapL: 59
  0, // CompareAndSwapP: 60
  0, // Con: 61
  0, // ConD: 62
  0, // ConF: 63
  0, // ConI: 64
  0, // ConL: 65
  0, // ConP: 66
  0, // Conv2B: 67
  0, // ConvD2F: 68
  0, // ConvD2I: 69
  0, // ConvD2L: 70
  0, // ConvF2D: 71
  0, // ConvF2I: 72
  0, // ConvF2L: 73
  0, // ConvI2D: 74
  0, // ConvI2F: 75
  0, // ConvI2L: 76
  0, // ConvL2D: 77
  0, // ConvL2F: 78
  0, // ConvL2I: 79
  0, // CosD: 80
  0, // CountedLoop: 81
  0, // CountedLoopEnd: 82
  0, // CreateEx: 83
  0, // DivD: 84
  0, // DivF: 85
  0, // DivI: 86
  0, // DivL: 87
  0, // DivMod: 88
  0, // DivModI: 89
  0, // DivModL: 90
  0, // ExpD: 91
  1, // FastLock: 92
  1, // FastUnlock: 93
  0, // Goto: 94
  0, // Halt: 95
  0, // If: 96
  0, // IfFalse: 97
  0, // IfTrue: 98
  0, // Initialize: 99
  0, // JProj: 100
  0, // Jump: 101
  0, // JumpProj: 102
  0, // LShiftI: 103
  0, // LShiftL: 104
  0, // LoadB: 105
  0, // LoadC: 106
  0, // LoadD: 107
  0, // LoadD_unaligned: 108
  0, // LoadF: 109
  0, // LoadI: 110
  0, // LoadKlass: 111
  0, // LoadL: 112
  0, // LoadL_unaligned: 113
  0, // LoadPLocked: 114
  0, // LoadLLocked: 115
  0, // LoadP: 116
  0, // LoadRange: 117
  0, // LoadS: 118
  0, // Lock: 119
  0, // LogD: 120
  0, // Log10D: 121
  0, // Loop: 122
  0, // Mach: 123
  0, // MachProj: 124
  0, // MaxI: 125
  0, // MemBarAcquire: 126
  0, // MemBarCPUOrder: 127
  0, // MemBarRelease: 128
  0, // MemBarVolatile: 129
  0, // MergeMem: 130
  0, // MinI: 131
  0, // ModD: 132
  0, // ModF: 133
  0, // ModI: 134
  0, // ModL: 135
  0, // MoveI2F: 136
  0, // MoveF2I: 137
  0, // MoveL2D: 138
  0, // MoveD2L: 139
  0, // MulD: 140
  0, // MulF: 141
  0, // MulI: 142
  0, // MulL: 143
  0, // Multi: 144
  0, // NegD: 145
  0, // NegF: 146
  0, // NeverBranch: 147
  0, // Opaque1: 148
  0, // Opaque2: 149
  0, // OrI: 150
  0, // OrL: 151
  0, // PCTable: 152
  0, // Parm: 153
  0, // PartialSubtypeCheck: 154
  0, // Phi: 155
  0, // PowD: 156
  0, // PrefetchRead: 157
  0, // PrefetchWrite: 158
  0, // Proj: 159
  0, // RShiftI: 160
  0, // RShiftL: 161
  0, // Region: 162
  0, // Rethrow: 163
  0, // Return: 164
  0, // Root: 165
  0, // RoundDouble: 166
  0, // RoundFloat: 167
  0, // SafePoint: 168
  0, // SCMemProj: 169
  0, // SinD: 170
  0, // SqrtD: 171
  0, // Start: 172
  0, // StartOSR: 173
  0, // StoreB: 174
  0, // StoreC: 175
  0, // StoreCM: 176
  0, // StorePConditional: 177
  0, // StoreLConditional: 178
  0, // StoreD: 179
  0, // StoreF: 180
  0, // StoreI: 181
  0, // StoreL: 182
  0, // StoreP: 183
  0, // StrComp: 184
  0, // SubD: 185
  0, // SubF: 186
  0, // SubI: 187
  0, // SubL: 188
  0, // TailCall: 189
  0, // TailJump: 190
  0, // TanD: 191
  0, // ThreadLocal: 192
  0, // Unlock: 193
  0, // URShiftI: 194
  0, // URShiftL: 195
  0, // XorI: 196
  0, // XorL: 197
  0, // Vector: 198
  0, // AddVB: 199
  0, // AddVC: 200
  0, // AddVS: 201
  0, // AddVI: 202
  0, // AddVL: 203
  0, // AddVF: 204
  0, // AddVD: 205
  0, // SubVB: 206
  0, // SubVC: 207
  0, // SubVS: 208
  0, // SubVI: 209
  0, // SubVL: 210
  0, // SubVF: 211
  0, // SubVD: 212
  0, // MulVF: 213
  0, // MulVD: 214
  0, // DivVF: 215
  0, // DivVD: 216
  0, // LShiftVB: 217
  0, // LShiftVC: 218
  0, // LShiftVS: 219
  0, // LShiftVI: 220
  0, // URShiftVB: 221
  0, // URShiftVC: 222
  0, // URShiftVS: 223
  0, // URShiftVI: 224
  0, // AndV: 225
  0, // OrV: 226
  0, // XorV: 227
  0, // VectorLoad: 228
  0, // Load16B: 229
  0, // Load8B: 230
  0, // Load4B: 231
  0, // Load8C: 232
  0, // Load4C: 233
  0, // Load2C: 234
  0, // Load8S: 235
  0, // Load4S: 236
  0, // Load2S: 237
  0, // Load4I: 238
  0, // Load2I: 239
  0, // Load2L: 240
  0, // Load4F: 241
  0, // Load2F: 242
  0, // Load2D: 243
  0, // VectorStore: 244
  0, // Store16B: 245
  0, // Store8B: 246
  0, // Store4B: 247
  0, // Store8C: 248
  0, // Store4C: 249
  0, // Store2C: 250
  0, // Store4I: 251
  0, // Store2I: 252
  0, // Store2L: 253
  0, // Store4F: 254
  0, // Store2F: 255
  0, // Store2D: 256
  0, // Pack: 257
  0, // PackB: 258
  0, // PackS: 259
  0, // PackC: 260
  0, // PackI: 261
  0, // PackL: 262
  0, // PackF: 263
  0, // PackD: 264
  0, // Pack2x1B: 265
  0, // Pack2x2B: 266
  0, // Replicate16B: 267
  0, // Replicate8B: 268
  0, // Replicate4B: 269
  0, // Replicate8S: 270
  0, // Replicate4S: 271
  0, // Replicate2S: 272
  0, // Replicate8C: 273
  0, // Replicate4C: 274
  0, // Replicate2C: 275
  0, // Replicate4I: 276
  0, // Replicate2I: 277
  0, // Replicate2L: 278
  0, // Replicate4F: 279
  0, // Replicate2F: 280
  0, // Replicate2D: 281
  0, // Extract: 282
  0, // ExtractB: 283
  0, // ExtractS: 284
  0, // ExtractC: 285
  0, // ExtractI: 286
  0, // ExtractL: 287
  0, // ExtractF: 288
  0 // no trailing comma // ExtractD: 289
};
//  The following instructions can cisc-spill
//  cmovI_reg can cisc-spill operand 4 to cmovI_mem
//  cmovI_regU can cisc-spill operand 4 to cmovI_memU
//  cmovL_reg can cisc-spill operand 4 to cmovL_mem
//  cmovL_regU can cisc-spill operand 4 to cmovL_memU
//  addI_rReg can cisc-spill operand 2 to addI_rReg_mem
//  addL_rReg can cisc-spill operand 2 to addL_rReg_mem
//  subI_rReg can cisc-spill operand 2 to subI_rReg_mem
//  subL_rReg can cisc-spill operand 2 to subL_rReg_mem
//  mulI_rReg can cisc-spill operand 2 to mulI_mem
//  mulI_rReg_imm can cisc-spill operand 1 to mulI_mem_imm
//  mulL_rReg can cisc-spill operand 2 to mulL_mem
//  mulL_rReg_imm can cisc-spill operand 1 to mulL_mem_imm
//  andI_rReg can cisc-spill operand 2 to andI_rReg_mem
//  orI_rReg can cisc-spill operand 2 to orI_rReg_mem
//  xorI_rReg can cisc-spill operand 2 to xorI_rReg_mem
//  andL_rReg can cisc-spill operand 2 to andL_rReg_mem
//  orL_rReg can cisc-spill operand 2 to orL_rReg_mem
//  xorL_rReg can cisc-spill operand 2 to xorL_rReg_mem
//  cmpF_cc_reg can cisc-spill operand 2 to cmpF_cc_mem
//  cmpD_cc_reg can cisc-spill operand 2 to cmpD_cc_mem
//  cmpF_reg can cisc-spill operand 2 to cmpF_mem
//  cmpD_reg can cisc-spill operand 2 to cmpD_mem
//  addF_reg can cisc-spill operand 2 to addF_mem
//  addD_reg can cisc-spill operand 2 to addD_mem
//  subF_reg can cisc-spill operand 2 to subF_mem
//  subD_reg can cisc-spill operand 2 to subD_mem
//  mulF_reg can cisc-spill operand 2 to mulF_mem
//  mulD_reg can cisc-spill operand 2 to mulD_mem
//  divF_reg can cisc-spill operand 2 to divF_mem
//  divD_reg can cisc-spill operand 2 to divD_mem
//  sqrtF_reg can cisc-spill operand 1 to sqrtF_mem
//  sqrtD_reg can cisc-spill operand 1 to sqrtD_mem
//  convF2D_reg_reg can cisc-spill operand 1 to convF2D_reg_mem
//  convD2F_reg_reg can cisc-spill operand 1 to convD2F_reg_mem
//  convI2F_reg_reg can cisc-spill operand 1 to convI2F_reg_mem
//  convI2D_reg_reg can cisc-spill operand 1 to convI2D_reg_mem
//  convL2F_reg_reg can cisc-spill operand 1 to convL2F_reg_mem
//  convL2D_reg_reg can cisc-spill operand 1 to convL2D_reg_mem
//  convI2L_reg_reg can cisc-spill operand 1 to convI2L_reg_mem
//  convI2L_reg_reg_zex can cisc-spill operand 1 to convI2L_reg_mem_zex
//  MoveF2I_reg_reg can cisc-spill operand 1 to MoveF2I_stack_reg
//  MoveD2L_reg_reg can cisc-spill operand 1 to MoveD2L_stack_reg
//  MoveI2F_reg_reg can cisc-spill operand 1 to MoveI2F_stack_reg
//  compI_rReg can cisc-spill operand 2 to compI_rReg_mem
//  compU_rReg can cisc-spill operand 2 to compU_rReg_mem
//  compP_rReg can cisc-spill operand 2 to compP_rReg_mem
//  testP_reg can cisc-spill operand 1 to testP_reg_mem
//  compL_rReg can cisc-spill operand 2 to compL_rReg_mem



// An array of character pointers to machine register names.
const char *Matcher::regName[REG_COUNT] = {
  "R10",
  "R10_H",
  "R11",
  "R11_H",
  "R8",
  "R8_H",
  "R9",
  "R9_H",
  "R12",
  "R12_H",
  "RCX",
  "RCX_H",
  "RBX",
  "RBX_H",
  "RDI",
  "RDI_H",
  "RDX",
  "RDX_H",
  "RSI",
  "RSI_H",
  "RAX",
  "RAX_H",
  "RBP",
  "RBP_H",
  "R13",
  "R13_H",
  "R14",
  "R14_H",
  "R15",
  "R15_H",
  "RSP",
  "RSP_H",
  "XMM0",
  "XMM0_H",
  "XMM1",
  "XMM1_H",
  "XMM2",
  "XMM2_H",
  "XMM3",
  "XMM3_H",
  "XMM4",
  "XMM4_H",
  "XMM5",
  "XMM5_H",
  "XMM6",
  "XMM6_H",
  "XMM7",
  "XMM7_H",
  "XMM8",
  "XMM8_H",
  "XMM9",
  "XMM9_H",
  "XMM10",
  "XMM10_H",
  "XMM11",
  "XMM11_H",
  "XMM12",
  "XMM12_H",
  "XMM13",
  "XMM13_H",
  "XMM14",
  "XMM14_H",
  "XMM15",
  "XMM15_H",
  "RFLAGS" // no trailing comma
};

// An array of character pointers to machine register names.
const VMReg OptoReg::opto2vm[REG_COUNT] = {
	r10->as_VMReg(),
	r10->as_VMReg()->next(),
	r11->as_VMReg(),
	r11->as_VMReg()->next(),
	r8->as_VMReg(),
	r8->as_VMReg()->next(),
	r9->as_VMReg(),
	r9->as_VMReg()->next(),
	r12->as_VMReg(),
	r12->as_VMReg()->next(),
	rcx->as_VMReg(),
	rcx->as_VMReg()->next(),
	rbx->as_VMReg(),
	rbx->as_VMReg()->next(),
	rdi->as_VMReg(),
	rdi->as_VMReg()->next(),
	rdx->as_VMReg(),
	rdx->as_VMReg()->next(),
	rsi->as_VMReg(),
	rsi->as_VMReg()->next(),
	rax->as_VMReg(),
	rax->as_VMReg()->next(),
	rbp->as_VMReg(),
	rbp->as_VMReg()->next(),
	r13->as_VMReg(),
	r13->as_VMReg()->next(),
	r14->as_VMReg(),
	r14->as_VMReg()->next(),
	r15->as_VMReg(),
	r15->as_VMReg()->next(),
	rsp->as_VMReg(),
	rsp->as_VMReg()->next(),
	xmm0->as_VMReg(),
	xmm0->as_VMReg()->next(),
	xmm1->as_VMReg(),
	xmm1->as_VMReg()->next(),
	xmm2->as_VMReg(),
	xmm2->as_VMReg()->next(),
	xmm3->as_VMReg(),
	xmm3->as_VMReg()->next(),
	xmm4->as_VMReg(),
	xmm4->as_VMReg()->next(),
	xmm5->as_VMReg(),
	xmm5->as_VMReg()->next(),
	xmm6->as_VMReg(),
	xmm6->as_VMReg()->next(),
	xmm7->as_VMReg(),
	xmm7->as_VMReg()->next(),
	xmm8->as_VMReg(),
	xmm8->as_VMReg()->next(),
	xmm9->as_VMReg(),
	xmm9->as_VMReg()->next(),
	xmm10->as_VMReg(),
	xmm10->as_VMReg()->next(),
	xmm11->as_VMReg(),
	xmm11->as_VMReg()->next(),
	xmm12->as_VMReg(),
	xmm12->as_VMReg()->next(),
	xmm13->as_VMReg(),
	xmm13->as_VMReg()->next(),
	xmm14->as_VMReg(),
	xmm14->as_VMReg()->next(),
	xmm15->as_VMReg(),
	xmm15->as_VMReg()->next(),
	VMRegImpl::Bad() // no trailing comma
	};

 OptoReg::Name OptoReg::vm2opto[ConcreteRegisterImpl::number_of_registers];

// An array of the machine register encode values
const unsigned char Matcher::_regEncode[REG_COUNT] = {
  (unsigned char)'\xA',  // R10
  (unsigned char)'\xA',  // R10_H
  (unsigned char)'\xB',  // R11
  (unsigned char)'\xB',  // R11_H
  (unsigned char)'\x8',  // R8
  (unsigned char)'\x8',  // R8_H
  (unsigned char)'\x9',  // R9
  (unsigned char)'\x9',  // R9_H
  (unsigned char)'\xC',  // R12
  (unsigned char)'\xC',  // R12_H
  (unsigned char)'\x1',  // RCX
  (unsigned char)'\x1',  // RCX_H
  (unsigned char)'\x3',  // RBX
  (unsigned char)'\x3',  // RBX_H
  (unsigned char)'\x7',  // RDI
  (unsigned char)'\x7',  // RDI_H
  (unsigned char)'\x2',  // RDX
  (unsigned char)'\x2',  // RDX_H
  (unsigned char)'\x6',  // RSI
  (unsigned char)'\x6',  // RSI_H
  (unsigned char)'\x0',  // RAX
  (unsigned char)'\x0',  // RAX_H
  (unsigned char)'\x5',  // RBP
  (unsigned char)'\x5',  // RBP_H
  (unsigned char)'\xD',  // R13
  (unsigned char)'\xD',  // R13_H
  (unsigned char)'\xE',  // R14
  (unsigned char)'\xE',  // R14_H
  (unsigned char)'\xF',  // R15
  (unsigned char)'\xF',  // R15_H
  (unsigned char)'\x4',  // RSP
  (unsigned char)'\x4',  // RSP_H
  (unsigned char)'\x0',  // XMM0
  (unsigned char)'\x0',  // XMM0_H
  (unsigned char)'\x1',  // XMM1
  (unsigned char)'\x1',  // XMM1_H
  (unsigned char)'\x2',  // XMM2
  (unsigned char)'\x2',  // XMM2_H
  (unsigned char)'\x3',  // XMM3
  (unsigned char)'\x3',  // XMM3_H
  (unsigned char)'\x4',  // XMM4
  (unsigned char)'\x4',  // XMM4_H
  (unsigned char)'\x5',  // XMM5
  (unsigned char)'\x5',  // XMM5_H
  (unsigned char)'\x6',  // XMM6
  (unsigned char)'\x6',  // XMM6_H
  (unsigned char)'\x7',  // XMM7
  (unsigned char)'\x7',  // XMM7_H
  (unsigned char)'\x8',  // XMM8
  (unsigned char)'\x8',  // XMM8_H
  (unsigned char)'\x9',  // XMM9
  (unsigned char)'\x9',  // XMM9_H
  (unsigned char)'\xA',  // XMM10
  (unsigned char)'\xA',  // XMM10_H
  (unsigned char)'\xB',  // XMM11
  (unsigned char)'\xB',  // XMM11_H
  (unsigned char)'\xC',  // XMM12
  (unsigned char)'\xC',  // XMM12_H
  (unsigned char)'\xD',  // XMM13
  (unsigned char)'\xD',  // XMM13_H
  (unsigned char)'\xE',  // XMM14
  (unsigned char)'\xE',  // XMM14_H
  (unsigned char)'\xF',  // XMM15
  (unsigned char)'\xF',  // XMM15_H
  (unsigned char)'\x10' // no trailing comma  // RFLAGS
};


//------------------Define classes derived from MachOper---------------------
MachOper  *labelOper::clone(Compile* C) const {
  return  new (C) labelOper(_label, _block_num);
}
uint labelOper::opcode() const { return LABEL; }

const RegMask *sRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

MachOper  *methodOper::clone(Compile* C) const {
  return  new (C) methodOper(_method);
}
uint methodOper::opcode() const { return METHOD; }

const RegMask *rRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask;
}

const RegMask *rax_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RAX_REG_mask;
}

const RegMask *rbx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RBX_REG_mask;
}

const RegMask *rcx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RCX_REG_mask;
}

const RegMask *rdx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RDX_REG_mask;
}

const RegMask *rdi_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RDI_REG_mask;
}

const RegMask *no_rcx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_NO_RCX_REG_mask;
}

const RegMask *no_rax_rdx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_NO_RAX_RDX_REG_mask;
}

const RegMask *any_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask;
}

const RegMask *rRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask;
}

const RegMask *no_rax_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_NO_RAX_REG_mask;
}

const RegMask *no_rbp_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_NO_RBP_REG_mask;
}

const RegMask *no_rax_rbx_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_NO_RAX_RBX_REG_mask;
}

const RegMask *rax_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RAX_REG_mask;
}

const RegMask *rbx_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RBX_REG_mask;
}

const RegMask *rsi_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RSI_REG_mask;
}

const RegMask *rdi_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RDI_REG_mask;
}

const RegMask *rbp_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RBP_REG_mask;
}

const RegMask *r15_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_R15_REG_mask;
}

const RegMask *rRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_REG_mask;
}

const RegMask *no_rax_rdx_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_NO_RAX_RDX_REG_mask;
}

const RegMask *no_rax_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_NO_RAX_RDX_REG_mask;
}

const RegMask *no_rcx_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_NO_RCX_REG_mask;
}

const RegMask *rax_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_RAX_REG_mask;
}

const RegMask *rcx_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_RCX_REG_mask;
}

const RegMask *rdx_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_RDX_REG_mask;
}

const RegMask *rFlagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *rFlagsRegUOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *regFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_mask;
}

const RegMask *regDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_mask;
}

const RegMask *indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask;
}

const RegMask *indOffset8Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask;
}

const RegMask *indOffset32Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask;
}

const RegMask *indIndexOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask;
  case 1: return &LONG_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask;
  case 1: return &LONG_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaleOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask;
  case 1: return &LONG_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaleOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask;
  case 1: return &LONG_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *stackSlotPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

//------------------Define members for classes derived from MachNode----------

void cmovI_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovI_regNode::cisc_version( int offset, Compile* C ) {
  cmovI_memNode *node = new (C) cmovI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovI_regUNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovI_regUNode::cisc_version( int offset, Compile* C ) {
  cmovI_memUNode *node = new (C) cmovI_memUNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovL_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovL_regNode::cisc_version( int offset, Compile* C ) {
  cmovL_memNode *node = new (C) cmovL_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovL_regUNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovL_regUNode::cisc_version( int offset, Compile* C ) {
  cmovL_memUNode *node = new (C) cmovL_memUNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *addI_rRegNode::cisc_version( int offset, Compile* C ) {
  addI_rReg_memNode *node = new (C) addI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *addL_rRegNode::cisc_version( int offset, Compile* C ) {
  addL_rReg_memNode *node = new (C) addL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *subI_rRegNode::cisc_version( int offset, Compile* C ) {
  subI_rReg_memNode *node = new (C) subI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *subL_rRegNode::cisc_version( int offset, Compile* C ) {
  subL_rReg_memNode *node = new (C) subL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulI_rRegNode::cisc_version( int offset, Compile* C ) {
  mulI_memNode *node = new (C) mulI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulI_rReg_immNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulI_rReg_immNode::cisc_version( int offset, Compile* C ) {
  mulI_mem_immNode *node = new (C) mulI_mem_immNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulL_rRegNode::cisc_version( int offset, Compile* C ) {
  mulL_memNode *node = new (C) mulL_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulL_rReg_immNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulL_rReg_immNode::cisc_version( int offset, Compile* C ) {
  mulL_mem_immNode *node = new (C) mulL_mem_immNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void andI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *andI_rRegNode::cisc_version( int offset, Compile* C ) {
  andI_rReg_memNode *node = new (C) andI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void orI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *orI_rRegNode::cisc_version( int offset, Compile* C ) {
  orI_rReg_memNode *node = new (C) orI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void xorI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *xorI_rRegNode::cisc_version( int offset, Compile* C ) {
  xorI_rReg_memNode *node = new (C) xorI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void andL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *andL_rRegNode::cisc_version( int offset, Compile* C ) {
  andL_rReg_memNode *node = new (C) andL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void orL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *orL_rRegNode::cisc_version( int offset, Compile* C ) {
  orL_rReg_memNode *node = new (C) orL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void xorL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *xorL_rRegNode::cisc_version( int offset, Compile* C ) {
  xorL_rReg_memNode *node = new (C) xorL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpF_cc_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpF_cc_regNode::cisc_version( int offset, Compile* C ) {
  cmpF_cc_memNode *node = new (C) cmpF_cc_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpD_cc_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpD_cc_regNode::cisc_version( int offset, Compile* C ) {
  cmpD_cc_memNode *node = new (C) cmpD_cc_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpF_regNode::cisc_version( int offset, Compile* C ) {
  cmpF_memNode *node = new (C) cmpF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpD_regNode::cisc_version( int offset, Compile* C ) {
  cmpD_memNode *node = new (C) cmpD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *addF_regNode::cisc_version( int offset, Compile* C ) {
  addF_memNode *node = new (C) addF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *addD_regNode::cisc_version( int offset, Compile* C ) {
  addD_memNode *node = new (C) addD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *subF_regNode::cisc_version( int offset, Compile* C ) {
  subF_memNode *node = new (C) subF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *subD_regNode::cisc_version( int offset, Compile* C ) {
  subD_memNode *node = new (C) subD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulF_regNode::cisc_version( int offset, Compile* C ) {
  mulF_memNode *node = new (C) mulF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulD_regNode::cisc_version( int offset, Compile* C ) {
  mulD_memNode *node = new (C) mulD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void divF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *divF_regNode::cisc_version( int offset, Compile* C ) {
  divF_memNode *node = new (C) divF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void divD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *divD_regNode::cisc_version( int offset, Compile* C ) {
  divD_memNode *node = new (C) divD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void sqrtF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *sqrtF_regNode::cisc_version( int offset, Compile* C ) {
  sqrtF_memNode *node = new (C) sqrtF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void sqrtD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *sqrtD_regNode::cisc_version( int offset, Compile* C ) {
  sqrtD_memNode *node = new (C) sqrtD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convF2D_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *convF2D_reg_regNode::cisc_version( int offset, Compile* C ) {
  convF2D_reg_memNode *node = new (C) convF2D_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convD2F_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *convD2F_reg_regNode::cisc_version( int offset, Compile* C ) {
  convD2F_reg_memNode *node = new (C) convD2F_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convI2F_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *convI2F_reg_regNode::cisc_version( int offset, Compile* C ) {
  convI2F_reg_memNode *node = new (C) convI2F_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convI2D_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *convI2D_reg_regNode::cisc_version( int offset, Compile* C ) {
  convI2D_reg_memNode *node = new (C) convI2D_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convL2F_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *convL2F_reg_regNode::cisc_version( int offset, Compile* C ) {
  convL2F_reg_memNode *node = new (C) convL2F_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convL2D_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *convL2D_reg_regNode::cisc_version( int offset, Compile* C ) {
  convL2D_reg_memNode *node = new (C) convL2D_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convI2L_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *convI2L_reg_regNode::cisc_version( int offset, Compile* C ) {
  convI2L_reg_memNode *node = new (C) convI2L_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convI2L_reg_reg_zexNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *convI2L_reg_reg_zexNode::cisc_version( int offset, Compile* C ) {
  convI2L_reg_mem_zexNode *node = new (C) convI2L_reg_mem_zexNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void MoveF2I_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *MoveF2I_reg_regNode::cisc_version( int offset, Compile* C ) {
  MoveF2I_stack_regNode *node = new (C) MoveF2I_stack_regNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void MoveD2L_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *MoveD2L_reg_regNode::cisc_version( int offset, Compile* C ) {
  MoveD2L_stack_regNode *node = new (C) MoveD2L_stack_regNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void MoveI2F_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *MoveI2F_reg_regNode::cisc_version( int offset, Compile* C ) {
  MoveI2F_stack_regNode *node = new (C) MoveI2F_stack_regNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *compI_rRegNode::cisc_version( int offset, Compile* C ) {
  compI_rReg_memNode *node = new (C) compI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compU_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *compU_rRegNode::cisc_version( int offset, Compile* C ) {
  compU_rReg_memNode *node = new (C) compU_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compP_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_PTR_REG_mask;
}

// Build CISC version of this instruction
MachNode *compP_rRegNode::cisc_version( int offset, Compile* C ) {
  compP_rReg_memNode *node = new (C) compP_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void testP_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_PTR_REG_mask;
}

// Build CISC version of this instruction
MachNode *testP_regNode::cisc_version( int offset, Compile* C ) {
  testP_reg_memNode *node = new (C) testP_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *compL_rRegNode::cisc_version( int offset, Compile* C ) {
  compL_rReg_memNode *node = new (C) compL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}

// Build short branch version of this instruction
MachNode *jmpDirNode::short_branch_version(Compile* C) {
  jmpDir_shortNode *node = new (C) jmpDir_shortNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConNode::short_branch_version(Compile* C) {
  jmpCon_shortNode *node = new (C) jmpCon_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndNode::short_branch_version(Compile* C) {
  jmpLoopEnd_shortNode *node = new (C) jmpLoopEnd_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndUNode::short_branch_version(Compile* C) {
  jmpLoopEndU_shortNode *node = new (C) jmpLoopEndU_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUNode::short_branch_version(Compile* C) {
  jmpConU_shortNode *node = new (C) jmpConU_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}


// Copy _idx, inputs and operands to new node
void MachNode::fill_new_machnode( MachNode* node, Compile* C) const {
  // New node must use same node index
  node->set_idx( _idx );
  // Copy machine-independent inputs
  for( uint j = 0; j < req(); j++ ) {
    node->add_req(in(j));
  }
  // Copy my operands, except for cisc position
  int nopnds = num_opnds();
  assert( node->num_opnds() == (uint)nopnds, "Must have same number of operands");
  MachOper **to = node->_opnds;
  for( int i = 0; i < nopnds; i++ ) {
    if( i != cisc_operand() ) 
      to[i] = _opnds[i]->clone(C);
  }
}

void jmpDirNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEndNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEndUNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConUNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpDir_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpCon_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEnd_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEndU_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConU_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void CallStaticJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallRuntimeDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafNoFPDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

int  loadBNode::reloc()   const {
  return  1;
}

int  loadUBNode::reloc()   const {
  return  1;
}

int  loadSNode::reloc()   const {
  return  1;
}

int  loadCNode::reloc()   const {
  return  1;
}

int  loadINode::reloc()   const {
  return  1;
}

int  loadLNode::reloc()   const {
  return  1;
}

int  loadRangeNode::reloc()   const {
  return  1;
}

int  loadPNode::reloc()   const {
  return  1;
}

int  loadKlassNode::reloc()   const {
  return  1;
}

int  loadFNode::reloc()   const {
  return  1;
}

int  loadD_partialNode::reloc()   const {
  return  1;
}

int  loadDNode::reloc()   const {
  return  1;
}

int  loadA8BNode::reloc()   const {
  return  1;
}

int  loadA4SNode::reloc()   const {
  return  1;
}

int  loadA4CNode::reloc()   const {
  return  1;
}

int  load2IUNode::reloc()   const {
  return  1;
}

int  loadA2FNode::reloc()   const {
  return  1;
}

int  loadConPNode::reloc()   const {
  return  1;
}

int  loadConP0Node::reloc()   const {
  return  1;
}

int  loadConP31Node::reloc()   const {
  return  1;
}

int  loadConFNode::reloc()   const {
  return  2;
}

int  loadConF0Node::reloc()   const {
  return  2;
}

int  loadConDNode::reloc()   const {
  return  2;
}

int  loadConD0Node::reloc()   const {
  return  2;
}

int  prefetchrNode::reloc()   const {
  return  1;
}

int  prefetchrNTANode::reloc()   const {
  return  1;
}

int  prefetchrT0Node::reloc()   const {
  return  1;
}

int  prefetchrT2Node::reloc()   const {
  return  1;
}

int  prefetchwNode::reloc()   const {
  return  1;
}

int  prefetchwNTANode::reloc()   const {
  return  1;
}

int  prefetchwT0Node::reloc()   const {
  return  1;
}

int  prefetchwT2Node::reloc()   const {
  return  1;
}

int  storeBNode::reloc()   const {
  return  2;
}

int  storeCNode::reloc()   const {
  return  2;
}

int  storeINode::reloc()   const {
  return  2;
}

int  storeLNode::reloc()   const {
  return  2;
}

int  storePNode::reloc()   const {
  return  2;
}

int  storeImmPNode::reloc()   const {
  return  3;
}

int  storeImmINode::reloc()   const {
  return  2;
}

int  storeImmLNode::reloc()   const {
  return  2;
}

int  storeImmI16Node::reloc()   const {
  return  2;
}

int  storeImmBNode::reloc()   const {
  return  2;
}

int  storeA8BNode::reloc()   const {
  return  2;
}

int  storeA4CNode::reloc()   const {
  return  2;
}

int  storeA2INode::reloc()   const {
  return  2;
}

int  storeImmCM0Node::reloc()   const {
  return  2;
}

int  storeA2FNode::reloc()   const {
  return  2;
}

int  storeFNode::reloc()   const {
  return  2;
}

int  storeF_immNode::reloc()   const {
  return  3;
}

int  storeDNode::reloc()   const {
  return  2;
}

int  storeD0_immNode::reloc()   const {
  return  3;
}

int  loadI_reversedNode::reloc()   const {
  return  1;
}

int  loadL_reversedNode::reloc()   const {
  return  1;
}

int  storeI_reversedNode::reloc()   const {
  return  2;
}

int  storeL_reversedNode::reloc()   const {
  return  2;
}

int  cmovI_memNode::reloc()   const {
  return  1;
}

int  cmovI_memUNode::reloc()   const {
  return  1;
}

int  cmovL_memNode::reloc()   const {
  return  1;
}

int  cmovL_memUNode::reloc()   const {
  return  1;
}

int  addI_rReg_memNode::reloc()   const {
  return  1;
}

int  addI_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  addI_mem_rRegNode::reloc()   const {
  return  3;
}

int  addI_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  addI_mem_immNode::reloc()   const {
  return  3;
}

int  incI_memNode::reloc()   const {
  return  3;
}

int  decI_memNode::reloc()   const {
  return  3;
}

int  addL_rReg_memNode::reloc()   const {
  return  1;
}

int  addL_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  addL_mem_rRegNode::reloc()   const {
  return  3;
}

int  addL_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  addL_mem_immNode::reloc()   const {
  return  3;
}

int  incL_memNode::reloc()   const {
  return  3;
}

int  decL_memNode::reloc()   const {
  return  3;
}

int  loadPLockedNode::reloc()   const {
  return  1;
}

int  loadLLockedNode::reloc()   const {
  return  1;
}

int  storePConditionalNode::reloc()   const {
  return  1;
}

int  storeLConditionalNode::reloc()   const {
  return  1;
}

int  storeLConditional_flagsNode::reloc()   const {
  return  1;
}

int  compareAndSwapPNode::reloc()   const {
  return  1;
}

int  compareAndSwapLNode::reloc()   const {
  return  1;
}

int  compareAndSwapINode::reloc()   const {
  return  1;
}

int  subI_rReg_memNode::reloc()   const {
  return  1;
}

int  subI_mem_rRegNode::reloc()   const {
  return  3;
}

int  subI_mem_immNode::reloc()   const {
  return  3;
}

int  subL_rReg_memNode::reloc()   const {
  return  1;
}

int  subL_mem_rRegNode::reloc()   const {
  return  3;
}

int  subL_mem_immNode::reloc()   const {
  return  3;
}

int  negI_memNode::reloc()   const {
  return  3;
}

int  negL_memNode::reloc()   const {
  return  3;
}

int  mulI_memNode::reloc()   const {
  return  1;
}

int  mulI_mem_0Node::reloc()   const {
  return  1;
}

int  mulI_mem_immNode::reloc()   const {
  return  1;
}

int  mulL_memNode::reloc()   const {
  return  1;
}

int  mulL_mem_0Node::reloc()   const {
  return  1;
}

int  mulL_mem_immNode::reloc()   const {
  return  1;
}

int  salI_mem_1Node::reloc()   const {
  return  3;
}

int  salI_mem_immNode::reloc()   const {
  return  3;
}

int  salI_mem_CLNode::reloc()   const {
  return  3;
}

int  sarI_mem_1Node::reloc()   const {
  return  3;
}

int  sarI_mem_immNode::reloc()   const {
  return  3;
}

int  sarI_mem_CLNode::reloc()   const {
  return  3;
}

int  shrI_mem_1Node::reloc()   const {
  return  3;
}

int  shrI_mem_immNode::reloc()   const {
  return  3;
}

int  shrI_mem_CLNode::reloc()   const {
  return  3;
}

int  salL_mem_1Node::reloc()   const {
  return  3;
}

int  salL_mem_immNode::reloc()   const {
  return  3;
}

int  salL_mem_CLNode::reloc()   const {
  return  3;
}

int  sarL_mem_1Node::reloc()   const {
  return  3;
}

int  sarL_mem_immNode::reloc()   const {
  return  3;
}

int  sarL_mem_CLNode::reloc()   const {
  return  3;
}

int  shrL_mem_1Node::reloc()   const {
  return  3;
}

int  shrL_mem_immNode::reloc()   const {
  return  3;
}

int  shrL_mem_CLNode::reloc()   const {
  return  3;
}

int  andI_rReg_memNode::reloc()   const {
  return  1;
}

int  andI_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  andI_mem_rRegNode::reloc()   const {
  return  3;
}

int  andI_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  andI_mem_immNode::reloc()   const {
  return  3;
}

int  orI_rReg_memNode::reloc()   const {
  return  1;
}

int  orI_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  orI_mem_rRegNode::reloc()   const {
  return  3;
}

int  orI_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  orI_mem_immNode::reloc()   const {
  return  3;
}

int  xorI_rReg_memNode::reloc()   const {
  return  1;
}

int  xorI_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  xorI_mem_rRegNode::reloc()   const {
  return  3;
}

int  xorI_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  xorI_mem_immNode::reloc()   const {
  return  3;
}

int  andL_rReg_memNode::reloc()   const {
  return  1;
}

int  andL_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  andL_mem_rRegNode::reloc()   const {
  return  3;
}

int  andL_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  andL_mem_immNode::reloc()   const {
  return  3;
}

int  orL_rReg_memNode::reloc()   const {
  return  1;
}

int  orL_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  orL_mem_rRegNode::reloc()   const {
  return  3;
}

int  orL_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  orL_mem_immNode::reloc()   const {
  return  3;
}

int  xorL_rReg_memNode::reloc()   const {
  return  1;
}

int  xorL_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  xorL_mem_rRegNode::reloc()   const {
  return  3;
}

int  xorL_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  xorL_mem_immNode::reloc()   const {
  return  3;
}

int  cmpF_cc_memNode::reloc()   const {
  return  1;
}

int  cmpF_cc_immNode::reloc()   const {
  return  1;
}

int  cmpD_cc_memNode::reloc()   const {
  return  1;
}

int  cmpD_cc_immNode::reloc()   const {
  return  1;
}

int  cmpF_memNode::reloc()   const {
  return  1;
}

int  cmpF_immNode::reloc()   const {
  return  1;
}

int  cmpD_memNode::reloc()   const {
  return  1;
}

int  cmpD_immNode::reloc()   const {
  return  1;
}

int  addF_memNode::reloc()   const {
  return  1;
}

int  addF_mem_0Node::reloc()   const {
  return  1;
}

int  addF_immNode::reloc()   const {
  return  1;
}

int  addD_memNode::reloc()   const {
  return  1;
}

int  addD_mem_0Node::reloc()   const {
  return  1;
}

int  addD_immNode::reloc()   const {
  return  1;
}

int  subF_memNode::reloc()   const {
  return  1;
}

int  subF_immNode::reloc()   const {
  return  1;
}

int  subD_memNode::reloc()   const {
  return  1;
}

int  subD_immNode::reloc()   const {
  return  1;
}

int  mulF_memNode::reloc()   const {
  return  1;
}

int  mulF_mem_0Node::reloc()   const {
  return  1;
}

int  mulF_immNode::reloc()   const {
  return  1;
}

int  mulD_memNode::reloc()   const {
  return  1;
}

int  mulD_mem_0Node::reloc()   const {
  return  1;
}

int  mulD_immNode::reloc()   const {
  return  1;
}

int  divF_memNode::reloc()   const {
  return  1;
}

int  divF_immNode::reloc()   const {
  return  1;
}

int  divD_memNode::reloc()   const {
  return  1;
}

int  divD_immNode::reloc()   const {
  return  1;
}

int  sqrtF_memNode::reloc()   const {
  return  1;
}

int  sqrtF_immNode::reloc()   const {
  return  1;
}

int  sqrtD_memNode::reloc()   const {
  return  1;
}

int  sqrtD_immNode::reloc()   const {
  return  1;
}

int  convF2D_reg_memNode::reloc()   const {
  return  1;
}

int  convD2F_reg_memNode::reloc()   const {
  return  1;
}

int  convI2F_reg_memNode::reloc()   const {
  return  1;
}

int  convI2D_reg_memNode::reloc()   const {
  return  1;
}

int  convL2F_reg_memNode::reloc()   const {
  return  1;
}

int  convL2D_reg_memNode::reloc()   const {
  return  1;
}

int  convI2L_reg_memNode::reloc()   const {
  return  1;
}

int  convI2L_reg_mem_zexNode::reloc()   const {
  return  1;
}

int  Repl2F_immF0Node::reloc()   const {
  return  1;
}

int  compI_rReg_memNode::reloc()   const {
  return  1;
}

int  testI_reg_memNode::reloc()   const {
  return  1;
}

int  testI_reg_mem_0Node::reloc()   const {
  return  1;
}

int  compU_rReg_memNode::reloc()   const {
  return  1;
}

int  compP_rReg_memNode::reloc()   const {
  return  1;
}

int  compP_mem_rRegNode::reloc()   const {
  return  1;
}

int  testP_regNode::reloc()   const {
  return  1;
}

int  testP_reg_memNode::reloc()   const {
  return  2;
}

int  compL_rReg_memNode::reloc()   const {
  return  1;
}

int  testL_reg_memNode::reloc()   const {
  return  1;
}

int  testL_reg_mem_0Node::reloc()   const {
  return  1;
}

int  partialSubtypeCheck_vs_ZeroNode::reloc()   const {
  return  1;
}

int  safePoint_pollNode::reloc()   const {
  return  1;
}

int  CallStaticJavaDirectNode::reloc()   const {
  return  1;
}

int  CallDynamicJavaDirectNode::reloc()   const {
  return  1;
}

int  CallRuntimeDirectNode::reloc()   const {
  return  1;
}

int  CallLeafDirectNode::reloc()   const {
  return  1;
}

int  CallLeafNoFPDirectNode::reloc()   const {
  return  1;
}

int  RetNode::reloc()   const {
  return  1;
}

int  TailCalljmpIndNode::reloc()   const {
  return  1;
}

int  tailjmpIndNode::reloc()   const {
  return  1;
}

int  RethrowExceptionNode::reloc()   const {
  return  1;
}


void  loadBNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xBE /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadUBNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB6 /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadSNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xBF /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadCNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB7 /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadRangeNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadKlassNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x10 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadD_partialNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf, (0x66 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x12 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x10 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadA8BNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(as_XMMRegister(opnd_array(0)->reg(ra_,this)), madr);
  }
}
void  loadA4SNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(as_XMMRegister(opnd_array(0)->reg(ra_,this)), madr);
  }
}
void  loadA4CNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(as_XMMRegister(opnd_array(0)->reg(ra_,this)), madr);
  }
}
void  load2IUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(as_XMMRegister(opnd_array(0)->reg(ra_,this)), madr);
  }
}
void  loadA2FNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(as_XMMRegister(opnd_array(0)->reg(ra_,this)), madr);
  }
}
void  leaP8Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8D /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  leaP32Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8D /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  leaPIdxOffNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8D /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  leaPIdxScaleNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8D /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  leaPIdxScaleOffNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8D /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadConINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    emit_d32(cbuf, opnd_array(1)->constant());
  }
}
void  loadConI0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x33 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  }
}
void  loadConLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    emit_d64(cbuf, opnd_array(1)->constantL());
  }
}
void  loadConL0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x33 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  }
}
void  loadConUL32Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // same as load_immI, but this time we care about zeroes in the high word
    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    emit_d32(cbuf, opnd_array(1)->constantL());
  }
}
void  loadConL32Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xC7);
    emit_rm(cbuf, 0x03, 0x00, dstenc);
    emit_d32(cbuf, opnd_array(1)->constantL());
  }
}
void  loadConPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    // This next line should be generated from ADLC
    if (opnd_array(1)->constant_is_oop()) {
      emit_d64_reloc(cbuf, opnd_array(1)->constant(), relocInfo::oop_type, RELOC_IMM64);
    } else {
      emit_d64(cbuf, opnd_array(1)->constant());
    }
  }
}
void  loadConP0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x33 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  }
}
void  loadConP31Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // same as load_immI, but this time we care about zeroes in the high word
    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    
if ( opnd_array(1)->constant_is_oop() ) {
  emit_d32_reloc(cbuf, opnd_array(1)->constant(), relocInfo::oop_type, 0);
} else {
  emit_d32(cbuf, opnd_array(1)->constant());
};
  }
}
void  loadConFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, 0xF3);
    if (opnd_array(0)->reg(ra_,this) >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x10);
    emit_rm(cbuf, 0x0, opnd_array(0)->reg(ra_,this) & 7, 0x5); // 00 reg 101
    emit_float_constant(cbuf, opnd_array(1)->constantF());
  }
}
void  loadConF0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x57 /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  }
}
void  loadConDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // UseXmmLoadAndClearUpper ? movsd(dst, con) : movlpd(dst, con)
    emit_opcode(cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    if (opnd_array(0)->reg(ra_,this) >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
    emit_rm(cbuf, 0x0, opnd_array(0)->reg(ra_,this) & 7, 0x5); // 00 reg 101
    emit_double_constant(cbuf, opnd_array(1)->constantD());
  }
}
void  loadConD0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0x66 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x57 /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  }
}
void  loadSSINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadSSLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadSSPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadSSFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x10 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadSSDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this), Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  }
}
void  prefetchrNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x0D /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  prefetchrNTANode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x18 /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  prefetchrT0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x18 /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x01);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  prefetchrT2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x18 /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x03);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  prefetchwNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x0D /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x01);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  prefetchwNTANode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x18 /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  prefetchwT0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x18 /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x01);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  prefetchwT2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x18 /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x03);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  storeBNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        } else if (opnd_array(2)->reg(ra_,this,idx2) >= 4) {
          emit_opcode(cbuf, Assembler::REX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x88 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storeCNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, 0x66);
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x89 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storeINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x89 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storeLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x89 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storePNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x89 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storeImmPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Output immediate
    
if ( opnd_array(2)->constant_is_oop() ) {
  emit_d32_reloc(cbuf, opnd_array(2)->constant(), relocInfo::oop_type, 0);
} else {
  emit_d32(cbuf, opnd_array(2)->constant());
};
  }
}
void  storeImmINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  }
}
void  storeImmLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constantL());
  }
}
void  storeImmI16Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, 0x66);
  }
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Output immediate
    emit_d16(cbuf, opnd_array(2)->constant());
  }
}
void  storeImmBNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC6 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  storeA8BNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(madr, as_XMMRegister(opnd_array(2)->reg(ra_,this,idx2)));
  }
}
void  storeA4CNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(madr, as_XMMRegister(opnd_array(2)->reg(ra_,this,idx2)));
  }
}
void  storeA2INode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(madr, as_XMMRegister(opnd_array(2)->reg(ra_,this,idx2)));
  }
}
void  storeImmCM0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC6 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  storeA2FNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(madr, as_XMMRegister(opnd_array(2)->reg(ra_,this,idx2)));
  }
}
void  storeFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x11 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storeF_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantF();
    jint jf_as_bits = jint_cast(jf);
    emit_d32(cbuf, jf_as_bits);
  }
}
void  storeDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x11 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storeD0_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantD();
    jint jf_as_bits = jint_cast(jf);
    emit_d32(cbuf, jf_as_bits);
  }
}
void  storeSSINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x89 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storeSSLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x89 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storeSSPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x89 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storeSSFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x11 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storeSSDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x11 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  bytes_reverse_intNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    // BSWAP
    emit_cc(cbuf, (0xC8 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  bytes_reverse_longNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    // BSWAP
    emit_cc(cbuf, (0xC8 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  loadI_reversedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
  {
    if (opnd_array(0)->reg(ra_,this) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    // BSWAP
    emit_cc(cbuf, (0xC8 /*tertiary()*/), opnd_array(0)->reg(ra_,this));
  }
}
void  loadL_reversedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    // BSWAP
    emit_cc(cbuf, (0xC8 /*tertiary()*/), opnd_array(0)->reg(ra_,this));
  }
}
void  storeI_reversedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    // BSWAP
    emit_cc(cbuf, (0xC8 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x89 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storeL_reversedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    // BSWAP
    emit_cc(cbuf, (0xC8 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x89 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  membar_acquireNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  membar_acquireNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  membar_acquire_lockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  membar_acquire_lockNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  membar_releaseNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  membar_releaseNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  membar_release_lockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  membar_release_lockNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  membar_volatileNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler masm(&cbuf);
    masm.membar(Assembler::Membar_mask_bits(Assembler::StoreLoad |
                                            Assembler::StoreStore));
  }
}
void  unnecessary_membar_volatileNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  unnecessary_membar_volatileNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  castX2PNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);
    int dstenc = opnd_array(0)->reg(ra_,this);
    
    if (dstenc != srcenc) {
      if (dstenc < 8) {
        if (srcenc < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WB);
          srcenc -= 8;
        }
      } else {
        if (srcenc < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRB);
          srcenc -= 8;
        }
        dstenc -= 8;
      }
      emit_opcode(cbuf, 0x8B);
      emit_rm(cbuf, 0x3, dstenc, srcenc);
    }
  }
}
void  castP2XNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);
    int dstenc = opnd_array(0)->reg(ra_,this);
    
    if (dstenc != srcenc) {
      if (dstenc < 8) {
        if (srcenc < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WB);
          srcenc -= 8;
        }
      } else {
        if (srcenc < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRB);
          srcenc -= 8;
        }
        dstenc -= 8;
      }
      emit_opcode(cbuf, 0x8B);
      emit_rm(cbuf, 0x3, dstenc, srcenc);
    }
  }
}
void  jumpXtnd_offsetNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler masm(&cbuf);

    Register switch_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1));
    Register dest_reg   = as_Register(opnd_array(3)->reg(ra_,this,idx3));
    address table_base  = masm.address_table_constant(_index2label);

    // We could use jump(ArrayAddress) except that the macro assembler needs to use r10
    // to do that and the compiler is using that register as one it can allocate.
    // So we build it all by hand.
    // Address index(noreg, switch_reg, (Address::ScaleFactor)$shift$$constant);
    // ArrayAddress dispatch(table, index);

    Address dispatch(dest_reg, switch_reg, (Address::ScaleFactor)opnd_array(2)->constant());
    masm.lea(dest_reg, InternalAddress(table_base));
    masm.jmp(dispatch);

  }
}
void  jumpXtnd_addrNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    MacroAssembler masm(&cbuf);

    Register switch_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1));
    Register dest_reg   = as_Register(opnd_array(4)->reg(ra_,this,idx4));
    address table_base  = masm.address_table_constant(_index2label);

    // We could use jump(ArrayAddress) except that the macro assembler needs to use r10
    // to do that and the compiler is using that register as one it can allocate.
    // So we build it all by hand.
    // Address index(noreg, switch_reg, (Address::ScaleFactor)$shift$$constant, (int)$offset$$constant);
    // ArrayAddress dispatch(table, index);

    Address dispatch(dest_reg, switch_reg, (Address::ScaleFactor)opnd_array(2)->constant(), (int)opnd_array(3)->constantL());

    masm.lea(dest_reg, InternalAddress(table_base));
    masm.jmp(dispatch);
  }
}
void  jumpXtndNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler masm(&cbuf);

    Register switch_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1));
    Register dest_reg   = as_Register(opnd_array(2)->reg(ra_,this,idx2));
    address table_base  = masm.address_table_constant(_index2label);

    // We could use jump(ArrayAddress) except that the macro assembler needs to use r10
    // to do that and the compiler is using that register as one it can allocate.
    // So we build it all by hand.
    // Address index(noreg, switch_reg, Address::times_1);
    // ArrayAddress dispatch(table, index);

    Address dispatch(dest_reg, switch_reg, Address::times_1);

    masm.lea(dest_reg, InternalAddress(table_base));
    masm.jmp(dispatch);
  }
}
void  cmovI_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  }
}
void  cmovI_regUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  }
}
void  cmovI_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int disp = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  cmovI_memUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int disp = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  cmovP_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  }
}
void  cmovP_regUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  }
}
void  cmovL_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  }
}
void  cmovL_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int disp = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  cmovL_regUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  }
}
void  cmovL_memUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int disp = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  cmovF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // Invert sense of branch from sense of cmov
    emit_cc(cbuf, 0x70, opnd_array(1)->ccode() ^ 1);
    emit_d8(cbuf, (opnd_array(3)->reg(ra_,this,idx3) < 8 && opnd_array(4)->reg(ra_,this,idx4) < 8)
                  ? (UseXmmRegToRegMoveAll ? 3 : 4)
                  : (UseXmmRegToRegMoveAll ? 4 : 5) ); // REX
    // UseXmmRegToRegMoveAll ? movaps(dst, src) : movss(dst, src)
    if (!UseXmmRegToRegMoveAll) emit_opcode(cbuf, 0xF3);
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, UseXmmRegToRegMoveAll ? 0x28 : 0x10);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  }
}
void  cmovF_regUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // Invert sense of branch from sense of cmov
    emit_cc(cbuf, 0x70, opnd_array(1)->ccode() ^ 1);
    emit_d8(cbuf, (opnd_array(3)->reg(ra_,this,idx3) < 8 && opnd_array(4)->reg(ra_,this,idx4) < 8)
                  ? (UseXmmRegToRegMoveAll ? 3 : 4)
                  : (UseXmmRegToRegMoveAll ? 4 : 5) ); // REX
    // UseXmmRegToRegMoveAll ? movaps(dst, src) : movss(dst, src)
    if (!UseXmmRegToRegMoveAll) emit_opcode(cbuf, 0xF3);
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, UseXmmRegToRegMoveAll ? 0x28 : 0x10);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  }
}
void  cmovD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // Invert sense of branch from sense of cmov
    emit_cc(cbuf, 0x70, opnd_array(1)->ccode() ^ 1);
    emit_d8(cbuf, opnd_array(3)->reg(ra_,this,idx3) < 8 && opnd_array(4)->reg(ra_,this,idx4) < 8 ? 4 : 5); // REX

    //  UseXmmRegToRegMoveAll ? movapd(dst, src) : movsd(dst, src)
    emit_opcode(cbuf, UseXmmRegToRegMoveAll ? 0x66 : 0xF2);
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, UseXmmRegToRegMoveAll ? 0x28 : 0x10);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  }
}
void  cmovD_regUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // Invert sense of branch from sense of cmov
    emit_cc(cbuf, 0x70, opnd_array(1)->ccode() ^ 1);
    emit_d8(cbuf, opnd_array(3)->reg(ra_,this,idx3) < 8 && opnd_array(4)->reg(ra_,this,idx4) < 8 ? 4 : 5); // REX

    //  UseXmmRegToRegMoveAll ? movapd(dst, src) : movsd(dst, src)
    emit_opcode(cbuf, UseXmmRegToRegMoveAll ? 0x66 : 0xF2);
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, UseXmmRegToRegMoveAll ? 0x28 : 0x10);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  }
}
void  addI_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x03 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  addI_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  addI_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x03 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  addI_rReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x03 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  addI_mem_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x01 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  addI_mem_rReg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x01 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  addI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  incI_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xFF /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  incI_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xFF /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  decI_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xFF /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  decI_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xFF /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x01);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  leaI_rReg_immINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0x67));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8D /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base         = opnd_array(1)->reg(ra_,this,idx1);      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constant(); // 0x00 indicates no displacement
    bool disp_is_oop = false;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  addL_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x03 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  addL_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  addL_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x03 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  addL_rReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x03 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  addL_mem_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x01 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  addL_mem_rReg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x01 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  addL_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  incL_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xFF /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  incL_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xFF /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  decL_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xFF /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  decL_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xFF /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x01);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  leaL_rReg_immLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8D /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base         = opnd_array(1)->reg(ra_,this,idx1);      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constantL(); // 0x00 indicates no displacement
    bool disp_is_oop = false;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  addP_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x03 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  addP_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  leaP_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8D /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base         = opnd_array(1)->reg(ra_,this,idx1);      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constantL(); // 0x00 indicates no displacement
    bool disp_is_oop = false;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  checkCastPPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  checkCastPPNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  castPPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  castPPNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  castIINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  castIINode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  loadPLockedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  loadLLockedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storePConditionalNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  }
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  storeLConditionalNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  }
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
  {
    if (opnd_array(0)->reg(ra_,this) >= 4) {
      emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0x0F));
  }
  {
    emit_opcode(cbuf, (0x94));
  }
  {
    emit_rm(cbuf, 0x3, 0, opnd_array(0)->reg(ra_,this) & 7);
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xF));
  }
  {
    emit_opcode(cbuf, (0xB6));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  }
}
void  storeLConditional_flagsNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  }
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  compareAndSwapPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  }
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
  {
    if (opnd_array(0)->reg(ra_,this) >= 4) {
      emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0x0F));
  }
  {
    emit_opcode(cbuf, (0x94));
  }
  {
    emit_rm(cbuf, 0x3, 0, opnd_array(0)->reg(ra_,this) & 7);
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xF));
  }
  {
    emit_opcode(cbuf, (0xB6));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  }
}
void  compareAndSwapLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  }
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
  {
    if (opnd_array(0)->reg(ra_,this) >= 4) {
      emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0x0F));
  }
  {
    emit_opcode(cbuf, (0x94));
  }
  {
    emit_rm(cbuf, 0x3, 0, opnd_array(0)->reg(ra_,this) & 7);
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xF));
  }
  {
    emit_opcode(cbuf, (0xB6));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  }
}
void  compareAndSwapINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  }
  {
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
  {
    if (opnd_array(0)->reg(ra_,this) >= 4) {
      emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0x0F));
  }
  {
    emit_opcode(cbuf, (0x94));
  }
  {
    emit_rm(cbuf, 0x3, 0, opnd_array(0)->reg(ra_,this) & 7);
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xF));
  }
  {
    emit_opcode(cbuf, (0xB6));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  }
}
void  subI_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x2B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  subI_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  subI_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x2B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  subI_mem_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x29 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  subI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x05);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  subL_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x2B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  subL_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  subL_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x2B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  subL_mem_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x29 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  subL_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x05);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  subP_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x2B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(3)->reg(ra_,this,idx3) & 7);
  }
}
void  negI_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x03 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  negI_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x03 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  negL_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x03 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  negL_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x03 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  mulI_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xAF /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  mulI_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  mulI_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xAF /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  mulI_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xAF /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  mulI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  mulL_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xAF /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  mulL_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  mulL_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xAF /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  mulL_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xAF /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  mulL_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  divI_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_int
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //    
    //    0:   3d 00 00 00 80          cmp    $0x80000000,%eax
    //    5:   75 07/08                jne    e <normal>
    //    7:   33 d2                   xor    %edx,%edx
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    9:   83 f9 ff                cmp    $0xffffffffffffffff,$div
    //    c:   74 03/04                je     11 <done>
    // 000000000000000e <normal>:
    //    e:   99                      cltd   
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    f:   f7 f9                   idiv   $div
    // 0000000000000011 <done>:

    // cmp    $0x80000000,%eax
    emit_opcode(cbuf, 0x3d);
    emit_d8(cbuf, 0x00);      
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // jne    e <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? 0x07 : 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,%ecx
    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2) & 7);
    emit_d8(cbuf, 0xFF);

    // je     11 <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? 0x03 : 0x04);

    // <normal>
    // cltd   
    emit_opcode(cbuf, 0x99);

    // idivl (note: must be emitted by the user of this rule)
    // <done>
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  divL_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Full implementation of Java ldiv and lrem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_long
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_long
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //    
    //    0:   48 ba 00 00 00 00 00    mov    $0x8000000000000000,%rdx
    //    7:   00 00 80 
    //    a:   48 39 d0                cmp    %rdx,%rax
    //    d:   75 08                   jne    17 <normal>
    //    f:   33 d2                   xor    %edx,%edx
    //   11:   48 83 f9 ff             cmp    $0xffffffffffffffff,$div
    //   15:   74 05                   je     1c <done>
    // 0000000000000017 <normal>:
    //   17:   48 99                   cqto   
    //   19:   48 f7 f9                idiv   $div
    // 000000000000001c <done>:

    // mov    $0x8000000000000000,%rdx
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0xBA);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // cmp    %rdx,%rax
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x39);
    emit_d8(cbuf, 0xD0);

    // jne    17 <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,$div
    emit_opcode(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? Assembler::REX_W : Assembler::REX_WB);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2) & 7);
    emit_d8(cbuf, 0xFF);

    // je     1e <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x05);

    // <normal>
    // cqto   
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x99);

    // idivq (note: must be emitted by the user of this rule)
    // <done>
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  divModI_rReg_divmodNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_int
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //    
    //    0:   3d 00 00 00 80          cmp    $0x80000000,%eax
    //    5:   75 07/08                jne    e <normal>
    //    7:   33 d2                   xor    %edx,%edx
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    9:   83 f9 ff                cmp    $0xffffffffffffffff,$div
    //    c:   74 03/04                je     11 <done>
    // 000000000000000e <normal>:
    //    e:   99                      cltd   
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    f:   f7 f9                   idiv   $div
    // 0000000000000011 <done>:

    // cmp    $0x80000000,%eax
    emit_opcode(cbuf, 0x3d);
    emit_d8(cbuf, 0x00);      
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // jne    e <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? 0x07 : 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,%ecx
    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2) & 7);
    emit_d8(cbuf, 0xFF);

    // je     11 <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? 0x03 : 0x04);

    // <normal>
    // cltd   
    emit_opcode(cbuf, 0x99);

    // idivl (note: must be emitted by the user of this rule)
    // <done>
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  divModL_rReg_divmodNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Full implementation of Java ldiv and lrem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_long
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_long
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //    
    //    0:   48 ba 00 00 00 00 00    mov    $0x8000000000000000,%rdx
    //    7:   00 00 80 
    //    a:   48 39 d0                cmp    %rdx,%rax
    //    d:   75 08                   jne    17 <normal>
    //    f:   33 d2                   xor    %edx,%edx
    //   11:   48 83 f9 ff             cmp    $0xffffffffffffffff,$div
    //   15:   74 05                   je     1c <done>
    // 0000000000000017 <normal>:
    //   17:   48 99                   cqto   
    //   19:   48 f7 f9                idiv   $div
    // 000000000000001c <done>:

    // mov    $0x8000000000000000,%rdx
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0xBA);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // cmp    %rdx,%rax
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x39);
    emit_d8(cbuf, 0xD0);

    // jne    17 <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,$div
    emit_opcode(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? Assembler::REX_W : Assembler::REX_WB);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2) & 7);
    emit_d8(cbuf, 0xFF);

    // je     1e <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x05);

    // <normal>
    // cqto   
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x99);

    // idivq (note: must be emitted by the user of this rule)
    // <done>
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  loadConL_0x6666666666666667Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    emit_d64(cbuf, (0x6666666666666667));
  }
}
void  mul_hiNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  sarL_rReg_63Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, (0x3F));
  }
}
void  sarL_rReg_2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, (0x2));
  }
}
void  modI_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_int
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //    
    //    0:   3d 00 00 00 80          cmp    $0x80000000,%eax
    //    5:   75 07/08                jne    e <normal>
    //    7:   33 d2                   xor    %edx,%edx
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    9:   83 f9 ff                cmp    $0xffffffffffffffff,$div
    //    c:   74 03/04                je     11 <done>
    // 000000000000000e <normal>:
    //    e:   99                      cltd   
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    f:   f7 f9                   idiv   $div
    // 0000000000000011 <done>:

    // cmp    $0x80000000,%eax
    emit_opcode(cbuf, 0x3d);
    emit_d8(cbuf, 0x00);      
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // jne    e <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? 0x07 : 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,%ecx
    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2) & 7);
    emit_d8(cbuf, 0xFF);

    // je     11 <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? 0x03 : 0x04);

    // <normal>
    // cltd   
    emit_opcode(cbuf, 0x99);

    // idivl (note: must be emitted by the user of this rule)
    // <done>
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  modL_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Full implementation of Java ldiv and lrem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_long
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_long
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //    
    //    0:   48 ba 00 00 00 00 00    mov    $0x8000000000000000,%rdx
    //    7:   00 00 80 
    //    a:   48 39 d0                cmp    %rdx,%rax
    //    d:   75 08                   jne    17 <normal>
    //    f:   33 d2                   xor    %edx,%edx
    //   11:   48 83 f9 ff             cmp    $0xffffffffffffffff,$div
    //   15:   74 05                   je     1c <done>
    // 0000000000000017 <normal>:
    //   17:   48 99                   cqto   
    //   19:   48 f7 f9                idiv   $div
    // 000000000000001c <done>:

    // mov    $0x8000000000000000,%rdx
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0xBA);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // cmp    %rdx,%rax
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x39);
    emit_d8(cbuf, 0xD0);

    // jne    17 <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,$div
    emit_opcode(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? Assembler::REX_W : Assembler::REX_WB);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2) & 7);
    emit_d8(cbuf, 0xFF);

    // je     1e <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x05);

    // <normal>
    // cqto   
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x99);

    // idivq (note: must be emitted by the user of this rule)
    // <done>
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  salI_rReg_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  salI_mem_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  salI_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
void  salI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  salI_rReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  salI_mem_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  sarI_rReg_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  sarI_mem_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  sarI_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
void  sarI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  sarI_rReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  sarI_mem_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  shrI_rReg_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  shrI_mem_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  shrI_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
void  shrI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  shrI_rReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  shrI_mem_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  salL_rReg_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  salL_mem_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  salL_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
void  salL_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  salL_rReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  salL_mem_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  sarL_rReg_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  sarL_mem_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  sarL_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
void  sarL_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  sarL_rReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  sarL_mem_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  shrL_rReg_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  shrL_mem_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  shrL_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
void  shrL_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xC1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  shrL_rReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  shrL_mem_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
}
void  i2bNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 4) {
        emit_opcode(cbuf, opnd_array(1)->reg(ra_,this,idx1) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xBE /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  i2sNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xBF /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  rolI_rReg_imm1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  rolI_rReg_imm8Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
void  rolI_rReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  rorI_rReg_imm1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  rorI_rReg_imm8Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
void  rorI_rReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  rolL_rReg_imm1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  rolL_rReg_imm8Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
void  rolL_rReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  rorL_rReg_imm1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xD1 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  rorL_rReg_imm8Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
void  rorL_rReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xD3 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  andI_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x23 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  andI_rReg_imm255Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 4) {
        emit_opcode(cbuf, opnd_array(1)->reg(ra_,this,idx1) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB6 /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  andI2L_rReg_imm255Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 4) {
        emit_opcode(cbuf, opnd_array(1)->reg(ra_,this,idx1) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB6 /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  andI_rReg_imm65535Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB7 /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  andI2L_rReg_imm65535Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB7 /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  andI_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  andI_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x23 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  andI_rReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x23 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  andI_mem_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x21 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  andI_mem_rReg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x21 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  andI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  orI_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  orI_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  orI_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  orI_rReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  orI_mem_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x09 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  orI_mem_rReg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x09 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  orI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x1 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  xorI_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x33 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  xorI_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  xorI_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x33 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  xorI_rReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x33 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  xorI_mem_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x31 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  xorI_mem_rReg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x31 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  xorI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x6 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  andL_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x23 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  andL_rReg_imm255Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB6 /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  andL_rReg_imm65535Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xB7 /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  andL_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  andL_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x23 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  andL_rReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x23 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  andL_mem_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x21 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  andL_mem_rReg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x21 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  andL_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  orL_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  orL_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  orL_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  orL_rReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  orL_mem_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x09 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  orL_mem_rReg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x09 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  orL_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x1 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  xorL_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x33 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  xorL_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  xorL_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x33 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  xorL_rReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x33 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  xorL_mem_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x31 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  xorL_mem_rReg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x31 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  xorL_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x6 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  convI2BNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x85));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    } else if (dstenc >= 4) {
      emit_opcode(cbuf, Assembler::REX);
    }
    // SETNZ $dst
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_rm(cbuf, 0x3, 0x0, dstenc);
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F));
  }
  {
    emit_opcode(cbuf, (0xB6));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  }
}
void  convP2BNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x85));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    } else if (dstenc >= 4) {
      emit_opcode(cbuf, Assembler::REX);
    }
    // SETNZ $dst
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_rm(cbuf, 0x3, 0x0, dstenc);
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F));
  }
  {
    emit_opcode(cbuf, (0xB6));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  }
}
void  cmpLTMaskNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x3B));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    } else if (dstenc >= 4) {
      emit_opcode(cbuf, Assembler::REX);
    }
    // SETLT $dst
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x9C);
    emit_rm(cbuf, 0x3, 0x0, dstenc);
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F));
  }
  {
    emit_opcode(cbuf, (0xB6));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // NEG $dst
    emit_opcode(cbuf, 0xF7);
    emit_rm(cbuf, 0x3, 0x03, dstenc);
  }
}
void  cmpLTMask0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, (0x1F));
  }
}
void  cadd_cmpLTMaskNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // cadd_cmpLT

    int tmpReg = opnd_array(4)->reg(ra_,this,idx4);

    int penc = opnd_array(1)->reg(ra_,this,idx1);
    int qenc = opnd_array(2)->reg(ra_,this,idx2);
    int yenc = opnd_array(3)->reg(ra_,this,idx3);

    // subl $p,$q
    if (penc < 8) {
      if (qenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (qenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x2B);
    emit_rm(cbuf, 0x3, penc & 7, qenc & 7);

    // sbbl $tmp, $tmp
    emit_opcode(cbuf, 0x1B);
    emit_rm(cbuf, 0x3, tmpReg, tmpReg);

    // andl $tmp, $y
    if (yenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x23);
    emit_rm(cbuf, 0x3, tmpReg, yenc & 7);

    // addl $p,$tmp
    if (penc >= 8) {
        emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x03);
    emit_rm(cbuf, 0x3, penc & 7, tmpReg);
  }
}
void  cadd_cmpLTMask_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // cadd_cmpLT

    int tmpReg = opnd_array(4)->reg(ra_,this,idx4);

    int penc = opnd_array(1)->reg(ra_,this,idx1);
    int qenc = opnd_array(2)->reg(ra_,this,idx2);
    int yenc = opnd_array(3)->reg(ra_,this,idx3);

    // subl $p,$q
    if (penc < 8) {
      if (qenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (qenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x2B);
    emit_rm(cbuf, 0x3, penc & 7, qenc & 7);

    // sbbl $tmp, $tmp
    emit_opcode(cbuf, 0x1B);
    emit_rm(cbuf, 0x3, tmpReg, tmpReg);

    // andl $tmp, $y
    if (yenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x23);
    emit_rm(cbuf, 0x3, tmpReg, yenc & 7);

    // addl $p,$tmp
    if (penc >= 8) {
        emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x03);
    emit_rm(cbuf, 0x3, penc & 7, tmpReg);
  }
}
void  cadd_cmpLTMask_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // cadd_cmpLT

    int tmpReg = opnd_array(4)->reg(ra_,this,idx4);

    int penc = opnd_array(2)->reg(ra_,this,idx2);
    int qenc = opnd_array(3)->reg(ra_,this,idx3);
    int yenc = opnd_array(1)->reg(ra_,this,idx1);

    // subl $p,$q
    if (penc < 8) {
      if (qenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (qenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x2B);
    emit_rm(cbuf, 0x3, penc & 7, qenc & 7);

    // sbbl $tmp, $tmp
    emit_opcode(cbuf, 0x1B);
    emit_rm(cbuf, 0x3, tmpReg, tmpReg);

    // andl $tmp, $y
    if (yenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x23);
    emit_rm(cbuf, 0x3, tmpReg, yenc & 7);

    // addl $p,$tmp
    if (penc >= 8) {
        emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x03);
    emit_rm(cbuf, 0x3, penc & 7, tmpReg);
  }
}
void  cadd_cmpLTMask_2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // cadd_cmpLT

    int tmpReg = opnd_array(4)->reg(ra_,this,idx4);

    int penc = opnd_array(1)->reg(ra_,this,idx1);
    int qenc = opnd_array(2)->reg(ra_,this,idx2);
    int yenc = opnd_array(3)->reg(ra_,this,idx3);

    // subl $p,$q
    if (penc < 8) {
      if (qenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (qenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x2B);
    emit_rm(cbuf, 0x3, penc & 7, qenc & 7);

    // sbbl $tmp, $tmp
    emit_opcode(cbuf, 0x1B);
    emit_rm(cbuf, 0x3, tmpReg, tmpReg);

    // andl $tmp, $y
    if (yenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x23);
    emit_rm(cbuf, 0x3, tmpReg, yenc & 7);

    // addl $p,$tmp
    if (penc >= 8) {
        emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x03);
    emit_rm(cbuf, 0x3, penc & 7, tmpReg);
  }
}
void  cmpF_cc_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x2E /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
  {
    // jnp,s exit
    emit_opcode(cbuf, 0x7B);
    emit_d8(cbuf, 0x0A);

    // pushfq
    emit_opcode(cbuf, 0x9C);

    // andq $0xffffff2b, (%rsp)
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x81);
    emit_opcode(cbuf, 0x24);
    emit_opcode(cbuf, 0x24);
    emit_d32(cbuf, 0xffffff2b);

    // popfq
    emit_opcode(cbuf, 0x9D);

    // nop (target for branch to avoid branch to branch)
    emit_opcode(cbuf, 0x90);
  }
}
void  cmpF_cc_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x2E /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
  {
    // jnp,s exit
    emit_opcode(cbuf, 0x7B);
    emit_d8(cbuf, 0x0A);

    // pushfq
    emit_opcode(cbuf, 0x9C);

    // andq $0xffffff2b, (%rsp)
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x81);
    emit_opcode(cbuf, 0x24);
    emit_opcode(cbuf, 0x24);
    emit_d32(cbuf, 0xffffff2b);

    // popfq
    emit_opcode(cbuf, 0x9D);

    // nop (target for branch to avoid branch to branch)
    emit_opcode(cbuf, 0x90);
  }
}
void  cmpF_cc_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x2E /*secondary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1) & 7, 0x5); // 00 reg 101
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
  {
    // jnp,s exit
    emit_opcode(cbuf, 0x7B);
    emit_d8(cbuf, 0x0A);

    // pushfq
    emit_opcode(cbuf, 0x9C);

    // andq $0xffffff2b, (%rsp)
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x81);
    emit_opcode(cbuf, 0x24);
    emit_opcode(cbuf, 0x24);
    emit_d32(cbuf, 0xffffff2b);

    // popfq
    emit_opcode(cbuf, 0x9D);

    // nop (target for branch to avoid branch to branch)
    emit_opcode(cbuf, 0x90);
  }
}
void  cmpD_cc_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0x66 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2E /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
  {
    // jnp,s exit
    emit_opcode(cbuf, 0x7B);
    emit_d8(cbuf, 0x0A);

    // pushfq
    emit_opcode(cbuf, 0x9C);

    // andq $0xffffff2b, (%rsp)
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x81);
    emit_opcode(cbuf, 0x24);
    emit_opcode(cbuf, 0x24);
    emit_d32(cbuf, 0xffffff2b);

    // popfq
    emit_opcode(cbuf, 0x9D);

    // nop (target for branch to avoid branch to branch)
    emit_opcode(cbuf, 0x90);
  }
}
void  cmpD_cc_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0x66 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2E /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
  {
    // jnp,s exit
    emit_opcode(cbuf, 0x7B);
    emit_d8(cbuf, 0x0A);

    // pushfq
    emit_opcode(cbuf, 0x9C);

    // andq $0xffffff2b, (%rsp)
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x81);
    emit_opcode(cbuf, 0x24);
    emit_opcode(cbuf, 0x24);
    emit_d32(cbuf, 0xffffff2b);

    // popfq
    emit_opcode(cbuf, 0x9D);

    // nop (target for branch to avoid branch to branch)
    emit_opcode(cbuf, 0x90);
  }
}
void  cmpD_cc_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0x66 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2E /*tertiary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1) & 7, 0x5); // 00 reg 101
    emit_double_constant(cbuf, opnd_array(2)->constantD());
  }
  {
    // jnp,s exit
    emit_opcode(cbuf, 0x7B);
    emit_d8(cbuf, 0x0A);

    // pushfq
    emit_opcode(cbuf, 0x9C);

    // andq $0xffffff2b, (%rsp)
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x81);
    emit_opcode(cbuf, 0x24);
    emit_opcode(cbuf, 0x24);
    emit_d32(cbuf, 0xffffff2b);

    // popfq
    emit_opcode(cbuf, 0x9D);

    // nop (target for branch to avoid branch to branch)
    emit_opcode(cbuf, 0x90);
  }
}
void  cmpF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x2E /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);

    // movl $dst, -1
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0xB8 | (dstenc & 7));
    emit_d32(cbuf, -1);

    // jp,s done
    emit_opcode(cbuf, 0x7A);
    emit_d8(cbuf, dstenc < 4 ? 0x08 : 0x0A);

    // jb,s done
    emit_opcode(cbuf, 0x72);
    emit_d8(cbuf, dstenc < 4 ? 0x06 : 0x08);

    // setne $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_opcode(cbuf, 0xC0 | (dstenc & 7));

    // movzbl $dst, $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_RB);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0xB6);
    emit_rm(cbuf, 0x3, dstenc & 7, dstenc & 7);
  }
}
void  cmpF_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x2E /*secondary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);

    // movl $dst, -1
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0xB8 | (dstenc & 7));
    emit_d32(cbuf, -1);

    // jp,s done
    emit_opcode(cbuf, 0x7A);
    emit_d8(cbuf, dstenc < 4 ? 0x08 : 0x0A);

    // jb,s done
    emit_opcode(cbuf, 0x72);
    emit_d8(cbuf, dstenc < 4 ? 0x06 : 0x08);

    // setne $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_opcode(cbuf, 0xC0 | (dstenc & 7));

    // movzbl $dst, $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_RB);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0xB6);
    emit_rm(cbuf, 0x3, dstenc & 7, dstenc & 7);
  }
}
void  cmpF_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x2E /*secondary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1) & 7, 0x5); // 00 reg 101
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);

    // movl $dst, -1
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0xB8 | (dstenc & 7));
    emit_d32(cbuf, -1);

    // jp,s done
    emit_opcode(cbuf, 0x7A);
    emit_d8(cbuf, dstenc < 4 ? 0x08 : 0x0A);

    // jb,s done
    emit_opcode(cbuf, 0x72);
    emit_d8(cbuf, dstenc < 4 ? 0x06 : 0x08);

    // setne $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_opcode(cbuf, 0xC0 | (dstenc & 7));

    // movzbl $dst, $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_RB);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0xB6);
    emit_rm(cbuf, 0x3, dstenc & 7, dstenc & 7);
  }
}
void  cmpD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0x66 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2E /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);

    // movl $dst, -1
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0xB8 | (dstenc & 7));
    emit_d32(cbuf, -1);

    // jp,s done
    emit_opcode(cbuf, 0x7A);
    emit_d8(cbuf, dstenc < 4 ? 0x08 : 0x0A);

    // jb,s done
    emit_opcode(cbuf, 0x72);
    emit_d8(cbuf, dstenc < 4 ? 0x06 : 0x08);

    // setne $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_opcode(cbuf, 0xC0 | (dstenc & 7));

    // movzbl $dst, $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_RB);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0xB6);
    emit_rm(cbuf, 0x3, dstenc & 7, dstenc & 7);
  }
}
void  cmpD_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0x66 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2E /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);

    // movl $dst, -1
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0xB8 | (dstenc & 7));
    emit_d32(cbuf, -1);

    // jp,s done
    emit_opcode(cbuf, 0x7A);
    emit_d8(cbuf, dstenc < 4 ? 0x08 : 0x0A);

    // jb,s done
    emit_opcode(cbuf, 0x72);
    emit_d8(cbuf, dstenc < 4 ? 0x06 : 0x08);

    // setne $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_opcode(cbuf, 0xC0 | (dstenc & 7));

    // movzbl $dst, $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_RB);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0xB6);
    emit_rm(cbuf, 0x3, dstenc & 7, dstenc & 7);
  }
}
void  cmpD_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0x66 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2E /*tertiary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1) & 7, 0x5); // 00 reg 101
    emit_double_constant(cbuf, opnd_array(2)->constantD());
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);

    // movl $dst, -1
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0xB8 | (dstenc & 7));
    emit_d32(cbuf, -1);

    // jp,s done
    emit_opcode(cbuf, 0x7A);
    emit_d8(cbuf, dstenc < 4 ? 0x08 : 0x0A);

    // jb,s done
    emit_opcode(cbuf, 0x72);
    emit_d8(cbuf, dstenc < 4 ? 0x06 : 0x08);

    // setne $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_opcode(cbuf, 0xC0 | (dstenc & 7));

    // movzbl $dst, $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_RB);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0xB6);
    emit_rm(cbuf, 0x3, dstenc & 7, dstenc & 7);
  }
}
void  addF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  addF_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  addF_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  addF_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1) & 7, 0x5); // 00 reg 101
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
}
void  addD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  addD_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  addD_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  addD_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1) & 7, 0x5); // 00 reg 101
    emit_double_constant(cbuf, opnd_array(2)->constantD());
  }
}
void  subF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5C /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  subF_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5C /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  subF_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5C /*tertiary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1) & 7, 0x5); // 00 reg 101
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
}
void  subD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5C /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  subD_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5C /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  subD_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5C /*tertiary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1) & 7, 0x5); // 00 reg 101
    emit_double_constant(cbuf, opnd_array(2)->constantD());
  }
}
void  mulF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  mulF_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  mulF_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  mulF_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1) & 7, 0x5); // 00 reg 101
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
}
void  mulD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  mulD_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  mulD_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  mulD_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1) & 7, 0x5); // 00 reg 101
    emit_double_constant(cbuf, opnd_array(2)->constantD());
  }
}
void  divF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5E /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  divF_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5E /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  divF_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5E /*tertiary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1) & 7, 0x5); // 00 reg 101
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
}
void  divD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5E /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  divD_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5E /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  divD_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5E /*tertiary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1) & 7, 0x5); // 00 reg 101
    emit_double_constant(cbuf, opnd_array(2)->constantD());
  }
}
void  sqrtF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x51 /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  sqrtF_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x51 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  sqrtF_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x51 /*tertiary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(0)->reg(ra_,this) & 7, 0x5); // 00 reg 101
    emit_float_constant(cbuf, opnd_array(1)->constantF());
  }
}
void  sqrtD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x51 /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  sqrtD_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x51 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  sqrtD_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x51 /*tertiary()*/));
  }
  {
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, opnd_array(0)->reg(ra_,this) & 7, 0x5); // 00 reg 101
    emit_double_constant(cbuf, opnd_array(1)->constantD());
  }
}
void  absF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    address signmask_address = (address) StubRoutines::amd64::float_sign_mask();

    cbuf.set_inst_mark();
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
      dstenc -= 8;
    }
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x54);
    emit_rm(cbuf, 0x0, dstenc, 0x5);  // 00 reg 101
    emit_d32_reloc(cbuf, signmask_address);
  }
}
void  absD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    address signmask_address = (address) StubRoutines::amd64::double_sign_mask();

    cbuf.set_inst_mark();
    emit_opcode(cbuf, 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
      dstenc -= 8;
    }
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x54);
    emit_rm(cbuf, 0x0, dstenc, 0x5);  // 00 reg 101
    emit_d32_reloc(cbuf, signmask_address);
  }
}
void  negF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    address signflip_address = (address) StubRoutines::amd64::float_sign_flip();

    cbuf.set_inst_mark();
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
      dstenc -= 8;
    }
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x57);
    emit_rm(cbuf, 0x0, dstenc, 0x5);  // 00 reg 101
    emit_d32_reloc(cbuf, signflip_address);
  }
}
void  negD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    address signflip_address = (address) StubRoutines::amd64::double_sign_flip();

    cbuf.set_inst_mark();
    emit_opcode(cbuf, 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
      dstenc -= 8;
    }
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x57);
    emit_rm(cbuf, 0x0, dstenc, 0x5);  // 00 reg 101
    emit_d32_reloc(cbuf, signflip_address);
  }
}
void  cosD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // subq rsp,#8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);           
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 0x8);

    // movsd [rsp],src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false);

    // fldd [rsp]
    emit_opcode(cbuf, 0x66);
    emit_opcode(cbuf, 0xDD);
    encode_RegMem(cbuf, 0x0, RSP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf, (0xD9 /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xFF /*secondary()*/));
  }
  {
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);

    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [RSP]

    // UseXmmLoadAndClearUpper ? movsd dst,[rsp] : movlpd dst,[rsp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12 );
    encode_RegMem(cbuf, dstenc, RSP_enc, 0x4, 0, 0, false);

    // add rsp,8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf,0x83);
    emit_rm(cbuf,0x3, 0x0, RSP_enc);
    emit_d8(cbuf,0x08);
  }
}
void  sinD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // subq rsp,#8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);           
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 0x8);

    // movsd [rsp],src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false);

    // fldd [rsp]
    emit_opcode(cbuf, 0x66);
    emit_opcode(cbuf, 0xDD);
    encode_RegMem(cbuf, 0x0, RSP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf, (0xD9 /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0xFE /*secondary()*/));
  }
  {
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);

    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [RSP]

    // UseXmmLoadAndClearUpper ? movsd dst,[rsp] : movlpd dst,[rsp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12 );
    encode_RegMem(cbuf, dstenc, RSP_enc, 0x4, 0, 0, false);

    // add rsp,8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf,0x83);
    emit_rm(cbuf,0x3, 0x0, RSP_enc);
    emit_d8(cbuf,0x08);
  }
}
void  tanD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // subq rsp,#8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);           
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 0x8);

    // movsd [rsp],src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false);

    // fldd [rsp]
    emit_opcode(cbuf, 0x66);
    emit_opcode(cbuf, 0xDD);
    encode_RegMem(cbuf, 0x0, RSP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf, (0xD9));
  }
  {
    emit_opcode(cbuf, (0xF2));
  }
  {
    emit_opcode(cbuf, (0xDD));
  }
  {
    emit_opcode(cbuf, (0xD8));
  }
  {
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);

    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [RSP]

    // UseXmmLoadAndClearUpper ? movsd dst,[rsp] : movlpd dst,[rsp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12 );
    encode_RegMem(cbuf, dstenc, RSP_enc, 0x4, 0, 0, false);

    // add rsp,8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf,0x83);
    emit_rm(cbuf,0x3, 0x0, RSP_enc);
    emit_d8(cbuf,0x08);
  }
}
void  log10D_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xD9));
  }
  {
    emit_opcode(cbuf, (0xEC));
  }
  {
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // subq rsp,#8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);           
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 0x8);

    // movsd [rsp],src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false);

    // fldd [rsp]
    emit_opcode(cbuf, 0x66);
    emit_opcode(cbuf, 0xDD);
    encode_RegMem(cbuf, 0x0, RSP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf, (0xD9));
  }
  {
    emit_opcode(cbuf, (0xF1));
  }
  {
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);

    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [RSP]

    // UseXmmLoadAndClearUpper ? movsd dst,[rsp] : movlpd dst,[rsp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12 );
    encode_RegMem(cbuf, dstenc, RSP_enc, 0x4, 0, 0, false);

    // add rsp,8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf,0x83);
    emit_rm(cbuf,0x3, 0x0, RSP_enc);
    emit_d8(cbuf,0x08);
  }
}
void  logD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xD9));
  }
  {
    emit_opcode(cbuf, (0xED));
  }
  {
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // subq rsp,#8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);           
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 0x8);

    // movsd [rsp],src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false);

    // fldd [rsp]
    emit_opcode(cbuf, 0x66);
    emit_opcode(cbuf, 0xDD);
    encode_RegMem(cbuf, 0x0, RSP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf, (0xD9));
  }
  {
    emit_opcode(cbuf, (0xF1));
  }
  {
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);

    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [RSP]

    // UseXmmLoadAndClearUpper ? movsd dst,[rsp] : movlpd dst,[rsp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12 );
    encode_RegMem(cbuf, dstenc, RSP_enc, 0x4, 0, 0, false);

    // add rsp,8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf,0x83);
    emit_rm(cbuf,0x3, 0x0, RSP_enc);
    emit_d8(cbuf,0x08);
  }
}
void  roundFloat_nopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
void  roundDouble_nopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
void  convF2D_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5A /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  convF2D_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5A /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  convD2F_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5A /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  convD2F_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x5A /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  convF2I_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2C /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // cmpl $dst, #0x80000000
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x81);
    emit_rm(cbuf, 0x3, 0x7, dstenc & 7);
    emit_d32(cbuf, 0x80000000);

    // jne,s done
    emit_opcode(cbuf, 0x75);
    if (srcenc < 8 && dstenc < 8) {
      emit_d8(cbuf, 0xF);
    } else if (srcenc >= 8 && dstenc >= 8) {
      emit_d8(cbuf, 0x11);
    } else {
      emit_d8(cbuf, 0x10);
    }

    // subq rsp, #8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 8);

    // movss [rsp], $src
    emit_opcode(cbuf, 0xF3);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false); // 2 bytes

    // call f2i_fixup
    cbuf.set_inst_mark();
    emit_opcode(cbuf, 0xE8);
    emit_d32_reloc(cbuf,
                   (int) 
                   (StubRoutines::amd64::f2i_fixup() - cbuf.code_end() - 4),
                   runtime_call_Relocation::spec(),
                   RELOC_DISP32);

    // popq $dst
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x58 | (dstenc & 7));

    // done:
  }
}
void  convF2L_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2C /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);
    address const_address = (address) StubRoutines::amd64::double_sign_flip();

    // cmpq $dst, [0x8000000000000000]
    cbuf.set_inst_mark();
    emit_opcode(cbuf, dstenc < 8 ? Assembler::REX_W : Assembler::REX_WR);
    emit_opcode(cbuf, 0x39);
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, dstenc & 7, 0x5); // 00 reg 101
    emit_d32_reloc(cbuf, const_address);


    // jne,s done
    emit_opcode(cbuf, 0x75);
    if (srcenc < 8 && dstenc < 8) {
      emit_d8(cbuf, 0xF);
    } else if (srcenc >= 8 && dstenc >= 8) {
      emit_d8(cbuf, 0x11);
    } else {
      emit_d8(cbuf, 0x10);
    }

    // subq rsp, #8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 8);

    // movss [rsp], $src
    emit_opcode(cbuf, 0xF3);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false); // 2 bytes

    // call f2l_fixup
    cbuf.set_inst_mark();
    emit_opcode(cbuf, 0xE8);
    emit_d32_reloc(cbuf,
                   (int) 
                   (StubRoutines::amd64::f2l_fixup() - cbuf.code_end() - 4),
                   runtime_call_Relocation::spec(),
                   RELOC_DISP32);

    // popq $dst
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x58 | (dstenc & 7));

    // done:
  }
}
void  convD2I_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2C /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // cmpl $dst, #0x80000000
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x81);
    emit_rm(cbuf, 0x3, 0x7, dstenc & 7);
    emit_d32(cbuf, 0x80000000);

    // jne,s done
    emit_opcode(cbuf, 0x75);
    if (srcenc < 8 && dstenc < 8) {
      emit_d8(cbuf, 0xF);
    } else if (srcenc >= 8 && dstenc >= 8) {
      emit_d8(cbuf, 0x11);
    } else {
      emit_d8(cbuf, 0x10);
    }

    // subq rsp, #8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 8);

    // movsd [rsp], $src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false); // 2 bytes

    // call d2i_fixup
    cbuf.set_inst_mark();
    emit_opcode(cbuf, 0xE8);
    emit_d32_reloc(cbuf,
                   (int) 
                   (StubRoutines::amd64::d2i_fixup() - cbuf.code_end() - 4),
                   runtime_call_Relocation::spec(),
                   RELOC_DISP32);

    // popq $dst
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x58 | (dstenc & 7));

    // done:
  }
}
void  convD2L_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2C /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
  {
    int dstenc = opnd_array(0)->reg(ra_,this);
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);
    address const_address = (address) StubRoutines::amd64::double_sign_flip();

    // cmpq $dst, [0x8000000000000000]
    cbuf.set_inst_mark();
    emit_opcode(cbuf, dstenc < 8 ? Assembler::REX_W : Assembler::REX_WR);
    emit_opcode(cbuf, 0x39);
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, dstenc & 7, 0x5); // 00 reg 101
    emit_d32_reloc(cbuf, const_address);


    // jne,s done
    emit_opcode(cbuf, 0x75);
    if (srcenc < 8 && dstenc < 8) {
      emit_d8(cbuf, 0xF);
    } else if (srcenc >= 8 && dstenc >= 8) {
      emit_d8(cbuf, 0x11);
    } else {
      emit_d8(cbuf, 0x10);
    }

    // subq rsp, #8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 8);

    // movsd [rsp], $src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false); // 2 bytes

    // call d2l_fixup
    cbuf.set_inst_mark();
    emit_opcode(cbuf, 0xE8);
    emit_d32_reloc(cbuf,
                   (int) 
                   (StubRoutines::amd64::d2l_fixup() - cbuf.code_end() - 4),
                   runtime_call_Relocation::spec(),
                   RELOC_DISP32);

    // popq $dst
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x58 | (dstenc & 7));

    // done:
  }
}
void  convI2F_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  convI2F_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  convI2D_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  convI2D_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  convL2F_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  convL2F_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  convL2D_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  convL2D_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  convI2L_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x63 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  convI2L_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x63 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  convI2L_reg_reg_zexNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    encode_copy(cbuf, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  convI2L_reg_mem_zexNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  zerox_long_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);
    int dstenc = opnd_array(0)->reg(ra_,this);

    if (dstenc < 8) {
      if (srcenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
        srcenc -= 8;
      }
    } else {
      if (srcenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
        srcenc -= 8;
      }
      dstenc -= 8;
    }
    
    emit_opcode(cbuf, 0x8B);
    emit_rm(cbuf, 0x3, dstenc, srcenc);
  }
}
void  convL2I_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);
    int dstenc = opnd_array(0)->reg(ra_,this);

    if (dstenc < 8) {
      if (srcenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
        srcenc -= 8;
      }
    } else {
      if (srcenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
        srcenc -= 8;
      }
      dstenc -= 8;
    }
    
    emit_opcode(cbuf, 0x8B);
    emit_rm(cbuf, 0x3, dstenc, srcenc);
  }
}
void  MoveF2I_stack_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  MoveI2F_stack_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x10 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  MoveD2L_stack_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x8B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  MoveL2D_stack_reg_partialNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0x66 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x12 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  MoveL2D_stack_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x10 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  MoveF2I_reg_stackNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF3 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x11 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  MoveI2F_reg_stackNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x89 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  MoveD2L_reg_stackNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xF2 /*primary()*/));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf, (0x11 /*tertiary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  MoveL2D_reg_stackNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x89 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  MoveF2I_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);
__ movdl(opnd_array(0)->as_Register(ra_,this), opnd_array(1)->as_XMMRegister(ra_,this,idx1)); }
}
void  MoveD2L_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);
__ movdq(opnd_array(0)->as_Register(ra_,this), opnd_array(1)->as_XMMRegister(ra_,this,idx1)); }
}
void  MoveI2F_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);
__ movdl(opnd_array(0)->as_XMMRegister(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1)); }
}
void  MoveL2D_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);
__ movdq(opnd_array(0)->as_XMMRegister(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1)); }
}
void  Repl8B_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    encode_CopyXD(cbuf, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
    __ punpcklbw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), 0x00);
  }
}
void  Repl8B_rRegINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ movdl(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_Register(opnd_array(1)->reg(ra_,this,idx1)));
  }
  {
    MacroAssembler _masm(&cbuf);

    encode_CopyXD(cbuf, opnd_array(0)->reg(ra_,this), opnd_array(0)->reg(ra_,this));
    __ punpcklbw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), 0x00);
  }
}
void  Repl8B_immI0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  }
}
void  Repl4S_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), 0x00);
  }
}
void  Repl4S_rRegINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ movdl(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_Register(opnd_array(1)->reg(ra_,this,idx1)));
  }
  {
    MacroAssembler _masm(&cbuf);

    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), 0x00);
  }
}
void  Repl4S_immI0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  }
}
void  Repl4C_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), 0x00);
  }
}
void  Repl4C_rRegINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ movdl(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_Register(opnd_array(1)->reg(ra_,this,idx1)));
  }
  {
    MacroAssembler _masm(&cbuf);

    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), 0x00);
  }
}
void  Repl4C_immI0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  }
}
void  Repl2I_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pshufd(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), (0x00));
  }
}
void  Repl2I_rRegINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ movdl(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_Register(opnd_array(1)->reg(ra_,this,idx1)));
  }
  {
    MacroAssembler _masm(&cbuf);

    __ pshufd(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), (0x00));
  }
}
void  Repl2I_immI0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  }
}
void  Repl2F_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pshufd(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), (0xe0));
  }
}
void  Repl2F_regFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pshufd(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), (0xe0));
  }
}
void  Repl2F_immF0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  }
}
void  rep_stosNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0x33));
    emit_rm(cbuf, 0x3, (RAX_enc) & 7, (RAX_enc) & 7);
  }
  {
    emit_opcode(cbuf, (0xF3));
  }
  {
    emit_opcode(cbuf, (0x48));
  }
  {
    emit_opcode(cbuf, (0xAB));
  }
}
void  string_compareNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    Label RCX_GOOD_LABEL, LENGTH_DIFF_LABEL, 
	  POP_LABEL, DONE_LABEL, CONT_LABEL, 
	  WHILE_HEAD_LABEL;
    MacroAssembler masm(&cbuf);

    // Get the first character position in both strings
    //         [8] char array, [12] offset, [16] count
    int value_offset  = java_lang_String::value_offset_in_bytes();
    int offset_offset = java_lang_String::offset_offset_in_bytes();
    int count_offset  = java_lang_String::count_offset_in_bytes();
    int base_offset   = arrayOopDesc::base_offset_in_bytes(T_CHAR);

    masm.movq(rax, Address(rsi, value_offset));
    masm.movl(rcx, Address(rsi, offset_offset));
    masm.leaq(rax, Address(rax, rcx, Address::times_2, base_offset));
    masm.movq(rbx, Address(rdi, value_offset));
    masm.movl(rcx, Address(rdi, offset_offset));
    masm.leaq(rbx, Address(rbx, rcx, Address::times_2, base_offset));

    // Compute the minimum of the string lengths(rsi) and the
    // difference of the string lengths (stack)

    masm.movl(rdi, Address(rdi, count_offset));
    masm.movl(rsi, Address(rsi, count_offset));
    masm.movl(rcx, rdi);
    masm.subl(rdi, rsi);
    masm.pushq(rdi);
    masm.cmovl(Assembler::lessEqual, rsi, rcx);
    		
    // Is the minimum length zero?
    masm.bind(RCX_GOOD_LABEL);
    masm.testl(rsi, rsi);
    masm.jcc(Assembler::zero, LENGTH_DIFF_LABEL);
    
    // Load first characters
    masm.load_unsigned_word(rcx, Address(rbx, 0));
    masm.load_unsigned_word(rdi, Address(rax, 0));
    
    // Compare first characters
    masm.subl(rcx, rdi);
    masm.jcc(Assembler::notZero,  POP_LABEL);
    masm.decrementl(rsi);
    masm.jcc(Assembler::zero, LENGTH_DIFF_LABEL);

    {
      // Check after comparing first character to see if strings are equivalent
      Label LSkip2;
      // Check if the strings start at same location
      masm.cmpq(rbx, rax);
      masm.jcc(Assembler::notEqual, LSkip2);
    
      // Check if the length difference is zero (from stack)
      masm.cmpl(Address(rsp, 0), 0x0);
      masm.jcc(Assembler::equal,  LENGTH_DIFF_LABEL);

      // Strings might not be equivalent
      masm.bind(LSkip2);
    }

    // Shift RAX and RBX to the end of the arrays, negate min
    masm.leaq(rax, Address(rax, rsi, Address::times_2, 2));
    masm.leaq(rbx, Address(rbx, rsi, Address::times_2, 2));
    masm.negq(rsi);

    // Compare the rest of the characters
    masm.bind(WHILE_HEAD_LABEL);
    masm.load_unsigned_word(rcx, Address(rbx, rsi, Address::times_2, 0));
    masm.load_unsigned_word(rdi, Address(rax, rsi, Address::times_2, 0));
    masm.subl(rcx, rdi);
    masm.jcc(Assembler::notZero, POP_LABEL);
    masm.incrementq(rsi);
    masm.jcc(Assembler::notZero, WHILE_HEAD_LABEL);
    
    // Strings are equal up to min length.  Return the length difference.
    masm.bind(LENGTH_DIFF_LABEL);
    masm.popq(rcx);
    masm.jmp(DONE_LABEL);

    // Discard the stored length difference
    masm.bind(POP_LABEL);
    masm.addq(rsp, 8);
       
    // That's it
    masm.bind(DONE_LABEL);
  }
}
void  compI_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x3B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  compI_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  compI_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x3B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  testI_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x85 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  testI_reg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
  {
    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  }
}
void  testI_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x85 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  testI_reg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x85 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  compU_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x3B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  compU_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  compU_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x3B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  testU_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x85 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  compP_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x3B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  compP_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x3B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  compP_mem_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x3B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  testP_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x85 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  testP_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  }
  {
    emit_d32(cbuf,(0xFFFFFFFF));
  }
}
void  compL_rRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x3B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  compL_rReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), dstenc);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  }
}
void  compL_rReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x3B /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  testL_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x85 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  testL_reg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  }
  {
    emit_opcode(cbuf, (0xF7 /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
  {
    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constantL());
  }
}
void  testL_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x85 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  testL_reg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  }
  {
    emit_opcode(cbuf, (0x85 /*primary()*/));
  }
  {
    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  }
}
void  cmpL3_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    int src1enc = opnd_array(1)->reg(ra_,this,idx1);
    int src2enc = opnd_array(2)->reg(ra_,this,idx2);
    int dstenc = opnd_array(0)->reg(ra_,this);
    
    // cmpq $src1, $src2
    if (src1enc < 8) {
      if (src2enc < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (src2enc < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
    emit_opcode(cbuf, 0x3B);
    emit_rm(cbuf, 0x3, src1enc & 7, src2enc & 7);

    // movl $dst, -1
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0xB8 | (dstenc & 7));
    emit_d32(cbuf, -1);

    // jl,s done
    emit_opcode(cbuf, 0x7C);
    emit_d8(cbuf, dstenc < 4 ? 0x06 : 0x08);

    // setne $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_opcode(cbuf, 0xC0 | (dstenc & 7));

    // movzbl $dst, $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_RB);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0xB6);
    emit_rm(cbuf, 0x3, dstenc & 7, dstenc & 7);
  }
}
void  cmovI_reg_gNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x4F /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  cmovI_reg_lNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  }
  {
    emit_opcode(cbuf, (0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf, (0x4C /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  }
}
void  jmpDirNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xE9 /*primary()*/));
  }
  {
    // JMP, CALL
    Label* l = opnd_array(1)->label();
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.code_size() + 4)) : 0);
  }
}
uint  jmpDirNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 5);
}
void  jmpConNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // JCC
    Label* l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.code_size() + 4)) : 0);
  }
}
uint  jmpConNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void  jmpLoopEndNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // JCC
    Label* l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.code_size() + 4)) : 0);
  }
}
uint  jmpLoopEndNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void  jmpLoopEndUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // JCC
    Label* l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.code_size() + 4)) : 0);
  }
}
uint  jmpLoopEndUNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void  jmpConUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // JCC
    Label* l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.code_size() + 4)) : 0);
  }
}
uint  jmpConUNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void  partialSubtypeCheckNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    Register Rrdi = as_Register(RDI_enc); // result register
    Register Rrax = as_Register(RAX_enc); // super class
    Register Rrcx = as_Register(RCX_enc); // killed
    Register Rrsi = as_Register(RSI_enc); // sub class
    Label hit, miss;

    MacroAssembler _masm(&cbuf);
    // Compare super with sub directly, since super is not in its own SSA.
    // The compiler used to emit this test, but we fold it in here,
    // to allow platform-specific tweaking on sparc.
    __ cmpq(Rrax, Rrsi);
    __ jcc(Assembler::equal, hit);
#ifndef PRODUCT
    __ lea(Rrcx, ExternalAddress((address)&SharedRuntime::_partial_subtype_ctr));
    __ incrementl(Address(Rrcx, 0));
#endif //PRODUCT
    __ movq(Rrdi, Address(Rrsi, 
                          sizeof(oopDesc) + 
                          Klass::secondary_supers_offset_in_bytes()));
    __ movl(Rrcx, Address(Rrdi, arrayOopDesc::length_offset_in_bytes()));
    __ addq(Rrdi, arrayOopDesc::base_offset_in_bytes(T_OBJECT));
    __ repne_scan();
    __ jcc(Assembler::notEqual, miss);
    __ movq(Address(Rrsi, 
                    sizeof(oopDesc) + 
                    Klass::secondary_super_cache_offset_in_bytes()),
            Rrax);
    __ bind(hit);
    if ((0x1 /*primary()*/)) {
      __ xorq(Rrdi, Rrdi);
    }
    __ bind(miss);
  }
}
void  partialSubtypeCheck_vs_ZeroNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Register Rrdi = as_Register(RDI_enc); // result register
    Register Rrax = as_Register(RAX_enc); // super class
    Register Rrcx = as_Register(RCX_enc); // killed
    Register Rrsi = as_Register(RSI_enc); // sub class
    Label hit, miss;

    MacroAssembler _masm(&cbuf);
    // Compare super with sub directly, since super is not in its own SSA.
    // The compiler used to emit this test, but we fold it in here,
    // to allow platform-specific tweaking on sparc.
    __ cmpq(Rrax, Rrsi);
    __ jcc(Assembler::equal, hit);
#ifndef PRODUCT
    __ lea(Rrcx, ExternalAddress((address)&SharedRuntime::_partial_subtype_ctr));
    __ incrementl(Address(Rrcx, 0));
#endif //PRODUCT
    __ movq(Rrdi, Address(Rrsi, 
                          sizeof(oopDesc) + 
                          Klass::secondary_supers_offset_in_bytes()));
    __ movl(Rrcx, Address(Rrdi, arrayOopDesc::length_offset_in_bytes()));
    __ addq(Rrdi, arrayOopDesc::base_offset_in_bytes(T_OBJECT));
    __ repne_scan();
    __ jcc(Assembler::notEqual, miss);
    __ movq(Address(Rrsi, 
                    sizeof(oopDesc) + 
                    Klass::secondary_super_cache_offset_in_bytes()),
            Rrax);
    __ bind(hit);
    if ((0x0 /*primary()*/)) {
      __ xorq(Rrdi, Rrdi);
    }
    __ bind(miss);
  }
}
void  jmpDir_shortNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, (0xEB /*primary()*/));
  }
  {
    // JMP, CALL
    Label* l = opnd_array(1)->label();
    int disp = l ? (l->loc_pos() - (cbuf.code_size() + 1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  }
}
uint  jmpDir_shortNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  jmpCon_shortNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
  // JCC
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.code_size() + 1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  }
}
uint  jmpCon_shortNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  jmpLoopEnd_shortNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
  // JCC
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.code_size() + 1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  }
}
uint  jmpLoopEnd_shortNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  jmpLoopEndU_shortNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
  // JCC
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.code_size() + 1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  }
}
uint  jmpLoopEndU_shortNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  jmpConU_shortNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
  // JCC
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.code_size() + 1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  }
}
uint  jmpConU_shortNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  cmpFastLockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    Register objReg = as_Register((int)opnd_array(1)->reg(ra_,this,idx1));
    Register boxReg = as_Register((int)opnd_array(2)->reg(ra_,this,idx2));
    Register tmpReg = as_Register(opnd_array(3)->reg(ra_,this,idx3));
    Register scrReg = as_Register(opnd_array(4)->reg(ra_,this,idx4));
    MacroAssembler masm(&cbuf);

    // Verify uniqueness of register assignments -- necessary but not sufficient
    assert (objReg != boxReg && objReg != tmpReg && 
            objReg != scrReg && tmpReg != scrReg, "invariant") ; 

    if (_counters != NULL) {
      masm.atomic_incl(ExternalAddress((address) _counters->total_entry_count_addr()));
    }
    if (EmitSync & 1) { 
        masm.movptr (Address(boxReg, 0), intptr_t(markOopDesc::unused_mark())) ; 
        masm.cmpq   (rsp, 0) ; 
    } else
    if (EmitSync & 2) { 
        Label DONE_LABEL;
        if (UseBiasedLocking) {
           // Note: tmpReg maps to the swap_reg argument and scrReg to the tmp_reg argument.
          masm.biased_locking_enter(boxReg, objReg, tmpReg, scrReg, false, DONE_LABEL, NULL, _counters);
        }
        masm.movl(tmpReg, 0x1);
        masm.orq(tmpReg, Address(objReg, 0));
        masm.movq(Address(boxReg, 0), tmpReg);
        if (os::is_MP()) {
          masm.lock();
        }
        masm.cmpxchgq(boxReg, Address(objReg, 0)); // Updates tmpReg
        masm.jcc(Assembler::equal, DONE_LABEL);
    
        // Recursive locking
        masm.subq(tmpReg, rsp);
        masm.andq(tmpReg, 7 - os::vm_page_size());
        masm.movq(Address(boxReg, 0), tmpReg);

        masm.bind(DONE_LABEL); 
        masm.nop(); // avoid branch to branch
    } else { 
        Label DONE_LABEL, IsInflated, Egress; 

        masm.movq  (tmpReg, Address(objReg, 0)) ; 
        masm.testq (tmpReg, 0x02) ;         // inflated vs stack-locked|neutral|biased
        masm.jcc   (Assembler::notZero, IsInflated) ; 
         
        // it's stack-locked, biased or neutral
        // TODO: optimize markword triage order to reduce the number of 
        // conditional branches in the most common cases. 
        // Beware -- there's a subtle invariant that fetch of the markword
        // at [FETCH], below, will never observe a biased encoding (*101b).  
        // If this invariant is not held we'll suffer exclusion (safety) failure.
          
        if (UseBiasedLocking) {
          masm.biased_locking_enter(boxReg, objReg, tmpReg, scrReg, true, DONE_LABEL, NULL, _counters);
          masm.movq  (tmpReg, Address(objReg, 0)) ;        // [FETCH]
        }

        masm.orq   (tmpReg, 1) ; 
        masm.movq  (Address(boxReg, 0), tmpReg) ;  
        if (os::is_MP()) { masm.lock(); } 
        masm.cmpxchgq(boxReg, Address(objReg, 0)); // Updates tmpReg
        if (_counters != NULL) {
           masm.cond_inc32(Assembler::equal,
                           ExternalAddress((address) _counters->fast_path_entry_count_addr()));
        }
        masm.jcc   (Assembler::equal, DONE_LABEL);
    
        // Recursive locking
        masm.subq  (tmpReg, rsp);
        masm.andq  (tmpReg, 7 - os::vm_page_size());
        masm.movq  (Address(boxReg, 0), tmpReg);
        if (_counters != NULL) {
           masm.cond_inc32(Assembler::equal,
                           ExternalAddress((address) _counters->fast_path_entry_count_addr()));
        }
        masm.jmp   (DONE_LABEL) ; 

        masm.bind  (IsInflated) ; 
        // It's inflated
        
        // TODO: someday avoid the ST-before-CAS penalty by
        // relocating (deferring) the following ST.  
        // We should also think about trying a CAS without having
        // fetched _owner.  If the CAS is successful we may
        // avoid an RTO->RTS upgrade on the $line.  
        masm.movptr(Address(boxReg, 0), intptr_t(markOopDesc::unused_mark())) ; 

        masm.movq  (boxReg, tmpReg) ; 
        masm.movq  (tmpReg, Address(tmpReg, ObjectMonitor::owner_offset_in_bytes()-2)) ; 
        masm.testq (tmpReg, tmpReg) ;   
        masm.jcc   (Assembler::notZero, DONE_LABEL) ; 

        // It's inflated and appears unlocked
        if (os::is_MP()) { masm.lock(); } 
        masm.cmpxchgq(r15_thread, Address(boxReg, ObjectMonitor::owner_offset_in_bytes()-2)) ; 
        // Intentional fall-through into DONE_LABEL ...
        
        masm.bind  (DONE_LABEL) ; 
        masm.nop   () ;                 // avoid jmp to jmp
    }
  }
}
void  cmpFastUnlockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
   
    Register objReg = as_Register(opnd_array(1)->reg(ra_,this,idx1));
    Register boxReg = as_Register(opnd_array(2)->reg(ra_,this,idx2));
    Register tmpReg = as_Register(opnd_array(3)->reg(ra_,this,idx3));
    MacroAssembler masm(&cbuf);

    if (EmitSync & 4) { 
       masm.cmpq  (rsp, 0) ; 
    } else
    if (EmitSync & 8) { 
       Label DONE_LABEL;
       if (UseBiasedLocking) {
         masm.biased_locking_exit(objReg, tmpReg, DONE_LABEL);
       }

       // Check whether the displaced header is 0 
       //(=> recursive unlock)
       masm.movq(tmpReg, Address(boxReg, 0));
       masm.testq(tmpReg, tmpReg);
       masm.jcc(Assembler::zero, DONE_LABEL);

       // If not recursive lock, reset the header to displaced header
       if (os::is_MP()) {
         masm.lock();
       }
       masm.cmpxchgq(tmpReg, Address(objReg, 0)); // Uses RAX which is box
       masm.bind(DONE_LABEL);
       masm.nop(); // avoid branch to branch
    } else { 
       Label DONE_LABEL, Stacked, CheckSucc ; 

       if (UseBiasedLocking) {
         masm.biased_locking_exit(objReg, tmpReg, DONE_LABEL);
       }
        
       masm.movq  (tmpReg, Address(objReg, 0)) ; 
       masm.cmpq  (Address(boxReg, 0), (int)NULL_WORD) ; 
       masm.jcc   (Assembler::zero, DONE_LABEL) ; 
       masm.testq (tmpReg, 0x02) ; 
       masm.jcc   (Assembler::zero, Stacked) ; 
        
       // It's inflated
       masm.movq  (boxReg, Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2)) ; 
       masm.xorq  (boxReg, r15_thread) ; 
       masm.orq   (boxReg, Address (tmpReg, ObjectMonitor::recursions_offset_in_bytes()-2)) ; 
       masm.jcc   (Assembler::notZero, DONE_LABEL) ; 
       masm.movq  (boxReg, Address (tmpReg, ObjectMonitor::cxq_offset_in_bytes()-2)) ; 
       masm.orq   (boxReg, Address (tmpReg, ObjectMonitor::EntryList_offset_in_bytes()-2)) ; 
       masm.jcc   (Assembler::notZero, CheckSucc) ; 
       masm.mov64 (Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2), (int)NULL_WORD) ; 
       masm.jmp   (DONE_LABEL) ; 
        
       if ((EmitSync & 65536) == 0) { 
         Label LSuccess, LGoSlowPath ;
         masm.bind  (CheckSucc) ;
         masm.cmpq  (Address (tmpReg, ObjectMonitor::succ_offset_in_bytes()-2), (int)NULL_WORD) ;
         masm.jcc   (Assembler::zero, LGoSlowPath) ;

         // I'd much rather use lock:andl m->_owner, 0 as it's faster than the
         // the explicit ST;MEMBAR combination, but masm doesn't currently support
         // "ANDQ M,IMM".  Don't use MFENCE here.  lock:add to TOS, xchg, etc
         // are all faster when the write buffer is populated.
         masm.movptr (Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2), (int)NULL_WORD) ;
         if (os::is_MP()) {
            masm.lock () ; masm.addq (Address(rsp, 0), 0) ;
         }
         masm.cmpq  (Address (tmpReg, ObjectMonitor::succ_offset_in_bytes()-2), (int)NULL_WORD) ;
         masm.jcc   (Assembler::notZero, LSuccess) ;

         masm.movptr (boxReg, (int)NULL_WORD) ;                   // box is really EAX
         if (os::is_MP()) { masm.lock(); }
         masm.cmpxchgq (r15_thread, Address(tmpReg, ObjectMonitor::owner_offset_in_bytes()-2));
         masm.jcc   (Assembler::notEqual, LSuccess) ;
         // Intentional fall-through into slow-path

         masm.bind  (LGoSlowPath) ;
         masm.orl   (boxReg, 1) ;                      // set ICC.ZF=0 to indicate failure
         masm.jmp   (DONE_LABEL) ;

         masm.bind  (LSuccess) ;
         masm.testl (boxReg, 0) ;                      // set ICC.ZF=1 to indicate success
         masm.jmp   (DONE_LABEL) ;
       }

       masm.bind  (Stacked) ; 
       masm.movq  (tmpReg, Address (boxReg, 0)) ;      // re-fetch
       if (os::is_MP()) { masm.lock(); } 
       masm.cmpxchgq(tmpReg, Address(objReg, 0)); // Uses RAX which is box

       if (EmitSync & 65536) { 
          masm.bind (CheckSucc) ; 
       }
       masm.bind(DONE_LABEL);
       if (EmitSync & 32768) { 
          masm.nop();                      // avoid branch to branch
       }
    }
  }
}
void  safePoint_pollNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {
    // testl %rax, off(%rip) // Opcode + ModRM + Disp32 == 6 bytes
    // XXX reg_mem doesn't support RIP-relative addressing yet
    cbuf.set_inst_mark();
    cbuf.relocate(cbuf.inst_mark(), relocInfo::poll_type, 0); // XXX
    emit_opcode(cbuf, 0x85); // testl
    emit_rm(cbuf, 0x0, RAX_enc, 0x5); // 00 rax 101 == 0x5
    // cbuf.inst_mark() is beginning of instruction
    emit_d32_reloc(cbuf, os::get_polling_page());
//                    relocInfo::poll_type, 
  }
}
uint  safePoint_pollNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void  CallStaticJavaDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // JAVA STATIC CALL
    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to
    // determine who we intended to call.
    cbuf.set_inst_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));

    if (!_method) {      
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.code_end()) - 4),
                     runtime_call_Relocation::spec(),
                     RELOC_DISP32);
    } else if (_optimized_virtual) {
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.code_end()) - 4),
                     opt_virtual_call_Relocation::spec(), 
                     RELOC_DISP32);
    } else {      
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.code_end()) - 4),
                     static_call_Relocation::spec(),
                     RELOC_DISP32);
    }
    if (_method) {
      // Emit stub for static call
      emit_java_to_interp(cbuf);      
    }
  }
  {
    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize = 
        ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      if (framesize) {
        if (framesize < 0x80) {
          emit_opcode(cbuf, Assembler::REX_W);
          emit_opcode(cbuf, 0x81); // cmpq [rsp+0],0xbadb1ood
          emit_d8(cbuf, 0x7C);
          emit_d8(cbuf, 0x24);
          emit_d8(cbuf, framesize); // Find majik cookie from ESP
          emit_d32(cbuf, 0xbadb100d);
        } else {
          emit_opcode(cbuf, Assembler::REX_W);
          emit_opcode(cbuf, 0x81); // cmpq [rsp+0],0xbadb1ood
          emit_d8(cbuf, 0xBC);
          emit_d8(cbuf, 0x24);
          emit_d32(cbuf, framesize); // Find majik cookie from ESP
          emit_d32(cbuf, 0xbadb100d);
        }
      }
      // jmp EQ around INT3
      // QQQ TODO
      const int jump_around = 5; // size of call to breakpoint, 1 for CC
      emit_opcode(cbuf, 0x74);
      emit_d8(cbuf, jump_around);
      // QQQ temporary
      emit_break(cbuf);
      // Die if stack mismatch
      // emit_opcode(cbuf,0xCC);
    }
  }
}
void  CallDynamicJavaDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // JAVA DYNAMIC CALL
    // !!!!!
    // Generate  "movq rax, -1", placeholder instruction to load oop-info
    // emit_call_dynamic_prologue( cbuf );
    cbuf.set_inst_mark();

    // movq rax, -1
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0xB8 | RAX_enc);
    emit_d64_reloc(cbuf, 
                   (int64_t) Universe::non_oop_word(),
                   oop_Relocation::spec_for_immediate(), RELOC_IMM64);
    address virtual_call_oop_addr = cbuf.inst_mark();
    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to determine
    // who we intended to call.
    cbuf.set_inst_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    emit_d32_reloc(cbuf, 
                   (int) (opnd_array(1)->method() - ((intptr_t) cbuf.code_end()) - 4),
                   virtual_call_Relocation::spec(virtual_call_oop_addr),
                   RELOC_DISP32);
  }
  {
    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize = 
        ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      if (framesize) {
        if (framesize < 0x80) {
          emit_opcode(cbuf, Assembler::REX_W);
          emit_opcode(cbuf, 0x81); // cmpq [rsp+0],0xbadb1ood
          emit_d8(cbuf, 0x7C);
          emit_d8(cbuf, 0x24);
          emit_d8(cbuf, framesize); // Find majik cookie from ESP
          emit_d32(cbuf, 0xbadb100d);
        } else {
          emit_opcode(cbuf, Assembler::REX_W);
          emit_opcode(cbuf, 0x81); // cmpq [rsp+0],0xbadb1ood
          emit_d8(cbuf, 0xBC);
          emit_d8(cbuf, 0x24);
          emit_d32(cbuf, framesize); // Find majik cookie from ESP
          emit_d32(cbuf, 0xbadb100d);
        }
      }
      // jmp EQ around INT3
      // QQQ TODO
      const int jump_around = 5; // size of call to breakpoint, 1 for CC
      emit_opcode(cbuf, 0x74);
      emit_d8(cbuf, jump_around);
      // QQQ temporary
      emit_break(cbuf);
      // Die if stack mismatch
      // emit_opcode(cbuf,0xCC);
    }
  }
}
void  CallRuntimeDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // No relocation needed

    // movq r10, <meth>
    emit_opcode(cbuf, Assembler::REX_WB);
    emit_opcode(cbuf, 0xB8 | (R10_enc - 8));
    emit_d64(cbuf, (int64_t) opnd_array(1)->method());

    // call (r10)
    emit_opcode(cbuf, Assembler::REX_B);
    emit_opcode(cbuf, 0xFF);
    emit_opcode(cbuf, 0xD0 | (R10_enc - 8));
  }
}
void  CallLeafDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // No relocation needed

    // movq r10, <meth>
    emit_opcode(cbuf, Assembler::REX_WB);
    emit_opcode(cbuf, 0xB8 | (R10_enc - 8));
    emit_d64(cbuf, (int64_t) opnd_array(1)->method());

    // call (r10)
    emit_opcode(cbuf, Assembler::REX_B);
    emit_opcode(cbuf, 0xFF);
    emit_opcode(cbuf, 0xD0 | (R10_enc - 8));
  }
}
void  CallLeafNoFPDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // No relocation needed

    // movq r10, <meth>
    emit_opcode(cbuf, Assembler::REX_WB);
    emit_opcode(cbuf, 0xB8 | (R10_enc - 8));
    emit_d64(cbuf, (int64_t) opnd_array(1)->method());

    // call (r10)
    emit_opcode(cbuf, Assembler::REX_B);
    emit_opcode(cbuf, 0xFF);
    emit_opcode(cbuf, 0xD0 | (R10_enc - 8));
  }
}
void  RetNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {
    emit_opcode(cbuf, (0xC3 /*primary()*/));
  }
}
void  TailCalljmpIndNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xFF /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  tailjmpIndNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0x5a));
  }
  {
    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  }
  {
    emit_opcode(cbuf, (0xFF /*primary()*/));
  }
  {
    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  }
}
void  CreateExceptionNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  CreateExceptionNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  RethrowExceptionNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {
    cbuf.set_inst_mark();
    emit_opcode(cbuf, 0xE9); // jmp entry
    emit_d32_reloc(cbuf, 
                   (int) (OptoRuntime::rethrow_stub() - cbuf.code_end() - 4), 
                   runtime_call_Relocation::spec(),
                   RELOC_DISP32);
  }
}
void  tlsLoadPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  tlsLoadPNode::size(PhaseRegAlloc *ra_) const {
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  ShouldNotReachHereNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {
    MacroAssembler* masm = new MacroAssembler(&cbuf);    
    masm->call(RuntimeAddress(CAST_FROM_FN_PTR(address, os::breakpoint)));
  }
}
const MachOper* loadBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadCNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadRangeNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadD_partialNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA8BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA4SNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA4CNode::memory_operand() const { return _opnds[1]; }
const MachOper* load2IUNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA2FNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrNTANode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrT0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrT2Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwNTANode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwT0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwT2Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeCNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmPNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmI16Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA8BNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA4CNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA2INode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmCM0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeA2FNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeF_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeD0_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI_reversedNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadL_reversedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeI_reversedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeL_reversedNode::memory_operand() const { return _opnds[1]; }
const TypePtr *membar_acquireNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_acquireNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_acquire_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_acquire_lockNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_releaseNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_releaseNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_release_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_release_lockNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* cmovI_memNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovI_memUNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovL_memNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovL_memUNode::memory_operand() const { return _opnds[4]; }
const MachOper* addI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* incI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* decI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* addL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addL_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* addL_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* incL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* decL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLConditional_flagsNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapPNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapLNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapINode::memory_operand() const { return _opnds[1]; }
const MachOper* subI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* subI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* subL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* subL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* negI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* negL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulI_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulL_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulL_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* salI_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* salI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* salI_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* shrI_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* shrI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* shrI_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* salL_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* salL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* salL_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* sarL_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* sarL_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* shrL_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* shrL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* shrL_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* orI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* andL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andL_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* andL_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* orL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orL_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* orL_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorL_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorL_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* cmpF_cc_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_cc_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* subF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* divF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* sqrtF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* sqrtD_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convF2D_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convD2F_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2F_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2D_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convL2F_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convL2D_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2L_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2L_reg_mem_zexNode::memory_operand() const { return _opnds[1]; }
const MachOper* compI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testI_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testI_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* compU_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_mem_rRegNode::memory_operand() const { return _opnds[2]; }
const MachOper* testP_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* compL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testL_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testL_reg_mem_0Node::memory_operand() const { return _opnds[1]; }


const bool Matcher::has_match_rule(int opcode) {
  assert(_last_machine_leaf < opcode && opcode < _last_opcode, "opcode in range");
  return _hasMatchRule[opcode];
}

const bool Matcher::_hasMatchRule[_last_opcode] = {
    false,  // Node
    false,  // Set
    false,  // RegI
    false,  // RegP
    false,  // RegF
    false,  // RegD
    false,  // RegL
    false,  // RegFlags
    false,  // _last_machine_leaf
    true ,  // AbsD
    true ,  // AbsF
    false,  // AbsI
    true ,  // AddD
    true ,  // AddF
    true ,  // AddI
    true ,  // AddL
    true ,  // AddP
    false,  // Allocate
    false,  // AllocateArray
    true ,  // AndI
    true ,  // AndL
    false,  // AtanD
    true ,  // Binary
    true ,  // Bool
    false,  // BoxLock
    true ,  // ReverseBytesI
    true ,  // ReverseBytesL
    false,  // CProj
    true ,  // CallDynamicJava
    false,  // CallJava
    true ,  // CallLeaf
    true ,  // CallLeafNoFP
    true ,  // CallRuntime
    true ,  // CallStaticJava
    true ,  // CastII
    true ,  // CastX2P
    true ,  // CastP2X
    true ,  // CastPP
    false,  // Catch
    false,  // CatchProj
    true ,  // CheckCastPP
    true ,  // ClearArray
    false,  // ConstraintCast
    true ,  // CMoveD
    true ,  // CMoveF
    true ,  // CMoveI
    true ,  // CMoveL
    true ,  // CMoveP
    true ,  // CmpD
    true ,  // CmpD3
    true ,  // CmpF
    true ,  // CmpF3
    true ,  // CmpI
    true ,  // CmpL
    true ,  // CmpL3
    true ,  // CmpLTMask
    true ,  // CmpP
    true ,  // CmpU
    true ,  // CompareAndSwapI
    true ,  // CompareAndSwapL
    true ,  // CompareAndSwapP
    false,  // Con
    true ,  // ConD
    true ,  // ConF
    true ,  // ConI
    true ,  // ConL
    true ,  // ConP
    true ,  // Conv2B
    true ,  // ConvD2F
    true ,  // ConvD2I
    true ,  // ConvD2L
    true ,  // ConvF2D
    true ,  // ConvF2I
    true ,  // ConvF2L
    true ,  // ConvI2D
    true ,  // ConvI2F
    true ,  // ConvI2L
    true ,  // ConvL2D
    true ,  // ConvL2F
    true ,  // ConvL2I
    true ,  // CosD
    false,  // CountedLoop
    true ,  // CountedLoopEnd
    true ,  // CreateEx
    true ,  // DivD
    true ,  // DivF
    true ,  // DivI
    true ,  // DivL
    false,  // DivMod
    true ,  // DivModI
    true ,  // DivModL
    false,  // ExpD
    true ,  // FastLock
    true ,  // FastUnlock
    true ,  // Goto
    true ,  // Halt
    true ,  // If
    false,  // IfFalse
    false,  // IfTrue
    false,  // Initialize
    false,  // JProj
    true ,  // Jump
    false,  // JumpProj
    true ,  // LShiftI
    true ,  // LShiftL
    true ,  // LoadB
    true ,  // LoadC
    true ,  // LoadD
    false,  // LoadD_unaligned
    true ,  // LoadF
    true ,  // LoadI
    true ,  // LoadKlass
    true ,  // LoadL
    false,  // LoadL_unaligned
    true ,  // LoadPLocked
    true ,  // LoadLLocked
    true ,  // LoadP
    true ,  // LoadRange
    true ,  // LoadS
    false,  // Lock
    true ,  // LogD
    true ,  // Log10D
    false,  // Loop
    false,  // Mach
    false,  // MachProj
    true ,  // MaxI
    true ,  // MemBarAcquire
    false,  // MemBarCPUOrder
    true ,  // MemBarRelease
    true ,  // MemBarVolatile
    false,  // MergeMem
    true ,  // MinI
    false,  // ModD
    false,  // ModF
    true ,  // ModI
    true ,  // ModL
    true ,  // MoveI2F
    true ,  // MoveF2I
    true ,  // MoveL2D
    true ,  // MoveD2L
    true ,  // MulD
    true ,  // MulF
    true ,  // MulI
    true ,  // MulL
    false,  // Multi
    true ,  // NegD
    true ,  // NegF
    false,  // NeverBranch
    false,  // Opaque1
    false,  // Opaque2
    true ,  // OrI
    true ,  // OrL
    false,  // PCTable
    false,  // Parm
    true ,  // PartialSubtypeCheck
    false,  // Phi
    false,  // PowD
    true ,  // PrefetchRead
    true ,  // PrefetchWrite
    false,  // Proj
    true ,  // RShiftI
    true ,  // RShiftL
    false,  // Region
    true ,  // Rethrow
    true ,  // Return
    false,  // Root
    true ,  // RoundDouble
    true ,  // RoundFloat
    true ,  // SafePoint
    false,  // SCMemProj
    true ,  // SinD
    true ,  // SqrtD
    false,  // Start
    false,  // StartOSR
    true ,  // StoreB
    true ,  // StoreC
    true ,  // StoreCM
    true ,  // StorePConditional
    true ,  // StoreLConditional
    true ,  // StoreD
    true ,  // StoreF
    true ,  // StoreI
    true ,  // StoreL
    true ,  // StoreP
    true ,  // StrComp
    true ,  // SubD
    true ,  // SubF
    true ,  // SubI
    true ,  // SubL
    true ,  // TailCall
    true ,  // TailJump
    true ,  // TanD
    true ,  // ThreadLocal
    false,  // Unlock
    true ,  // URShiftI
    true ,  // URShiftL
    true ,  // XorI
    true ,  // XorL
    false,  // Vector
    false,  // AddVB
    false,  // AddVC
    false,  // AddVS
    false,  // AddVI
    false,  // AddVL
    false,  // AddVF
    false,  // AddVD
    false,  // SubVB
    false,  // SubVC
    false,  // SubVS
    false,  // SubVI
    false,  // SubVL
    false,  // SubVF
    false,  // SubVD
    false,  // MulVF
    false,  // MulVD
    false,  // DivVF
    false,  // DivVD
    false,  // LShiftVB
    false,  // LShiftVC
    false,  // LShiftVS
    false,  // LShiftVI
    false,  // URShiftVB
    false,  // URShiftVC
    false,  // URShiftVS
    false,  // URShiftVI
    false,  // AndV
    false,  // OrV
    false,  // XorV
    false,  // VectorLoad
    false,  // Load16B
    true ,  // Load8B
    false,  // Load4B
    false,  // Load8C
    true ,  // Load4C
    false,  // Load2C
    false,  // Load8S
    true ,  // Load4S
    false,  // Load2S
    false,  // Load4I
    true ,  // Load2I
    false,  // Load2L
    false,  // Load4F
    true ,  // Load2F
    false,  // Load2D
    false,  // VectorStore
    false,  // Store16B
    true ,  // Store8B
    false,  // Store4B
    false,  // Store8C
    true ,  // Store4C
    false,  // Store2C
    false,  // Store4I
    true ,  // Store2I
    false,  // Store2L
    false,  // Store4F
    true ,  // Store2F
    false,  // Store2D
    false,  // Pack
    false,  // PackB
    false,  // PackS
    false,  // PackC
    false,  // PackI
    false,  // PackL
    false,  // PackF
    false,  // PackD
    false,  // Pack2x1B
    false,  // Pack2x2B
    false,  // Replicate16B
    true ,  // Replicate8B
    false,  // Replicate4B
    false,  // Replicate8S
    true ,  // Replicate4S
    false,  // Replicate2S
    false,  // Replicate8C
    true ,  // Replicate4C
    false,  // Replicate2C
    false,  // Replicate4I
    true ,  // Replicate2I
    false,  // Replicate2L
    false,  // Replicate4F
    true ,  // Replicate2F
    false,  // Replicate2D
    false,  // Extract
    false,  // ExtractB
    false,  // ExtractS
    false,  // ExtractC
    false,  // ExtractI
    false,  // ExtractL
    false,  // ExtractF
    false   // ExtractD
};


bool Matcher::stack_direction() const { return false; }

int Compile::sync_stack_slots() const { return 2; }

uint Matcher::stack_alignment_in_bytes() { return StackAlignmentInBytes; }

OptoReg::Name Matcher::return_addr() const { return OptoReg::stack2reg(- 2 +
              round_to(2 + 2 * VerifyStackAtCalls +
                       Compile::current()->fixed_slots(),
                       WordsPerLong * 2)); }

uint Compile::in_preserve_stack_slots() { return 4 + 2 * VerifyStackAtCalls; }

uint Compile::out_preserve_stack_slots() { return SharedRuntime::out_preserve_stack_slots(); }

uint Compile::varargs_C_out_slots_killed() const { return frame::arg_reg_save_area_bytes/BytesPerInt; }

void Matcher::calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length, bool is_outgoing) {

    // No difference between ingoing/outgoing just pass false
    SharedRuntime::java_calling_convention(sig_bt, regs, length, false);
  
}

void Matcher::c_calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length) {

    // This is obviously always outgoing
    (void) SharedRuntime::c_calling_convention(sig_bt, regs, length);
  
}

OptoRegPair Matcher::return_value(int ideal_reg, bool is_outgoing) {

    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = {
      0, 
      0, 
      RAX_num,  // Op_RegI
      RAX_num,  // Op_RegP
      XMM0_num, // Op_RegF
      XMM0_num, // Op_RegD
      RAX_num   // Op_RegL
    };
    static const int hi[Op_RegL + 1] = {
      0,
      0,
      OptoReg::Bad, // Op_RegI
      RAX_H_num,    // Op_RegP
      OptoReg::Bad, // Op_RegF
      XMM0_H_num,   // Op_RegD
      RAX_H_num     // Op_RegL
    };

    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
}

OptoRegPair Matcher::c_return_value(int ideal_reg, bool is_outgoing) {

    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = {
      0, 
      0, 
      RAX_num,  // Op_RegI
      RAX_num,  // Op_RegP
      XMM0_num, // Op_RegF
      XMM0_num, // Op_RegD
      RAX_num   // Op_RegL
    };
    static const int hi[Op_RegL + 1] = {
      0,
      0,
      OptoReg::Bad, // Op_RegI
      RAX_H_num,    // Op_RegP
      OptoReg::Bad, // Op_RegF
      XMM0_H_num,   // Op_RegD
      RAX_H_num     // Op_RegL
    };

    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
}

OptoReg::Name Matcher::inline_cache_reg() { return OptoReg::Name(RAX_num); }

const RegMask &Matcher::inline_cache_reg_mask() { return INLINE_CACHE_REG_mask; }

int Matcher::inline_cache_reg_encode() { return _regEncode[inline_cache_reg()]; }

OptoReg::Name Matcher::interpreter_method_oop_reg() { return OptoReg::Name(RBX_num); }

const RegMask &Matcher::interpreter_method_oop_reg_mask() { return INTERPRETER_METHOD_OOP_REG_mask; }

int Matcher::interpreter_method_oop_reg_encode() { return _regEncode[interpreter_method_oop_reg()]; }

OptoReg::Name Matcher::interpreter_frame_pointer_reg() { return OptoReg::Name(RBP_num); }

const RegMask &Matcher::interpreter_frame_pointer_reg_mask() { return INTERPRETER_FRAME_POINTER_REG_mask; }

OptoReg::Name Matcher::c_frame_pointer() const { return OptoReg::Name(RSP_num); }

// Number of callee-save + always-save registers
int  Matcher::number_of_saved_registers() {
  return 0;
};

// Check consistency of C++ compilation with ADLC options:
// Check adlc -D_LP64=1
#ifndef _LP64
#  error "_LP64 must be defined"
#endif // _LP64
